using LinearAlgebra: det
using StaticArrays: MMatrix

@testset "Test the simplest `supercell` for a `Lattice`" begin
    lattice = Lattice(2, 1, 5, 90, 90, 90)
    a = super(lattice, 3)
    b = Lattice(6, 3, 15, 90, 90, 90)
    @test a == b
    @test reciprocal(a) == reciprocal(b)
end

# Example from http://staff.ustc.edu.cn/~zqj/posts/Transforming-between-supercell-and-primitive-cell/
@testset "Test `supercell` from hexagonal to orthorhombic for a `Lattice`" begin
    lattice = Lattice(2, 2, 3, 90, 90, 120; axis=:c)  # Primitive is hexagonal
    P = [
        2 1 0  # 𝐨₁ = 2𝐡₁ + 𝐡₂
        0 1 0  # 𝐨₂ = 𝐡₂
        0 0 1  # 𝐨₃ = 𝐡₃
    ]'  # Note the transpose!
    a = super(lattice, P)  # Hexagonal to orthorhombic
    b = Lattice([
        2*√3 0 0
        0 2 0
        0 0 3
    ])  # The orthorhombic supercell
    @test a ≈ b
    @test reciprocal(a) ≈ reciprocal(b)
    transform = StandardizedFromPrimitive(P)
    c = transform(lattice)
    @test a ≈ c
    @test reciprocal(a) ≈ reciprocal(c)
    @test inv(transform)(c) ≈ lattice
end

# Example from https://www.researchgate.net/figure/color-online-a-Body-centered-cubic-structures-described-by-two-atom-cubic-supercell_fig1_261404912
@testset "Test a two-atom cubic supercell to a four-atom tetragonal supercell" begin
    lattice = Lattice(2, 2, 2, 90, 90, 90)
    P = [
        1 0 0
        0 1 1
        0 1 -1
    ]'  # Note the transpose!
    @test det(P) == -2
    a = super(lattice, P)  # Four-atom tetragonal supercell
    b = StandardizedFromPrimitive(P)(lattice)  # Two-atom cell to four-atom tetragonal cell
    @test a == b
end

# Example from https://gitlab.com/ase/ase/-/issues/938
@testset "Test ASE issue 938" begin
    lattice = Lattice([[0, 1.805, 1.805], [1.805, 0, 1.805], [1.805, 1.805, 0]])
    P = [
        2 2 -2
        2 -2 2
        -2 2 2
    ]
    @test det(P) == -32
    a = super(lattice, P)  # Four-atom tetragonal supercell
    b = StandardizedFromPrimitive(P)(lattice)  # Two-atom cell to four-atom tetragonal cell
    @test a == b
end

# Example from https://github.com/SimonEnsemble/Xtals.jl/blob/f99aa60/test/crystal.jl#L360-L389
@testset "Test `SBMOF-1` from Xtals.jl" begin
    lattice = Lattice(11.6193, 5.5667, 22.9312, 90, 100.897, 90)
    positions = [
        [0.4932, 0.2586, 0.262],
        [0.0068, 0.7586, 0.238],
        [0.5068, 0.7414, 0.738],
        [0.9932, 0.2414, 0.762],
        [0.3998, 0.2111, 0.2991],
        [0.1002, 0.7111, 0.2009],
        [0.6002, 0.7889, 0.7009],
        [0.8998, 0.2889, 0.7991],
        [0.3121, 0.3789, 0.3001],
        [0.1879, 0.8789, 0.1999],
        [0.6879, 0.6211, 0.6999],
        [0.8121, 0.1211, 0.8001],
        [0.3142, 0.5287, 0.2804],
        [0.1858, 0.0287, 0.2196],
        [0.6858, 0.4713, 0.7196],
        [0.8142, 0.9713, 0.7804],
        [0.2208, 0.3282, 0.3299],
        [0.2792, 0.8282, 0.1701],
        [0.7792, 0.6718, 0.6701],
        [0.7208, 0.1718, 0.8299],
        [0.1597, 0.4413, 0.3304],
        [0.3403, 0.9413, 0.1696],
        [0.8403, 0.5587, 0.6696],
        [0.6597, 0.0587, 0.8304],
        [0.2209, 0.1096, 0.3589],
        [0.2791, 0.6096, 0.1411],
        [0.7791, 0.8904, 0.6411],
        [0.7209, 0.3904, 0.8589],
        [0.3109, 0.9413, 0.3603],
        [0.1891, 0.4413, 0.1397],
        [0.6891, 0.0587, 0.6397],
        [0.8109, 0.5587, 0.8603],
        [0.311, 0.7958, 0.382],
        [0.189, 0.2958, 0.118],
        [0.689, 0.2042, 0.618],
        [0.811, 0.7042, 0.882],
        [0.4002, 0.9912, 0.3293],
        [0.0998, 0.4912, 0.1707],
        [0.5998, 0.0088, 0.6707],
        [0.9002, 0.5088, 0.8293],
        [0.4609, 0.8777, 0.3286],
        [0.0391, 0.3777, 0.1714],
        [0.5391, 0.1223, 0.6714],
        [0.9609, 0.6223, 0.8286],
        [0.149, 0.1119, 0.4702],
        [0.351, 0.6119, 0.0298],
        [0.851, 0.8881, 0.5298],
        [0.649, 0.3881, 0.9702],
        [0.1194, 0.334, 0.4901],
        [0.3806, 0.834, 0.0099],
        [0.8806, 0.666, 0.5099],
        [0.6194, 0.166, 0.9901],
        [0.0803, 0.4502, 0.463],
        [0.4197, 0.9502, 0.037],
        [0.9197, 0.5498, 0.537],
        [0.5803, 0.0498, 0.963],
        [0.1482, 0.3837, 0.5508],
        [0.3518, 0.8837, 0.9492],
        [0.8518, 0.6163, 0.4492],
        [0.6482, 0.1163, 0.0508],
        [0.1298, 0.5361, 0.5652],
        [0.3702, 0.0361, 0.9348],
        [0.8702, 0.4639, 0.4348],
        [0.6298, 0.9639, 0.0652],
        [0.2036, 0.2114, 0.59],
        [0.2964, 0.7114, 0.91],
        [0.7964, 0.7886, 0.41],
        [0.7036, 0.2886, 0.09],
        [0.2367, 0.9905, 0.5693],
        [0.2633, 0.4905, 0.9307],
        [0.7633, 0.0095, 0.4307],
        [0.7367, 0.5095, 0.0693],
        [0.2775, 0.8759, 0.5962],
        [0.2225, 0.3759, 0.9038],
        [0.7225, 0.1241, 0.4038],
        [0.7775, 0.6241, 0.0962],
        [0.2095, 0.9398, 0.5088],
        [0.2905, 0.4398, 0.9912],
        [0.7905, 0.0602, 0.4912],
        [0.7095, 0.5602, 0.0088],
        [0.2317, 0.791, 0.4939],
        [0.2683, 0.291, 0.0061],
        [0.7683, 0.209, 0.5061],
        [0.7317, 0.709, 0.9939],
        [0.2275, 0.2624, 0.6557],
        [0.2725, 0.7624, 0.8443],
        [0.7725, 0.7376, 0.3443],
        [0.7275, 0.2376, 0.1557],
        [0.5124, 0.4755, 0.25221],
        [0.9876, 0.9755, 0.24779],
        [0.4876, 0.5245, 0.74779],
        [0.0124, 0.0245, 0.75221],
        [0.5409, 0.0808, 0.24372],
        [0.9591, 0.5808, 0.25628],
        [0.4591, 0.9192, 0.75628],
        [0.0409, 0.4192, 0.74372],
        [0.0054, 0.1958, 0.37124],
        [0.4946, 0.6958, 0.12876],
        [0.9946, 0.8042, 0.62876],
        [0.5054, 0.3042, 0.87124],
        [0.0866, 0.7796, 0.39054],
        [0.4134, 0.2796, 0.10946],
        [0.9134, 0.2204, 0.60946],
        [0.5866, 0.7204, 0.89054],
        [0.2371, 0.4768, 0.67265],
        [0.2629, 0.9768, 0.82735],
        [0.7629, 0.5232, 0.32735],
        [0.7371, 0.0232, 0.17265],
        [0.2406, 0.085, 0.69066],
        [0.2594, 0.585, 0.80934],
        [0.7594, 0.915, 0.30934],
        [0.7406, 0.415, 0.19066],
        [0.10323, 0.0364, 0.39414],
        [0.39677, 0.5364, 0.10586],
        [0.89677, 0.9636, 0.60586],
        [0.60323, 0.4636, 0.89414],
        [0.6372, 0.74331, 0.22117],
        [0.8628, 0.24331, 0.27883],
        [0.3628, 0.25669, 0.77883],
        [0.1372, 0.75669, 0.72117],
    ]
    atoms = [
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :H,
        :H,
        :H,
        :H,
        :C,
        :C,
        :C,
        :C,
        :H,
        :H,
        :H,
        :H,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :H,
        :H,
        :H,
        :H,
        :C,
        :C,
        :C,
        :C,
        :H,
        :H,
        :H,
        :H,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :H,
        :H,
        :H,
        :H,
        :C,
        :C,
        :C,
        :C,
        :H,
        :H,
        :H,
        :H,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :H,
        :H,
        :H,
        :H,
        :C,
        :C,
        :C,
        :C,
        :H,
        :H,
        :H,
        :H,
        :C,
        :C,
        :C,
        :C,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :S,
        :S,
        :S,
        :S,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
    ]
    cell = Cell(lattice, positions, atoms)
    new_cell = super(cell, [3, 6, 2])
    @test natoms(new_cell) == natoms(cell) * 3 * 6 * 2
    @test cellvolume(new_cell) ≈ cellvolume(cell) * 3 * 6 * 2
    @test reciprocal(Lattice(new_cell)) ≈ ReciprocalLattice(  # Compared with Xtals.jl results
        [
            0.028687901451320934 0.0 0.0
            -1.7566273343307443e-18 0.02993994047939833 0.0
            0.005522861283328621 -2.219893391258545e-18 0.022204736097296805
        ],
    )
    order = sortperm(new_cell.atoms)
    @test new_cell.atoms[order] == [
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :C,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :Ca,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :H,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :O,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
        :S,
    ]  # Compared with Xtals.jl results
    @test extrema(reduce(hcat, new_cell.positions)) ==
        (0.0014666666666666667, 0.9985333333333334)  # Compared with Xtals.jl results
    # Compared with Xtals.jl results
    @test isempty(
        symdiff(
            new_cell.positions[order],
            [
                [0.16440000000000002, 0.0431, 0.131],
                [0.0022666666666666664, 0.12643333333333334, 0.119],
                [0.16893333333333335, 0.12356666666666666, 0.369],
                [0.3310666666666667, 0.040233333333333336, 0.381],
                [0.13326666666666667, 0.03518333333333334, 0.14955],
                [0.0334, 0.11851666666666666, 0.10045],
                [0.20006666666666664, 0.13148333333333334, 0.35045],
                [0.29993333333333333, 0.04815, 0.39955],
                [0.10403333333333332, 0.06315, 0.15005],
                [0.06263333333333333, 0.14648333333333333, 0.09995],
                [0.22929999999999998, 0.10351666666666666, 0.34995],
                [0.2707, 0.020183333333333334, 0.40005],
                [0.0736, 0.0547, 0.16495],
                [0.09306666666666667, 0.13803333333333334, 0.08505],
                [0.2597333333333333, 0.11196666666666666, 0.33505],
                [0.24026666666666666, 0.028633333333333334, 0.41495],
                [0.07363333333333334, 0.018266666666666667, 0.17945],
                [0.09303333333333334, 0.10160000000000001, 0.07055],
                [0.2597, 0.1484, 0.32055],
                [0.24029999999999999, 0.06506666666666668, 0.42945],
                [0.10363333333333334, 0.15688333333333335, 0.18015],
                [0.06303333333333333, 0.07355, 0.06985],
                [0.22970000000000002, 0.009783333333333333, 0.31985],
                [0.2703, 0.09311666666666667, 0.43015],
                [0.1334, 0.16519999999999999, 0.16465],
                [0.03326666666666667, 0.08186666666666667, 0.08535],
                [0.19993333333333332, 0.0014666666666666667, 0.33535],
                [0.30006666666666665, 0.0848, 0.41465],
                [0.049666666666666665, 0.01865, 0.2351],
                [0.11699999999999999, 0.10198333333333333, 0.0149],
                [0.2836666666666667, 0.14801666666666666, 0.2649],
                [0.21633333333333335, 0.06468333333333333, 0.4851],
                [0.0398, 0.05566666666666667, 0.24505],
                [0.12686666666666666, 0.13899999999999998, 0.00495],
                [0.29353333333333337, 0.111, 0.25495],
                [0.20646666666666666, 0.02766666666666667, 0.49505],
                [0.0494, 0.06394999999999999, 0.2754],
                [0.11726666666666667, 0.14728333333333335, 0.4746],
                [0.2839333333333333, 0.10271666666666666, 0.2246],
                [0.21606666666666666, 0.019383333333333332, 0.0254],
                [0.06786666666666667, 0.03523333333333333, 0.295],
                [0.0988, 0.11856666666666667, 0.455],
                [0.2654666666666667, 0.13143333333333332, 0.205],
                [0.23453333333333334, 0.048100000000000004, 0.045],
                [0.0789, 0.16508333333333333, 0.28465],
                [0.08776666666666666, 0.08175, 0.46535],
                [0.25443333333333334, 0.0015833333333333333, 0.21535],
                [0.24556666666666668, 0.08491666666666665, 0.03465],
                [0.06983333333333333, 0.15663333333333332, 0.2544],
                [0.09683333333333333, 0.0733, 0.4956],
                [0.2635, 0.010033333333333333, 0.2456],
                [0.23650000000000002, 0.09336666666666667, 0.0044],
                [0.07583333333333334, 0.04373333333333334, 0.32785],
                [0.09083333333333334, 0.12706666666666666, 0.42215],
                [0.2575, 0.12293333333333334, 0.17215],
                [0.24250000000000002, 0.0396, 0.07785],
                [0.16440000000000002, 0.0431, 0.631],
                [0.0022666666666666664, 0.12643333333333334, 0.619],
                [0.16893333333333335, 0.12356666666666666, 0.869],
                [0.3310666666666667, 0.040233333333333336, 0.881],
                [0.13326666666666667, 0.03518333333333334, 0.64955],
                [0.0334, 0.11851666666666666, 0.60045],
                [0.20006666666666664, 0.13148333333333334, 0.8504499999999999],
                [0.29993333333333333, 0.04815, 0.8995500000000001],
                [0.10403333333333332, 0.06315, 0.65005],
                [0.06263333333333333, 0.14648333333333333, 0.59995],
                [0.22929999999999998, 0.10351666666666666, 0.84995],
                [0.2707, 0.020183333333333334, 0.90005],
                [0.0736, 0.0547, 0.66495],
                [0.09306666666666667, 0.13803333333333334, 0.58505],
                [0.2597333333333333, 0.11196666666666666, 0.8350500000000001],
                [0.24026666666666666, 0.028633333333333334, 0.9149499999999999],
                [0.07363333333333334, 0.018266666666666667, 0.67945],
                [0.09303333333333334, 0.10160000000000001, 0.57055],
                [0.2597, 0.1484, 0.82055],
                [0.24029999999999999, 0.06506666666666668, 0.92945],
                [0.10363333333333334, 0.15688333333333335, 0.68015],
                [0.06303333333333333, 0.07355, 0.56985],
                [0.22970000000000002, 0.009783333333333333, 0.81985],
                [0.2703, 0.09311666666666667, 0.93015],
                [0.1334, 0.16519999999999999, 0.66465],
                [0.03326666666666667, 0.08186666666666667, 0.58535],
                [0.19993333333333332, 0.0014666666666666667, 0.83535],
                [0.30006666666666665, 0.0848, 0.91465],
                [0.049666666666666665, 0.01865, 0.7351],
                [0.11699999999999999, 0.10198333333333333, 0.5149],
                [0.2836666666666667, 0.14801666666666666, 0.7649],
                [0.21633333333333335, 0.06468333333333333, 0.9851],
                [0.0398, 0.05566666666666667, 0.74505],
                [0.12686666666666666, 0.13899999999999998, 0.50495],
                [0.29353333333333337, 0.111, 0.75495],
                [0.20646666666666666, 0.02766666666666667, 0.99505],
                [0.0494, 0.06394999999999999, 0.7754],
                [0.11726666666666667, 0.14728333333333335, 0.9746],
                [0.2839333333333333, 0.10271666666666666, 0.7246],
                [0.21606666666666666, 0.019383333333333332, 0.5254],
                [0.06786666666666667, 0.03523333333333333, 0.7949999999999999],
                [0.0988, 0.11856666666666667, 0.9550000000000001],
                [0.2654666666666667, 0.13143333333333332, 0.705],
                [0.23453333333333334, 0.048100000000000004, 0.545],
                [0.0789, 0.16508333333333333, 0.7846500000000001],
                [0.08776666666666666, 0.08175, 0.9653499999999999],
                [0.25443333333333334, 0.0015833333333333333, 0.71535],
                [0.24556666666666668, 0.08491666666666665, 0.53465],
                [0.06983333333333333, 0.15663333333333332, 0.7544],
                [0.09683333333333333, 0.0733, 0.9956],
                [0.2635, 0.010033333333333333, 0.7456],
                [0.23650000000000002, 0.09336666666666667, 0.5044],
                [0.07583333333333334, 0.04373333333333334, 0.82785],
                [0.09083333333333334, 0.12706666666666666, 0.92215],
                [0.2575, 0.12293333333333334, 0.67215],
                [0.24250000000000002, 0.0396, 0.57785],
                [0.16440000000000002, 0.20976666666666666, 0.131],
                [0.0022666666666666664, 0.29309999999999997, 0.119],
                [0.16893333333333335, 0.29023333333333334, 0.369],
                [0.3310666666666667, 0.2069, 0.381],
                [0.13326666666666667, 0.20185, 0.14955],
                [0.0334, 0.28518333333333334, 0.10045],
                [0.20006666666666664, 0.29814999999999997, 0.35045],
                [0.29993333333333333, 0.21481666666666666, 0.39955],
                [0.10403333333333332, 0.22981666666666667, 0.15005],
                [0.06263333333333333, 0.31315, 0.09995],
                [0.22929999999999998, 0.27018333333333333, 0.34995],
                [0.2707, 0.18685, 0.40005],
                [0.0736, 0.22136666666666668, 0.16495],
                [0.09306666666666667, 0.3047, 0.08505],
                [0.2597333333333333, 0.27863333333333334, 0.33505],
                [0.24026666666666666, 0.1953, 0.41495],
                [0.07363333333333334, 0.1849333333333333, 0.17945],
                [0.09303333333333334, 0.26826666666666665, 0.07055],
                [0.2597, 0.31506666666666666, 0.32055],
                [0.24029999999999999, 0.23173333333333335, 0.42945],
                [0.10363333333333334, 0.32355, 0.18015],
                [0.06303333333333333, 0.24021666666666666, 0.06985],
                [0.22970000000000002, 0.17645, 0.31985],
                [0.2703, 0.2597833333333333, 0.43015],
                [0.1334, 0.3318666666666667, 0.16465],
                [0.03326666666666667, 0.24853333333333336, 0.08535],
                [0.19993333333333332, 0.16813333333333333, 0.33535],
                [0.30006666666666665, 0.25146666666666667, 0.41465],
                [0.049666666666666665, 0.18531666666666666, 0.2351],
                [0.11699999999999999, 0.26865, 0.0149],
                [0.2836666666666667, 0.31468333333333337, 0.2649],
                [0.21633333333333335, 0.23135000000000003, 0.4851],
                [0.0398, 0.22233333333333336, 0.24505],
                [0.12686666666666666, 0.3056666666666667, 0.00495],
                [0.29353333333333337, 0.2776666666666667, 0.25495],
                [0.20646666666666666, 0.19433333333333333, 0.49505],
                [0.0494, 0.23061666666666666, 0.2754],
                [0.11726666666666667, 0.31395, 0.4746],
                [0.2839333333333333, 0.2693833333333333, 0.2246],
                [0.21606666666666666, 0.18605000000000002, 0.0254],
                [0.06786666666666667, 0.2019, 0.295],
                [0.0988, 0.28523333333333334, 0.455],
                [0.2654666666666667, 0.2981, 0.205],
                [0.23453333333333334, 0.21476666666666666, 0.045],
                [0.0789, 0.33175, 0.28465],
                [0.08776666666666666, 0.24841666666666665, 0.46535],
                [0.25443333333333334, 0.16825, 0.21535],
                [0.24556666666666668, 0.2515833333333333, 0.03465],
                [0.06983333333333333, 0.3233, 0.2544],
                [0.09683333333333333, 0.23996666666666666, 0.4956],
                [0.2635, 0.1767, 0.2456],
                [0.23650000000000002, 0.26003333333333334, 0.0044],
                [0.07583333333333334, 0.2104, 0.32785],
                [0.09083333333333334, 0.29373333333333335, 0.42215],
                [0.2575, 0.2896, 0.17215],
                [0.24250000000000002, 0.20626666666666668, 0.07785],
                [0.16440000000000002, 0.20976666666666666, 0.631],
                [0.0022666666666666664, 0.29309999999999997, 0.619],
                [0.16893333333333335, 0.29023333333333334, 0.869],
                [0.3310666666666667, 0.2069, 0.881],
                [0.13326666666666667, 0.20185, 0.64955],
                [0.0334, 0.28518333333333334, 0.60045],
                [0.20006666666666664, 0.29814999999999997, 0.8504499999999999],
                [0.29993333333333333, 0.21481666666666666, 0.8995500000000001],
                [0.10403333333333332, 0.22981666666666667, 0.65005],
                [0.06263333333333333, 0.31315, 0.59995],
                [0.22929999999999998, 0.27018333333333333, 0.84995],
                [0.2707, 0.18685, 0.90005],
                [0.0736, 0.22136666666666668, 0.66495],
                [0.09306666666666667, 0.3047, 0.58505],
                [0.2597333333333333, 0.27863333333333334, 0.8350500000000001],
                [0.24026666666666666, 0.1953, 0.9149499999999999],
                [0.07363333333333334, 0.1849333333333333, 0.67945],
                [0.09303333333333334, 0.26826666666666665, 0.57055],
                [0.2597, 0.31506666666666666, 0.82055],
                [0.24029999999999999, 0.23173333333333335, 0.92945],
                [0.10363333333333334, 0.32355, 0.68015],
                [0.06303333333333333, 0.24021666666666666, 0.56985],
                [0.22970000000000002, 0.17645, 0.81985],
                [0.2703, 0.2597833333333333, 0.93015],
                [0.1334, 0.3318666666666667, 0.66465],
                [0.03326666666666667, 0.24853333333333336, 0.58535],
                [0.19993333333333332, 0.16813333333333333, 0.83535],
                [0.30006666666666665, 0.25146666666666667, 0.91465],
                [0.049666666666666665, 0.18531666666666666, 0.7351],
                [0.11699999999999999, 0.26865, 0.5149],
                [0.2836666666666667, 0.31468333333333337, 0.7649],
                [0.21633333333333335, 0.23135000000000003, 0.9851],
                [0.0398, 0.22233333333333336, 0.74505],
                [0.12686666666666666, 0.3056666666666667, 0.50495],
                [0.29353333333333337, 0.2776666666666667, 0.75495],
                [0.20646666666666666, 0.19433333333333333, 0.99505],
                [0.0494, 0.23061666666666666, 0.7754],
                [0.11726666666666667, 0.31395, 0.9746],
                [0.2839333333333333, 0.2693833333333333, 0.7246],
                [0.21606666666666666, 0.18605000000000002, 0.5254],
                [0.06786666666666667, 0.2019, 0.7949999999999999],
                [0.0988, 0.28523333333333334, 0.9550000000000001],
                [0.2654666666666667, 0.2981, 0.705],
                [0.23453333333333334, 0.21476666666666666, 0.545],
                [0.0789, 0.33175, 0.7846500000000001],
                [0.08776666666666666, 0.24841666666666665, 0.9653499999999999],
                [0.25443333333333334, 0.16825, 0.71535],
                [0.24556666666666668, 0.2515833333333333, 0.53465],
                [0.06983333333333333, 0.3233, 0.7544],
                [0.09683333333333333, 0.23996666666666666, 0.9956],
                [0.2635, 0.1767, 0.7456],
                [0.23650000000000002, 0.26003333333333334, 0.5044],
                [0.07583333333333334, 0.2104, 0.82785],
                [0.09083333333333334, 0.29373333333333335, 0.92215],
                [0.2575, 0.2896, 0.67215],
                [0.24250000000000002, 0.20626666666666668, 0.57785],
                [0.16440000000000002, 0.37643333333333334, 0.131],
                [0.0022666666666666664, 0.45976666666666666, 0.119],
                [0.16893333333333335, 0.45690000000000003, 0.369],
                [0.3310666666666667, 0.37356666666666666, 0.381],
                [0.13326666666666667, 0.36851666666666666, 0.14955],
                [0.0334, 0.45185000000000003, 0.10045],
                [0.20006666666666664, 0.46481666666666666, 0.35045],
                [0.29993333333333333, 0.38148333333333334, 0.39955],
                [0.10403333333333332, 0.3964833333333333, 0.15005],
                [0.06263333333333333, 0.4798166666666666, 0.09995],
                [0.22929999999999998, 0.43685, 0.34995],
                [0.2707, 0.3535166666666667, 0.40005],
                [0.0736, 0.3880333333333333, 0.16495],
                [0.09306666666666667, 0.47136666666666666, 0.08505],
                [0.2597333333333333, 0.44530000000000003, 0.33505],
                [0.24026666666666666, 0.3619666666666667, 0.41495],
                [0.07363333333333334, 0.35159999999999997, 0.17945],
                [0.09303333333333334, 0.43493333333333334, 0.07055],
                [0.2597, 0.48173333333333335, 0.32055],
                [0.24029999999999999, 0.39840000000000003, 0.42945],
                [0.10363333333333334, 0.4902166666666667, 0.18015],
                [0.06303333333333333, 0.4068833333333333, 0.06985],
                [0.22970000000000002, 0.3431166666666667, 0.31985],
                [0.2703, 0.42645, 0.43015],
                [0.1334, 0.49853333333333333, 0.16465],
                [0.03326666666666667, 0.4152, 0.08535],
                [0.19993333333333332, 0.3348, 0.33535],
                [0.30006666666666665, 0.4181333333333333, 0.41465],
                [0.049666666666666665, 0.3519833333333333, 0.2351],
                [0.11699999999999999, 0.43531666666666663, 0.0149],
                [0.2836666666666667, 0.48135, 0.2649],
                [0.21633333333333335, 0.3980166666666667, 0.4851],
                [0.0398, 0.389, 0.24505],
                [0.12686666666666666, 0.4723333333333333, 0.00495],
                [0.29353333333333337, 0.4443333333333333, 0.25495],
                [0.20646666666666666, 0.361, 0.49505],
                [0.0494, 0.3972833333333334, 0.2754],
                [0.11726666666666667, 0.4806166666666667, 0.4746],
                [0.2839333333333333, 0.43605, 0.2246],
                [0.21606666666666666, 0.3527166666666666, 0.0254],
                [0.06786666666666667, 0.36856666666666665, 0.295],
                [0.0988, 0.4519, 0.455],
                [0.2654666666666667, 0.4647666666666666, 0.205],
                [0.23453333333333334, 0.38143333333333335, 0.045],
                [0.0789, 0.4984166666666667, 0.28465],
                [0.08776666666666666, 0.4150833333333333, 0.46535],
                [0.25443333333333334, 0.3349166666666667, 0.21535],
                [0.24556666666666668, 0.41825, 0.03465],
                [0.06983333333333333, 0.48996666666666666, 0.2544],
                [0.09683333333333333, 0.40663333333333335, 0.4956],
                [0.2635, 0.34336666666666665, 0.2456],
                [0.23650000000000002, 0.4267, 0.0044],
                [0.07583333333333334, 0.37706666666666666, 0.32785],
                [0.09083333333333334, 0.4604, 0.42215],
                [0.2575, 0.45626666666666665, 0.17215],
                [0.24250000000000002, 0.37293333333333334, 0.07785],
                [0.16440000000000002, 0.37643333333333334, 0.631],
                [0.0022666666666666664, 0.45976666666666666, 0.619],
                [0.16893333333333335, 0.45690000000000003, 0.869],
                [0.3310666666666667, 0.37356666666666666, 0.881],
                [0.13326666666666667, 0.36851666666666666, 0.64955],
                [0.0334, 0.45185000000000003, 0.60045],
                [0.20006666666666664, 0.46481666666666666, 0.8504499999999999],
                [0.29993333333333333, 0.38148333333333334, 0.8995500000000001],
                [0.10403333333333332, 0.3964833333333333, 0.65005],
                [0.06263333333333333, 0.4798166666666666, 0.59995],
                [0.22929999999999998, 0.43685, 0.84995],
                [0.2707, 0.3535166666666667, 0.90005],
                [0.0736, 0.3880333333333333, 0.66495],
                [0.09306666666666667, 0.47136666666666666, 0.58505],
                [0.2597333333333333, 0.44530000000000003, 0.8350500000000001],
                [0.24026666666666666, 0.3619666666666667, 0.9149499999999999],
                [0.07363333333333334, 0.35159999999999997, 0.67945],
                [0.09303333333333334, 0.43493333333333334, 0.57055],
                [0.2597, 0.48173333333333335, 0.82055],
                [0.24029999999999999, 0.39840000000000003, 0.92945],
                [0.10363333333333334, 0.4902166666666667, 0.68015],
                [0.06303333333333333, 0.4068833333333333, 0.56985],
                [0.22970000000000002, 0.3431166666666667, 0.81985],
                [0.2703, 0.42645, 0.93015],
                [0.1334, 0.49853333333333333, 0.66465],
                [0.03326666666666667, 0.4152, 0.58535],
                [0.19993333333333332, 0.3348, 0.83535],
                [0.30006666666666665, 0.4181333333333333, 0.91465],
                [0.049666666666666665, 0.3519833333333333, 0.7351],
                [0.11699999999999999, 0.43531666666666663, 0.5149],
                [0.2836666666666667, 0.48135, 0.7649],
                [0.21633333333333335, 0.3980166666666667, 0.9851],
                [0.0398, 0.389, 0.74505],
                [0.12686666666666666, 0.4723333333333333, 0.50495],
                [0.29353333333333337, 0.4443333333333333, 0.75495],
                [0.20646666666666666, 0.361, 0.99505],
                [0.0494, 0.3972833333333334, 0.7754],
                [0.11726666666666667, 0.4806166666666667, 0.9746],
                [0.2839333333333333, 0.43605, 0.7246],
                [0.21606666666666666, 0.3527166666666666, 0.5254],
                [0.06786666666666667, 0.36856666666666665, 0.7949999999999999],
                [0.0988, 0.4519, 0.9550000000000001],
                [0.2654666666666667, 0.4647666666666666, 0.705],
                [0.23453333333333334, 0.38143333333333335, 0.545],
                [0.0789, 0.4984166666666667, 0.7846500000000001],
                [0.08776666666666666, 0.4150833333333333, 0.9653499999999999],
                [0.25443333333333334, 0.3349166666666667, 0.71535],
                [0.24556666666666668, 0.41825, 0.53465],
                [0.06983333333333333, 0.48996666666666666, 0.7544],
                [0.09683333333333333, 0.40663333333333335, 0.9956],
                [0.2635, 0.34336666666666665, 0.7456],
                [0.23650000000000002, 0.4267, 0.5044],
                [0.07583333333333334, 0.37706666666666666, 0.82785],
                [0.09083333333333334, 0.4604, 0.92215],
                [0.2575, 0.45626666666666665, 0.67215],
                [0.24250000000000002, 0.37293333333333334, 0.57785],
                [0.16440000000000002, 0.5431, 0.131],
                [0.0022666666666666664, 0.6264333333333333, 0.119],
                [0.16893333333333335, 0.6235666666666667, 0.369],
                [0.3310666666666667, 0.5402333333333333, 0.381],
                [0.13326666666666667, 0.5351833333333333, 0.14955],
                [0.0334, 0.6185166666666667, 0.10045],
                [0.20006666666666664, 0.6314833333333333, 0.35045],
                [0.29993333333333333, 0.54815, 0.39955],
                [0.10403333333333332, 0.5631499999999999, 0.15005],
                [0.06263333333333333, 0.6464833333333333, 0.09995],
                [0.22929999999999998, 0.6035166666666667, 0.34995],
                [0.2707, 0.5201833333333333, 0.40005],
                [0.0736, 0.5547, 0.16495],
                [0.09306666666666667, 0.6380333333333333, 0.08505],
                [0.2597333333333333, 0.6119666666666667, 0.33505],
                [0.24026666666666666, 0.5286333333333334, 0.41495],
                [0.07363333333333334, 0.5182666666666667, 0.17945],
                [0.09303333333333334, 0.6016, 0.07055],
                [0.2597, 0.6484, 0.32055],
                [0.24029999999999999, 0.5650666666666667, 0.42945],
                [0.10363333333333334, 0.6568833333333334, 0.18015],
                [0.06303333333333333, 0.57355, 0.06985],
                [0.22970000000000002, 0.5097833333333334, 0.31985],
                [0.2703, 0.5931166666666666, 0.43015],
                [0.1334, 0.6652, 0.16465],
                [0.03326666666666667, 0.5818666666666666, 0.08535],
                [0.19993333333333332, 0.5014666666666666, 0.33535],
                [0.30006666666666665, 0.5848, 0.41465],
                [0.049666666666666665, 0.5186499999999999, 0.2351],
                [0.11699999999999999, 0.6019833333333333, 0.0149],
                [0.2836666666666667, 0.6480166666666667, 0.2649],
                [0.21633333333333335, 0.5646833333333333, 0.4851],
                [0.0398, 0.5556666666666666, 0.24505],
                [0.12686666666666666, 0.639, 0.00495],
                [0.29353333333333337, 0.611, 0.25495],
                [0.20646666666666666, 0.5276666666666666, 0.49505],
                [0.0494, 0.5639500000000001, 0.2754],
                [0.11726666666666667, 0.6472833333333333, 0.4746],
                [0.2839333333333333, 0.6027166666666667, 0.2246],
                [0.21606666666666666, 0.5193833333333333, 0.0254],
                [0.06786666666666667, 0.5352333333333333, 0.295],
                [0.0988, 0.6185666666666667, 0.455],
                [0.2654666666666667, 0.6314333333333333, 0.205],
                [0.23453333333333334, 0.5481, 0.045],
                [0.0789, 0.6650833333333334, 0.28465],
                [0.08776666666666666, 0.58175, 0.46535],
                [0.25443333333333334, 0.5015833333333334, 0.21535],
                [0.24556666666666668, 0.5849166666666666, 0.03465],
                [0.06983333333333333, 0.6566333333333333, 0.2544],
                [0.09683333333333333, 0.5733, 0.4956],
                [0.2635, 0.5100333333333333, 0.2456],
                [0.23650000000000002, 0.5933666666666667, 0.0044],
                [0.07583333333333334, 0.5437333333333333, 0.32785],
                [0.09083333333333334, 0.6270666666666667, 0.42215],
                [0.2575, 0.6229333333333333, 0.17215],
                [0.24250000000000002, 0.5396, 0.07785],
                [0.16440000000000002, 0.5431, 0.631],
                [0.0022666666666666664, 0.6264333333333333, 0.619],
                [0.16893333333333335, 0.6235666666666667, 0.869],
                [0.3310666666666667, 0.5402333333333333, 0.881],
                [0.13326666666666667, 0.5351833333333333, 0.64955],
                [0.0334, 0.6185166666666667, 0.60045],
                [0.20006666666666664, 0.6314833333333333, 0.8504499999999999],
                [0.29993333333333333, 0.54815, 0.8995500000000001],
                [0.10403333333333332, 0.5631499999999999, 0.65005],
                [0.06263333333333333, 0.6464833333333333, 0.59995],
                [0.22929999999999998, 0.6035166666666667, 0.84995],
                [0.2707, 0.5201833333333333, 0.90005],
                [0.0736, 0.5547, 0.66495],
                [0.09306666666666667, 0.6380333333333333, 0.58505],
                [0.2597333333333333, 0.6119666666666667, 0.8350500000000001],
                [0.24026666666666666, 0.5286333333333334, 0.9149499999999999],
                [0.07363333333333334, 0.5182666666666667, 0.67945],
                [0.09303333333333334, 0.6016, 0.57055],
                [0.2597, 0.6484, 0.82055],
                [0.24029999999999999, 0.5650666666666667, 0.92945],
                [0.10363333333333334, 0.6568833333333334, 0.68015],
                [0.06303333333333333, 0.57355, 0.56985],
                [0.22970000000000002, 0.5097833333333334, 0.81985],
                [0.2703, 0.5931166666666666, 0.93015],
                [0.1334, 0.6652, 0.66465],
                [0.03326666666666667, 0.5818666666666666, 0.58535],
                [0.19993333333333332, 0.5014666666666666, 0.83535],
                [0.30006666666666665, 0.5848, 0.91465],
                [0.049666666666666665, 0.5186499999999999, 0.7351],
                [0.11699999999999999, 0.6019833333333333, 0.5149],
                [0.2836666666666667, 0.6480166666666667, 0.7649],
                [0.21633333333333335, 0.5646833333333333, 0.9851],
                [0.0398, 0.5556666666666666, 0.74505],
                [0.12686666666666666, 0.639, 0.50495],
                [0.29353333333333337, 0.611, 0.75495],
                [0.20646666666666666, 0.5276666666666666, 0.99505],
                [0.0494, 0.5639500000000001, 0.7754],
                [0.11726666666666667, 0.6472833333333333, 0.9746],
                [0.2839333333333333, 0.6027166666666667, 0.7246],
                [0.21606666666666666, 0.5193833333333333, 0.5254],
                [0.06786666666666667, 0.5352333333333333, 0.7949999999999999],
                [0.0988, 0.6185666666666667, 0.9550000000000001],
                [0.2654666666666667, 0.6314333333333333, 0.705],
                [0.23453333333333334, 0.5481, 0.545],
                [0.0789, 0.6650833333333334, 0.7846500000000001],
                [0.08776666666666666, 0.58175, 0.9653499999999999],
                [0.25443333333333334, 0.5015833333333334, 0.71535],
                [0.24556666666666668, 0.5849166666666666, 0.53465],
                [0.06983333333333333, 0.6566333333333333, 0.7544],
                [0.09683333333333333, 0.5733, 0.9956],
                [0.2635, 0.5100333333333333, 0.7456],
                [0.23650000000000002, 0.5933666666666667, 0.5044],
                [0.07583333333333334, 0.5437333333333333, 0.82785],
                [0.09083333333333334, 0.6270666666666667, 0.92215],
                [0.2575, 0.6229333333333333, 0.67215],
                [0.24250000000000002, 0.5396, 0.57785],
                [0.16440000000000002, 0.7097666666666668, 0.131],
                [0.0022666666666666664, 0.7931, 0.119],
                [0.16893333333333335, 0.7902333333333332, 0.369],
                [0.3310666666666667, 0.7069, 0.381],
                [0.13326666666666667, 0.70185, 0.14955],
                [0.0334, 0.7851833333333333, 0.10045],
                [0.20006666666666664, 0.79815, 0.35045],
                [0.29993333333333333, 0.7148166666666667, 0.39955],
                [0.10403333333333332, 0.7298166666666667, 0.15005],
                [0.06263333333333333, 0.8131499999999999, 0.09995],
                [0.22929999999999998, 0.7701833333333333, 0.34995],
                [0.2707, 0.6868500000000001, 0.40005],
                [0.0736, 0.7213666666666666, 0.16495],
                [0.09306666666666667, 0.8047, 0.08505],
                [0.2597333333333333, 0.7786333333333334, 0.33505],
                [0.24026666666666666, 0.6953, 0.41495],
                [0.07363333333333334, 0.6849333333333334, 0.17945],
                [0.09303333333333334, 0.7682666666666668, 0.07055],
                [0.2597, 0.8150666666666666, 0.32055],
                [0.24029999999999999, 0.7317333333333332, 0.42945],
                [0.10363333333333334, 0.82355, 0.18015],
                [0.06303333333333333, 0.7402166666666666, 0.06985],
                [0.22970000000000002, 0.67645, 0.31985],
                [0.2703, 0.7597833333333334, 0.43015],
                [0.1334, 0.8318666666666666, 0.16465],
                [0.03326666666666667, 0.7485333333333334, 0.08535],
                [0.19993333333333332, 0.6681333333333334, 0.33535],
                [0.30006666666666665, 0.7514666666666666, 0.41465],
                [0.049666666666666665, 0.6853166666666667, 0.2351],
                [0.11699999999999999, 0.76865, 0.0149],
                [0.2836666666666667, 0.8146833333333333, 0.2649],
                [0.21633333333333335, 0.73135, 0.4851],
                [0.0398, 0.7223333333333333, 0.24505],
                [0.12686666666666666, 0.8056666666666666, 0.00495],
                [0.29353333333333337, 0.7776666666666667, 0.25495],
                [0.20646666666666666, 0.6943333333333334, 0.49505],
                [0.0494, 0.7306166666666667, 0.2754],
                [0.11726666666666667, 0.8139500000000001, 0.4746],
                [0.2839333333333333, 0.7693833333333333, 0.2246],
                [0.21606666666666666, 0.6860499999999999, 0.0254],
                [0.06786666666666667, 0.7019000000000001, 0.295],
                [0.0988, 0.7852333333333333, 0.455],
                [0.2654666666666667, 0.7980999999999999, 0.205],
                [0.23453333333333334, 0.7147666666666667, 0.045],
                [0.0789, 0.83175, 0.28465],
                [0.08776666666666666, 0.7484166666666666, 0.46535],
                [0.25443333333333334, 0.66825, 0.21535],
                [0.24556666666666668, 0.7515833333333334, 0.03465],
                [0.06983333333333333, 0.8233, 0.2544],
                [0.09683333333333333, 0.7399666666666667, 0.4956],
                [0.2635, 0.6767, 0.2456],
                [0.23650000000000002, 0.7600333333333333, 0.0044],
                [0.07583333333333334, 0.7104, 0.32785],
                [0.09083333333333334, 0.7937333333333333, 0.42215],
                [0.2575, 0.7896000000000001, 0.17215],
                [0.24250000000000002, 0.7062666666666666, 0.07785],
                [0.16440000000000002, 0.7097666666666668, 0.631],
                [0.0022666666666666664, 0.7931, 0.619],
                [0.16893333333333335, 0.7902333333333332, 0.869],
                [0.3310666666666667, 0.7069, 0.881],
                [0.13326666666666667, 0.70185, 0.64955],
                [0.0334, 0.7851833333333333, 0.60045],
                [0.20006666666666664, 0.79815, 0.8504499999999999],
                [0.29993333333333333, 0.7148166666666667, 0.8995500000000001],
                [0.10403333333333332, 0.7298166666666667, 0.65005],
                [0.06263333333333333, 0.8131499999999999, 0.59995],
                [0.22929999999999998, 0.7701833333333333, 0.84995],
                [0.2707, 0.6868500000000001, 0.90005],
                [0.0736, 0.7213666666666666, 0.66495],
                [0.09306666666666667, 0.8047, 0.58505],
                [0.2597333333333333, 0.7786333333333334, 0.8350500000000001],
                [0.24026666666666666, 0.6953, 0.9149499999999999],
                [0.07363333333333334, 0.6849333333333334, 0.67945],
                [0.09303333333333334, 0.7682666666666668, 0.57055],
                [0.2597, 0.8150666666666666, 0.82055],
                [0.24029999999999999, 0.7317333333333332, 0.92945],
                [0.10363333333333334, 0.82355, 0.68015],
                [0.06303333333333333, 0.7402166666666666, 0.56985],
                [0.22970000000000002, 0.67645, 0.81985],
                [0.2703, 0.7597833333333334, 0.93015],
                [0.1334, 0.8318666666666666, 0.66465],
                [0.03326666666666667, 0.7485333333333334, 0.58535],
                [0.19993333333333332, 0.6681333333333334, 0.83535],
                [0.30006666666666665, 0.7514666666666666, 0.91465],
                [0.049666666666666665, 0.6853166666666667, 0.7351],
                [0.11699999999999999, 0.76865, 0.5149],
                [0.2836666666666667, 0.8146833333333333, 0.7649],
                [0.21633333333333335, 0.73135, 0.9851],
                [0.0398, 0.7223333333333333, 0.74505],
                [0.12686666666666666, 0.8056666666666666, 0.50495],
                [0.29353333333333337, 0.7776666666666667, 0.75495],
                [0.20646666666666666, 0.6943333333333334, 0.99505],
                [0.0494, 0.7306166666666667, 0.7754],
                [0.11726666666666667, 0.8139500000000001, 0.9746],
                [0.2839333333333333, 0.7693833333333333, 0.7246],
                [0.21606666666666666, 0.6860499999999999, 0.5254],
                [0.06786666666666667, 0.7019000000000001, 0.7949999999999999],
                [0.0988, 0.7852333333333333, 0.9550000000000001],
                [0.2654666666666667, 0.7980999999999999, 0.705],
                [0.23453333333333334, 0.7147666666666667, 0.545],
                [0.0789, 0.83175, 0.7846500000000001],
                [0.08776666666666666, 0.7484166666666666, 0.9653499999999999],
                [0.25443333333333334, 0.66825, 0.71535],
                [0.24556666666666668, 0.7515833333333334, 0.53465],
                [0.06983333333333333, 0.8233, 0.7544],
                [0.09683333333333333, 0.7399666666666667, 0.9956],
                [0.2635, 0.6767, 0.7456],
                [0.23650000000000002, 0.7600333333333333, 0.5044],
                [0.07583333333333334, 0.7104, 0.82785],
                [0.09083333333333334, 0.7937333333333333, 0.92215],
                [0.2575, 0.7896000000000001, 0.67215],
                [0.24250000000000002, 0.7062666666666666, 0.57785],
                [0.16440000000000002, 0.8764333333333334, 0.131],
                [0.0022666666666666664, 0.9597666666666668, 0.119],
                [0.16893333333333335, 0.9569, 0.369],
                [0.3310666666666667, 0.8735666666666666, 0.381],
                [0.13326666666666667, 0.8685166666666667, 0.14955],
                [0.0334, 0.95185, 0.10045],
                [0.20006666666666664, 0.9648166666666667, 0.35045],
                [0.29993333333333333, 0.8814833333333333, 0.39955],
                [0.10403333333333332, 0.8964833333333333, 0.15005],
                [0.06263333333333333, 0.9798166666666667, 0.09995],
                [0.22929999999999998, 0.9368500000000001, 0.34995],
                [0.2707, 0.8535166666666667, 0.40005],
                [0.0736, 0.8880333333333333, 0.16495],
                [0.09306666666666667, 0.9713666666666666, 0.08505],
                [0.2597333333333333, 0.9453, 0.33505],
                [0.24026666666666666, 0.8619666666666667, 0.41495],
                [0.07363333333333334, 0.8516, 0.17945],
                [0.09303333333333334, 0.9349333333333334, 0.07055],
                [0.2597, 0.9817333333333332, 0.32055],
                [0.24029999999999999, 0.8984, 0.42945],
                [0.10363333333333334, 0.9902166666666666, 0.18015],
                [0.06303333333333333, 0.9068833333333334, 0.06985],
                [0.22970000000000002, 0.8431166666666666, 0.31985],
                [0.2703, 0.92645, 0.43015],
                [0.1334, 0.9985333333333334, 0.16465],
                [0.03326666666666667, 0.9152, 0.08535],
                [0.19993333333333332, 0.8348, 0.33535],
                [0.30006666666666665, 0.9181333333333334, 0.41465],
                [0.049666666666666665, 0.8519833333333334, 0.2351],
                [0.11699999999999999, 0.9353166666666667, 0.0149],
                [0.2836666666666667, 0.98135, 0.2649],
                [0.21633333333333335, 0.8980166666666666, 0.4851],
                [0.0398, 0.8889999999999999, 0.24505],
                [0.12686666666666666, 0.9723333333333333, 0.00495],
                [0.29353333333333337, 0.9443333333333334, 0.25495],
                [0.20646666666666666, 0.8610000000000001, 0.49505],
                [0.0494, 0.8972833333333333, 0.2754],
                [0.11726666666666667, 0.9806166666666667, 0.4746],
                [0.2839333333333333, 0.9360499999999999, 0.2246],
                [0.21606666666666666, 0.8527166666666667, 0.0254],
                [0.06786666666666667, 0.8685666666666667, 0.295],
                [0.0988, 0.9519000000000001, 0.455],
                [0.2654666666666667, 0.9647666666666667, 0.205],
                [0.23453333333333334, 0.8814333333333333, 0.045],
                [0.0789, 0.9984166666666666, 0.28465],
                [0.08776666666666666, 0.9150833333333334, 0.46535],
                [0.25443333333333334, 0.8349166666666666, 0.21535],
                [0.24556666666666668, 0.91825, 0.03465],
                [0.06983333333333333, 0.9899666666666667, 0.2544],
                [0.09683333333333333, 0.9066333333333333, 0.4956],
                [0.2635, 0.8433666666666667, 0.2456],
                [0.23650000000000002, 0.9267, 0.0044],
                [0.07583333333333334, 0.8770666666666668, 0.32785],
                [0.09083333333333334, 0.9603999999999999, 0.42215],
                [0.2575, 0.9562666666666667, 0.17215],
                [0.24250000000000002, 0.8729333333333332, 0.07785],
                [0.16440000000000002, 0.8764333333333334, 0.631],
                [0.0022666666666666664, 0.9597666666666668, 0.619],
                [0.16893333333333335, 0.9569, 0.869],
                [0.3310666666666667, 0.8735666666666666, 0.881],
                [0.13326666666666667, 0.8685166666666667, 0.64955],
                [0.0334, 0.95185, 0.60045],
                [0.20006666666666664, 0.9648166666666667, 0.8504499999999999],
                [0.29993333333333333, 0.8814833333333333, 0.8995500000000001],
                [0.10403333333333332, 0.8964833333333333, 0.65005],
                [0.06263333333333333, 0.9798166666666667, 0.59995],
                [0.22929999999999998, 0.9368500000000001, 0.84995],
                [0.2707, 0.8535166666666667, 0.90005],
                [0.0736, 0.8880333333333333, 0.66495],
                [0.09306666666666667, 0.9713666666666666, 0.58505],
                [0.2597333333333333, 0.9453, 0.8350500000000001],
                [0.24026666666666666, 0.8619666666666667, 0.9149499999999999],
                [0.07363333333333334, 0.8516, 0.67945],
                [0.09303333333333334, 0.9349333333333334, 0.57055],
                [0.2597, 0.9817333333333332, 0.82055],
                [0.24029999999999999, 0.8984, 0.92945],
                [0.10363333333333334, 0.9902166666666666, 0.68015],
                [0.06303333333333333, 0.9068833333333334, 0.56985],
                [0.22970000000000002, 0.8431166666666666, 0.81985],
                [0.2703, 0.92645, 0.93015],
                [0.1334, 0.9985333333333334, 0.66465],
                [0.03326666666666667, 0.9152, 0.58535],
                [0.19993333333333332, 0.8348, 0.83535],
                [0.30006666666666665, 0.9181333333333334, 0.91465],
                [0.049666666666666665, 0.8519833333333334, 0.7351],
                [0.11699999999999999, 0.9353166666666667, 0.5149],
                [0.2836666666666667, 0.98135, 0.7649],
                [0.21633333333333335, 0.8980166666666666, 0.9851],
                [0.0398, 0.8889999999999999, 0.74505],
                [0.12686666666666666, 0.9723333333333333, 0.50495],
                [0.29353333333333337, 0.9443333333333334, 0.75495],
                [0.20646666666666666, 0.8610000000000001, 0.99505],
                [0.0494, 0.8972833333333333, 0.7754],
                [0.11726666666666667, 0.9806166666666667, 0.9746],
                [0.2839333333333333, 0.9360499999999999, 0.7246],
                [0.21606666666666666, 0.8527166666666667, 0.5254],
                [0.06786666666666667, 0.8685666666666667, 0.7949999999999999],
                [0.0988, 0.9519000000000001, 0.9550000000000001],
                [0.2654666666666667, 0.9647666666666667, 0.705],
                [0.23453333333333334, 0.8814333333333333, 0.545],
                [0.0789, 0.9984166666666666, 0.7846500000000001],
                [0.08776666666666666, 0.9150833333333334, 0.9653499999999999],
                [0.25443333333333334, 0.8349166666666666, 0.71535],
                [0.24556666666666668, 0.91825, 0.53465],
                [0.06983333333333333, 0.9899666666666667, 0.7544],
                [0.09683333333333333, 0.9066333333333333, 0.9956],
                [0.2635, 0.8433666666666667, 0.7456],
                [0.23650000000000002, 0.9267, 0.5044],
                [0.07583333333333334, 0.8770666666666668, 0.82785],
                [0.09083333333333334, 0.9603999999999999, 0.92215],
                [0.2575, 0.9562666666666667, 0.67215],
                [0.24250000000000002, 0.8729333333333332, 0.57785],
                [0.49773333333333336, 0.0431, 0.131],
                [0.33559999999999995, 0.12643333333333334, 0.119],
                [0.5022666666666668, 0.12356666666666666, 0.369],
                [0.6644, 0.040233333333333336, 0.381],
                [0.46659999999999996, 0.03518333333333334, 0.14955],
                [0.36673333333333336, 0.11851666666666666, 0.10045],
                [0.5334, 0.13148333333333334, 0.35045],
                [0.6332666666666666, 0.04815, 0.39955],
                [0.4373666666666667, 0.06315, 0.15005],
                [0.39596666666666663, 0.14648333333333333, 0.09995],
                [0.5626333333333333, 0.10351666666666666, 0.34995],
                [0.6040333333333333, 0.020183333333333334, 0.40005],
                [0.40693333333333337, 0.0547, 0.16495],
                [0.42639999999999995, 0.13803333333333334, 0.08505],
                [0.5930666666666666, 0.11196666666666666, 0.33505],
                [0.5736, 0.028633333333333334, 0.41495],
                [0.4069666666666667, 0.018266666666666667, 0.17945],
                [0.4263666666666667, 0.10160000000000001, 0.07055],
                [0.5930333333333334, 0.1484, 0.32055],
                [0.5736333333333333, 0.06506666666666668, 0.42945],
                [0.43696666666666667, 0.15688333333333335, 0.18015],
                [0.3963666666666667, 0.07355, 0.06985],
                [0.5630333333333334, 0.009783333333333333, 0.31985],
                [0.6036333333333334, 0.09311666666666667, 0.43015],
                [0.4667333333333333, 0.16519999999999999, 0.16465],
                [0.36660000000000004, 0.08186666666666667, 0.08535],
                [0.5332666666666667, 0.0014666666666666667, 0.33535],
                [0.6334, 0.0848, 0.41465],
                [0.383, 0.01865, 0.2351],
                [0.4503333333333333, 0.10198333333333333, 0.0149],
                [0.617, 0.14801666666666666, 0.2649],
                [0.5496666666666666, 0.06468333333333333, 0.4851],
                [0.3731333333333333, 0.05566666666666667, 0.24505],
                [0.4602, 0.13899999999999998, 0.00495],
                [0.6268666666666667, 0.111, 0.25495],
                [0.5398, 0.02766666666666667, 0.49505],
                [0.38273333333333337, 0.06394999999999999, 0.2754],
                [0.45059999999999995, 0.14728333333333335, 0.4746],
                [0.6172666666666666, 0.10271666666666666, 0.2246],
                [0.5494, 0.019383333333333332, 0.0254],
                [0.4012, 0.03523333333333333, 0.295],
                [0.4321333333333333, 0.11856666666666667, 0.455],
                [0.5988, 0.13143333333333332, 0.205],
                [0.5678666666666666, 0.048100000000000004, 0.045],
                [0.4122333333333333, 0.16508333333333333, 0.28465],
                [0.42110000000000003, 0.08175, 0.46535],
                [0.5877666666666667, 0.0015833333333333333, 0.21535],
                [0.5789, 0.08491666666666665, 0.03465],
                [0.4031666666666667, 0.15663333333333332, 0.2544],
                [0.43016666666666664, 0.0733, 0.4956],
                [0.5968333333333333, 0.010033333333333333, 0.2456],
                [0.5698333333333333, 0.09336666666666667, 0.0044],
                [0.4091666666666667, 0.04373333333333334, 0.32785],
                [0.42416666666666664, 0.12706666666666666, 0.42215],
                [0.5908333333333333, 0.12293333333333334, 0.17215],
                [0.5758333333333333, 0.0396, 0.07785],
                [0.49773333333333336, 0.0431, 0.631],
                [0.33559999999999995, 0.12643333333333334, 0.619],
                [0.5022666666666668, 0.12356666666666666, 0.869],
                [0.6644, 0.040233333333333336, 0.881],
                [0.46659999999999996, 0.03518333333333334, 0.64955],
                [0.36673333333333336, 0.11851666666666666, 0.60045],
                [0.5334, 0.13148333333333334, 0.8504499999999999],
                [0.6332666666666666, 0.04815, 0.8995500000000001],
                [0.4373666666666667, 0.06315, 0.65005],
                [0.39596666666666663, 0.14648333333333333, 0.59995],
                [0.5626333333333333, 0.10351666666666666, 0.84995],
                [0.6040333333333333, 0.020183333333333334, 0.90005],
                [0.40693333333333337, 0.0547, 0.66495],
                [0.42639999999999995, 0.13803333333333334, 0.58505],
                [0.5930666666666666, 0.11196666666666666, 0.8350500000000001],
                [0.5736, 0.028633333333333334, 0.9149499999999999],
                [0.4069666666666667, 0.018266666666666667, 0.67945],
                [0.4263666666666667, 0.10160000000000001, 0.57055],
                [0.5930333333333334, 0.1484, 0.82055],
                [0.5736333333333333, 0.06506666666666668, 0.92945],
                [0.43696666666666667, 0.15688333333333335, 0.68015],
                [0.3963666666666667, 0.07355, 0.56985],
                [0.5630333333333334, 0.009783333333333333, 0.81985],
                [0.6036333333333334, 0.09311666666666667, 0.93015],
                [0.4667333333333333, 0.16519999999999999, 0.66465],
                [0.36660000000000004, 0.08186666666666667, 0.58535],
                [0.5332666666666667, 0.0014666666666666667, 0.83535],
                [0.6334, 0.0848, 0.91465],
                [0.383, 0.01865, 0.7351],
                [0.4503333333333333, 0.10198333333333333, 0.5149],
                [0.617, 0.14801666666666666, 0.7649],
                [0.5496666666666666, 0.06468333333333333, 0.9851],
                [0.3731333333333333, 0.05566666666666667, 0.74505],
                [0.4602, 0.13899999999999998, 0.50495],
                [0.6268666666666667, 0.111, 0.75495],
                [0.5398, 0.02766666666666667, 0.99505],
                [0.38273333333333337, 0.06394999999999999, 0.7754],
                [0.45059999999999995, 0.14728333333333335, 0.9746],
                [0.6172666666666666, 0.10271666666666666, 0.7246],
                [0.5494, 0.019383333333333332, 0.5254],
                [0.4012, 0.03523333333333333, 0.7949999999999999],
                [0.4321333333333333, 0.11856666666666667, 0.9550000000000001],
                [0.5988, 0.13143333333333332, 0.705],
                [0.5678666666666666, 0.048100000000000004, 0.545],
                [0.4122333333333333, 0.16508333333333333, 0.7846500000000001],
                [0.42110000000000003, 0.08175, 0.9653499999999999],
                [0.5877666666666667, 0.0015833333333333333, 0.71535],
                [0.5789, 0.08491666666666665, 0.53465],
                [0.4031666666666667, 0.15663333333333332, 0.7544],
                [0.43016666666666664, 0.0733, 0.9956],
                [0.5968333333333333, 0.010033333333333333, 0.7456],
                [0.5698333333333333, 0.09336666666666667, 0.5044],
                [0.4091666666666667, 0.04373333333333334, 0.82785],
                [0.42416666666666664, 0.12706666666666666, 0.92215],
                [0.5908333333333333, 0.12293333333333334, 0.67215],
                [0.5758333333333333, 0.0396, 0.57785],
                [0.49773333333333336, 0.20976666666666666, 0.131],
                [0.33559999999999995, 0.29309999999999997, 0.119],
                [0.5022666666666668, 0.29023333333333334, 0.369],
                [0.6644, 0.2069, 0.381],
                [0.46659999999999996, 0.20185, 0.14955],
                [0.36673333333333336, 0.28518333333333334, 0.10045],
                [0.5334, 0.29814999999999997, 0.35045],
                [0.6332666666666666, 0.21481666666666666, 0.39955],
                [0.4373666666666667, 0.22981666666666667, 0.15005],
                [0.39596666666666663, 0.31315, 0.09995],
                [0.5626333333333333, 0.27018333333333333, 0.34995],
                [0.6040333333333333, 0.18685, 0.40005],
                [0.40693333333333337, 0.22136666666666668, 0.16495],
                [0.42639999999999995, 0.3047, 0.08505],
                [0.5930666666666666, 0.27863333333333334, 0.33505],
                [0.5736, 0.1953, 0.41495],
                [0.4069666666666667, 0.1849333333333333, 0.17945],
                [0.4263666666666667, 0.26826666666666665, 0.07055],
                [0.5930333333333334, 0.31506666666666666, 0.32055],
                [0.5736333333333333, 0.23173333333333335, 0.42945],
                [0.43696666666666667, 0.32355, 0.18015],
                [0.3963666666666667, 0.24021666666666666, 0.06985],
                [0.5630333333333334, 0.17645, 0.31985],
                [0.6036333333333334, 0.2597833333333333, 0.43015],
                [0.4667333333333333, 0.3318666666666667, 0.16465],
                [0.36660000000000004, 0.24853333333333336, 0.08535],
                [0.5332666666666667, 0.16813333333333333, 0.33535],
                [0.6334, 0.25146666666666667, 0.41465],
                [0.383, 0.18531666666666666, 0.2351],
                [0.4503333333333333, 0.26865, 0.0149],
                [0.617, 0.31468333333333337, 0.2649],
                [0.5496666666666666, 0.23135000000000003, 0.4851],
                [0.3731333333333333, 0.22233333333333336, 0.24505],
                [0.4602, 0.3056666666666667, 0.00495],
                [0.6268666666666667, 0.2776666666666667, 0.25495],
                [0.5398, 0.19433333333333333, 0.49505],
                [0.38273333333333337, 0.23061666666666666, 0.2754],
                [0.45059999999999995, 0.31395, 0.4746],
                [0.6172666666666666, 0.2693833333333333, 0.2246],
                [0.5494, 0.18605000000000002, 0.0254],
                [0.4012, 0.2019, 0.295],
                [0.4321333333333333, 0.28523333333333334, 0.455],
                [0.5988, 0.2981, 0.205],
                [0.5678666666666666, 0.21476666666666666, 0.045],
                [0.4122333333333333, 0.33175, 0.28465],
                [0.42110000000000003, 0.24841666666666665, 0.46535],
                [0.5877666666666667, 0.16825, 0.21535],
                [0.5789, 0.2515833333333333, 0.03465],
                [0.4031666666666667, 0.3233, 0.2544],
                [0.43016666666666664, 0.23996666666666666, 0.4956],
                [0.5968333333333333, 0.1767, 0.2456],
                [0.5698333333333333, 0.26003333333333334, 0.0044],
                [0.4091666666666667, 0.2104, 0.32785],
                [0.42416666666666664, 0.29373333333333335, 0.42215],
                [0.5908333333333333, 0.2896, 0.17215],
                [0.5758333333333333, 0.20626666666666668, 0.07785],
                [0.49773333333333336, 0.20976666666666666, 0.631],
                [0.33559999999999995, 0.29309999999999997, 0.619],
                [0.5022666666666668, 0.29023333333333334, 0.869],
                [0.6644, 0.2069, 0.881],
                [0.46659999999999996, 0.20185, 0.64955],
                [0.36673333333333336, 0.28518333333333334, 0.60045],
                [0.5334, 0.29814999999999997, 0.8504499999999999],
                [0.6332666666666666, 0.21481666666666666, 0.8995500000000001],
                [0.4373666666666667, 0.22981666666666667, 0.65005],
                [0.39596666666666663, 0.31315, 0.59995],
                [0.5626333333333333, 0.27018333333333333, 0.84995],
                [0.6040333333333333, 0.18685, 0.90005],
                [0.40693333333333337, 0.22136666666666668, 0.66495],
                [0.42639999999999995, 0.3047, 0.58505],
                [0.5930666666666666, 0.27863333333333334, 0.8350500000000001],
                [0.5736, 0.1953, 0.9149499999999999],
                [0.4069666666666667, 0.1849333333333333, 0.67945],
                [0.4263666666666667, 0.26826666666666665, 0.57055],
                [0.5930333333333334, 0.31506666666666666, 0.82055],
                [0.5736333333333333, 0.23173333333333335, 0.92945],
                [0.43696666666666667, 0.32355, 0.68015],
                [0.3963666666666667, 0.24021666666666666, 0.56985],
                [0.5630333333333334, 0.17645, 0.81985],
                [0.6036333333333334, 0.2597833333333333, 0.93015],
                [0.4667333333333333, 0.3318666666666667, 0.66465],
                [0.36660000000000004, 0.24853333333333336, 0.58535],
                [0.5332666666666667, 0.16813333333333333, 0.83535],
                [0.6334, 0.25146666666666667, 0.91465],
                [0.383, 0.18531666666666666, 0.7351],
                [0.4503333333333333, 0.26865, 0.5149],
                [0.617, 0.31468333333333337, 0.7649],
                [0.5496666666666666, 0.23135000000000003, 0.9851],
                [0.3731333333333333, 0.22233333333333336, 0.74505],
                [0.4602, 0.3056666666666667, 0.50495],
                [0.6268666666666667, 0.2776666666666667, 0.75495],
                [0.5398, 0.19433333333333333, 0.99505],
                [0.38273333333333337, 0.23061666666666666, 0.7754],
                [0.45059999999999995, 0.31395, 0.9746],
                [0.6172666666666666, 0.2693833333333333, 0.7246],
                [0.5494, 0.18605000000000002, 0.5254],
                [0.4012, 0.2019, 0.7949999999999999],
                [0.4321333333333333, 0.28523333333333334, 0.9550000000000001],
                [0.5988, 0.2981, 0.705],
                [0.5678666666666666, 0.21476666666666666, 0.545],
                [0.4122333333333333, 0.33175, 0.7846500000000001],
                [0.42110000000000003, 0.24841666666666665, 0.9653499999999999],
                [0.5877666666666667, 0.16825, 0.71535],
                [0.5789, 0.2515833333333333, 0.53465],
                [0.4031666666666667, 0.3233, 0.7544],
                [0.43016666666666664, 0.23996666666666666, 0.9956],
                [0.5968333333333333, 0.1767, 0.7456],
                [0.5698333333333333, 0.26003333333333334, 0.5044],
                [0.4091666666666667, 0.2104, 0.82785],
                [0.42416666666666664, 0.29373333333333335, 0.92215],
                [0.5908333333333333, 0.2896, 0.67215],
                [0.5758333333333333, 0.20626666666666668, 0.57785],
                [0.49773333333333336, 0.37643333333333334, 0.131],
                [0.33559999999999995, 0.45976666666666666, 0.119],
                [0.5022666666666668, 0.45690000000000003, 0.369],
                [0.6644, 0.37356666666666666, 0.381],
                [0.46659999999999996, 0.36851666666666666, 0.14955],
                [0.36673333333333336, 0.45185000000000003, 0.10045],
                [0.5334, 0.46481666666666666, 0.35045],
                [0.6332666666666666, 0.38148333333333334, 0.39955],
                [0.4373666666666667, 0.3964833333333333, 0.15005],
                [0.39596666666666663, 0.4798166666666666, 0.09995],
                [0.5626333333333333, 0.43685, 0.34995],
                [0.6040333333333333, 0.3535166666666667, 0.40005],
                [0.40693333333333337, 0.3880333333333333, 0.16495],
                [0.42639999999999995, 0.47136666666666666, 0.08505],
                [0.5930666666666666, 0.44530000000000003, 0.33505],
                [0.5736, 0.3619666666666667, 0.41495],
                [0.4069666666666667, 0.35159999999999997, 0.17945],
                [0.4263666666666667, 0.43493333333333334, 0.07055],
                [0.5930333333333334, 0.48173333333333335, 0.32055],
                [0.5736333333333333, 0.39840000000000003, 0.42945],
                [0.43696666666666667, 0.4902166666666667, 0.18015],
                [0.3963666666666667, 0.4068833333333333, 0.06985],
                [0.5630333333333334, 0.3431166666666667, 0.31985],
                [0.6036333333333334, 0.42645, 0.43015],
                [0.4667333333333333, 0.49853333333333333, 0.16465],
                [0.36660000000000004, 0.4152, 0.08535],
                [0.5332666666666667, 0.3348, 0.33535],
                [0.6334, 0.4181333333333333, 0.41465],
                [0.383, 0.3519833333333333, 0.2351],
                [0.4503333333333333, 0.43531666666666663, 0.0149],
                [0.617, 0.48135, 0.2649],
                [0.5496666666666666, 0.3980166666666667, 0.4851],
                [0.3731333333333333, 0.389, 0.24505],
                [0.4602, 0.4723333333333333, 0.00495],
                [0.6268666666666667, 0.4443333333333333, 0.25495],
                [0.5398, 0.361, 0.49505],
                [0.38273333333333337, 0.3972833333333334, 0.2754],
                [0.45059999999999995, 0.4806166666666667, 0.4746],
                [0.6172666666666666, 0.43605, 0.2246],
                [0.5494, 0.3527166666666666, 0.0254],
                [0.4012, 0.36856666666666665, 0.295],
                [0.4321333333333333, 0.4519, 0.455],
                [0.5988, 0.4647666666666666, 0.205],
                [0.5678666666666666, 0.38143333333333335, 0.045],
                [0.4122333333333333, 0.4984166666666667, 0.28465],
                [0.42110000000000003, 0.4150833333333333, 0.46535],
                [0.5877666666666667, 0.3349166666666667, 0.21535],
                [0.5789, 0.41825, 0.03465],
                [0.4031666666666667, 0.48996666666666666, 0.2544],
                [0.43016666666666664, 0.40663333333333335, 0.4956],
                [0.5968333333333333, 0.34336666666666665, 0.2456],
                [0.5698333333333333, 0.4267, 0.0044],
                [0.4091666666666667, 0.37706666666666666, 0.32785],
                [0.42416666666666664, 0.4604, 0.42215],
                [0.5908333333333333, 0.45626666666666665, 0.17215],
                [0.5758333333333333, 0.37293333333333334, 0.07785],
                [0.49773333333333336, 0.37643333333333334, 0.631],
                [0.33559999999999995, 0.45976666666666666, 0.619],
                [0.5022666666666668, 0.45690000000000003, 0.869],
                [0.6644, 0.37356666666666666, 0.881],
                [0.46659999999999996, 0.36851666666666666, 0.64955],
                [0.36673333333333336, 0.45185000000000003, 0.60045],
                [0.5334, 0.46481666666666666, 0.8504499999999999],
                [0.6332666666666666, 0.38148333333333334, 0.8995500000000001],
                [0.4373666666666667, 0.3964833333333333, 0.65005],
                [0.39596666666666663, 0.4798166666666666, 0.59995],
                [0.5626333333333333, 0.43685, 0.84995],
                [0.6040333333333333, 0.3535166666666667, 0.90005],
                [0.40693333333333337, 0.3880333333333333, 0.66495],
                [0.42639999999999995, 0.47136666666666666, 0.58505],
                [0.5930666666666666, 0.44530000000000003, 0.8350500000000001],
                [0.5736, 0.3619666666666667, 0.9149499999999999],
                [0.4069666666666667, 0.35159999999999997, 0.67945],
                [0.4263666666666667, 0.43493333333333334, 0.57055],
                [0.5930333333333334, 0.48173333333333335, 0.82055],
                [0.5736333333333333, 0.39840000000000003, 0.92945],
                [0.43696666666666667, 0.4902166666666667, 0.68015],
                [0.3963666666666667, 0.4068833333333333, 0.56985],
                [0.5630333333333334, 0.3431166666666667, 0.81985],
                [0.6036333333333334, 0.42645, 0.93015],
                [0.4667333333333333, 0.49853333333333333, 0.66465],
                [0.36660000000000004, 0.4152, 0.58535],
                [0.5332666666666667, 0.3348, 0.83535],
                [0.6334, 0.4181333333333333, 0.91465],
                [0.383, 0.3519833333333333, 0.7351],
                [0.4503333333333333, 0.43531666666666663, 0.5149],
                [0.617, 0.48135, 0.7649],
                [0.5496666666666666, 0.3980166666666667, 0.9851],
                [0.3731333333333333, 0.389, 0.74505],
                [0.4602, 0.4723333333333333, 0.50495],
                [0.6268666666666667, 0.4443333333333333, 0.75495],
                [0.5398, 0.361, 0.99505],
                [0.38273333333333337, 0.3972833333333334, 0.7754],
                [0.45059999999999995, 0.4806166666666667, 0.9746],
                [0.6172666666666666, 0.43605, 0.7246],
                [0.5494, 0.3527166666666666, 0.5254],
                [0.4012, 0.36856666666666665, 0.7949999999999999],
                [0.4321333333333333, 0.4519, 0.9550000000000001],
                [0.5988, 0.4647666666666666, 0.705],
                [0.5678666666666666, 0.38143333333333335, 0.545],
                [0.4122333333333333, 0.4984166666666667, 0.7846500000000001],
                [0.42110000000000003, 0.4150833333333333, 0.9653499999999999],
                [0.5877666666666667, 0.3349166666666667, 0.71535],
                [0.5789, 0.41825, 0.53465],
                [0.4031666666666667, 0.48996666666666666, 0.7544],
                [0.43016666666666664, 0.40663333333333335, 0.9956],
                [0.5968333333333333, 0.34336666666666665, 0.7456],
                [0.5698333333333333, 0.4267, 0.5044],
                [0.4091666666666667, 0.37706666666666666, 0.82785],
                [0.42416666666666664, 0.4604, 0.92215],
                [0.5908333333333333, 0.45626666666666665, 0.67215],
                [0.5758333333333333, 0.37293333333333334, 0.57785],
                [0.49773333333333336, 0.5431, 0.131],
                [0.33559999999999995, 0.6264333333333333, 0.119],
                [0.5022666666666668, 0.6235666666666667, 0.369],
                [0.6644, 0.5402333333333333, 0.381],
                [0.46659999999999996, 0.5351833333333333, 0.14955],
                [0.36673333333333336, 0.6185166666666667, 0.10045],
                [0.5334, 0.6314833333333333, 0.35045],
                [0.6332666666666666, 0.54815, 0.39955],
                [0.4373666666666667, 0.5631499999999999, 0.15005],
                [0.39596666666666663, 0.6464833333333333, 0.09995],
                [0.5626333333333333, 0.6035166666666667, 0.34995],
                [0.6040333333333333, 0.5201833333333333, 0.40005],
                [0.40693333333333337, 0.5547, 0.16495],
                [0.42639999999999995, 0.6380333333333333, 0.08505],
                [0.5930666666666666, 0.6119666666666667, 0.33505],
                [0.5736, 0.5286333333333334, 0.41495],
                [0.4069666666666667, 0.5182666666666667, 0.17945],
                [0.4263666666666667, 0.6016, 0.07055],
                [0.5930333333333334, 0.6484, 0.32055],
                [0.5736333333333333, 0.5650666666666667, 0.42945],
                [0.43696666666666667, 0.6568833333333334, 0.18015],
                [0.3963666666666667, 0.57355, 0.06985],
                [0.5630333333333334, 0.5097833333333334, 0.31985],
                [0.6036333333333334, 0.5931166666666666, 0.43015],
                [0.4667333333333333, 0.6652, 0.16465],
                [0.36660000000000004, 0.5818666666666666, 0.08535],
                [0.5332666666666667, 0.5014666666666666, 0.33535],
                [0.6334, 0.5848, 0.41465],
                [0.383, 0.5186499999999999, 0.2351],
                [0.4503333333333333, 0.6019833333333333, 0.0149],
                [0.617, 0.6480166666666667, 0.2649],
                [0.5496666666666666, 0.5646833333333333, 0.4851],
                [0.3731333333333333, 0.5556666666666666, 0.24505],
                [0.4602, 0.639, 0.00495],
                [0.6268666666666667, 0.611, 0.25495],
                [0.5398, 0.5276666666666666, 0.49505],
                [0.38273333333333337, 0.5639500000000001, 0.2754],
                [0.45059999999999995, 0.6472833333333333, 0.4746],
                [0.6172666666666666, 0.6027166666666667, 0.2246],
                [0.5494, 0.5193833333333333, 0.0254],
                [0.4012, 0.5352333333333333, 0.295],
                [0.4321333333333333, 0.6185666666666667, 0.455],
                [0.5988, 0.6314333333333333, 0.205],
                [0.5678666666666666, 0.5481, 0.045],
                [0.4122333333333333, 0.6650833333333334, 0.28465],
                [0.42110000000000003, 0.58175, 0.46535],
                [0.5877666666666667, 0.5015833333333334, 0.21535],
                [0.5789, 0.5849166666666666, 0.03465],
                [0.4031666666666667, 0.6566333333333333, 0.2544],
                [0.43016666666666664, 0.5733, 0.4956],
                [0.5968333333333333, 0.5100333333333333, 0.2456],
                [0.5698333333333333, 0.5933666666666667, 0.0044],
                [0.4091666666666667, 0.5437333333333333, 0.32785],
                [0.42416666666666664, 0.6270666666666667, 0.42215],
                [0.5908333333333333, 0.6229333333333333, 0.17215],
                [0.5758333333333333, 0.5396, 0.07785],
                [0.49773333333333336, 0.5431, 0.631],
                [0.33559999999999995, 0.6264333333333333, 0.619],
                [0.5022666666666668, 0.6235666666666667, 0.869],
                [0.6644, 0.5402333333333333, 0.881],
                [0.46659999999999996, 0.5351833333333333, 0.64955],
                [0.36673333333333336, 0.6185166666666667, 0.60045],
                [0.5334, 0.6314833333333333, 0.8504499999999999],
                [0.6332666666666666, 0.54815, 0.8995500000000001],
                [0.4373666666666667, 0.5631499999999999, 0.65005],
                [0.39596666666666663, 0.6464833333333333, 0.59995],
                [0.5626333333333333, 0.6035166666666667, 0.84995],
                [0.6040333333333333, 0.5201833333333333, 0.90005],
                [0.40693333333333337, 0.5547, 0.66495],
                [0.42639999999999995, 0.6380333333333333, 0.58505],
                [0.5930666666666666, 0.6119666666666667, 0.8350500000000001],
                [0.5736, 0.5286333333333334, 0.9149499999999999],
                [0.4069666666666667, 0.5182666666666667, 0.67945],
                [0.4263666666666667, 0.6016, 0.57055],
                [0.5930333333333334, 0.6484, 0.82055],
                [0.5736333333333333, 0.5650666666666667, 0.92945],
                [0.43696666666666667, 0.6568833333333334, 0.68015],
                [0.3963666666666667, 0.57355, 0.56985],
                [0.5630333333333334, 0.5097833333333334, 0.81985],
                [0.6036333333333334, 0.5931166666666666, 0.93015],
                [0.4667333333333333, 0.6652, 0.66465],
                [0.36660000000000004, 0.5818666666666666, 0.58535],
                [0.5332666666666667, 0.5014666666666666, 0.83535],
                [0.6334, 0.5848, 0.91465],
                [0.383, 0.5186499999999999, 0.7351],
                [0.4503333333333333, 0.6019833333333333, 0.5149],
                [0.617, 0.6480166666666667, 0.7649],
                [0.5496666666666666, 0.5646833333333333, 0.9851],
                [0.3731333333333333, 0.5556666666666666, 0.74505],
                [0.4602, 0.639, 0.50495],
                [0.6268666666666667, 0.611, 0.75495],
                [0.5398, 0.5276666666666666, 0.99505],
                [0.38273333333333337, 0.5639500000000001, 0.7754],
                [0.45059999999999995, 0.6472833333333333, 0.9746],
                [0.6172666666666666, 0.6027166666666667, 0.7246],
                [0.5494, 0.5193833333333333, 0.5254],
                [0.4012, 0.5352333333333333, 0.7949999999999999],
                [0.4321333333333333, 0.6185666666666667, 0.9550000000000001],
                [0.5988, 0.6314333333333333, 0.705],
                [0.5678666666666666, 0.5481, 0.545],
                [0.4122333333333333, 0.6650833333333334, 0.7846500000000001],
                [0.42110000000000003, 0.58175, 0.9653499999999999],
                [0.5877666666666667, 0.5015833333333334, 0.71535],
                [0.5789, 0.5849166666666666, 0.53465],
                [0.4031666666666667, 0.6566333333333333, 0.7544],
                [0.43016666666666664, 0.5733, 0.9956],
                [0.5968333333333333, 0.5100333333333333, 0.7456],
                [0.5698333333333333, 0.5933666666666667, 0.5044],
                [0.4091666666666667, 0.5437333333333333, 0.82785],
                [0.42416666666666664, 0.6270666666666667, 0.92215],
                [0.5908333333333333, 0.6229333333333333, 0.67215],
                [0.5758333333333333, 0.5396, 0.57785],
                [0.49773333333333336, 0.7097666666666668, 0.131],
                [0.33559999999999995, 0.7931, 0.119],
                [0.5022666666666668, 0.7902333333333332, 0.369],
                [0.6644, 0.7069, 0.381],
                [0.46659999999999996, 0.70185, 0.14955],
                [0.36673333333333336, 0.7851833333333333, 0.10045],
                [0.5334, 0.79815, 0.35045],
                [0.6332666666666666, 0.7148166666666667, 0.39955],
                [0.4373666666666667, 0.7298166666666667, 0.15005],
                [0.39596666666666663, 0.8131499999999999, 0.09995],
                [0.5626333333333333, 0.7701833333333333, 0.34995],
                [0.6040333333333333, 0.6868500000000001, 0.40005],
                [0.40693333333333337, 0.7213666666666666, 0.16495],
                [0.42639999999999995, 0.8047, 0.08505],
                [0.5930666666666666, 0.7786333333333334, 0.33505],
                [0.5736, 0.6953, 0.41495],
                [0.4069666666666667, 0.6849333333333334, 0.17945],
                [0.4263666666666667, 0.7682666666666668, 0.07055],
                [0.5930333333333334, 0.8150666666666666, 0.32055],
                [0.5736333333333333, 0.7317333333333332, 0.42945],
                [0.43696666666666667, 0.82355, 0.18015],
                [0.3963666666666667, 0.7402166666666666, 0.06985],
                [0.5630333333333334, 0.67645, 0.31985],
                [0.6036333333333334, 0.7597833333333334, 0.43015],
                [0.4667333333333333, 0.8318666666666666, 0.16465],
                [0.36660000000000004, 0.7485333333333334, 0.08535],
                [0.5332666666666667, 0.6681333333333334, 0.33535],
                [0.6334, 0.7514666666666666, 0.41465],
                [0.383, 0.6853166666666667, 0.2351],
                [0.4503333333333333, 0.76865, 0.0149],
                [0.617, 0.8146833333333333, 0.2649],
                [0.5496666666666666, 0.73135, 0.4851],
                [0.3731333333333333, 0.7223333333333333, 0.24505],
                [0.4602, 0.8056666666666666, 0.00495],
                [0.6268666666666667, 0.7776666666666667, 0.25495],
                [0.5398, 0.6943333333333334, 0.49505],
                [0.38273333333333337, 0.7306166666666667, 0.2754],
                [0.45059999999999995, 0.8139500000000001, 0.4746],
                [0.6172666666666666, 0.7693833333333333, 0.2246],
                [0.5494, 0.6860499999999999, 0.0254],
                [0.4012, 0.7019000000000001, 0.295],
                [0.4321333333333333, 0.7852333333333333, 0.455],
                [0.5988, 0.7980999999999999, 0.205],
                [0.5678666666666666, 0.7147666666666667, 0.045],
                [0.4122333333333333, 0.83175, 0.28465],
                [0.42110000000000003, 0.7484166666666666, 0.46535],
                [0.5877666666666667, 0.66825, 0.21535],
                [0.5789, 0.7515833333333334, 0.03465],
                [0.4031666666666667, 0.8233, 0.2544],
                [0.43016666666666664, 0.7399666666666667, 0.4956],
                [0.5968333333333333, 0.6767, 0.2456],
                [0.5698333333333333, 0.7600333333333333, 0.0044],
                [0.4091666666666667, 0.7104, 0.32785],
                [0.42416666666666664, 0.7937333333333333, 0.42215],
                [0.5908333333333333, 0.7896000000000001, 0.17215],
                [0.5758333333333333, 0.7062666666666666, 0.07785],
                [0.49773333333333336, 0.7097666666666668, 0.631],
                [0.33559999999999995, 0.7931, 0.619],
                [0.5022666666666668, 0.7902333333333332, 0.869],
                [0.6644, 0.7069, 0.881],
                [0.46659999999999996, 0.70185, 0.64955],
                [0.36673333333333336, 0.7851833333333333, 0.60045],
                [0.5334, 0.79815, 0.8504499999999999],
                [0.6332666666666666, 0.7148166666666667, 0.8995500000000001],
                [0.4373666666666667, 0.7298166666666667, 0.65005],
                [0.39596666666666663, 0.8131499999999999, 0.59995],
                [0.5626333333333333, 0.7701833333333333, 0.84995],
                [0.6040333333333333, 0.6868500000000001, 0.90005],
                [0.40693333333333337, 0.7213666666666666, 0.66495],
                [0.42639999999999995, 0.8047, 0.58505],
                [0.5930666666666666, 0.7786333333333334, 0.8350500000000001],
                [0.5736, 0.6953, 0.9149499999999999],
                [0.4069666666666667, 0.6849333333333334, 0.67945],
                [0.4263666666666667, 0.7682666666666668, 0.57055],
                [0.5930333333333334, 0.8150666666666666, 0.82055],
                [0.5736333333333333, 0.7317333333333332, 0.92945],
                [0.43696666666666667, 0.82355, 0.68015],
                [0.3963666666666667, 0.7402166666666666, 0.56985],
                [0.5630333333333334, 0.67645, 0.81985],
                [0.6036333333333334, 0.7597833333333334, 0.93015],
                [0.4667333333333333, 0.8318666666666666, 0.66465],
                [0.36660000000000004, 0.7485333333333334, 0.58535],
                [0.5332666666666667, 0.6681333333333334, 0.83535],
                [0.6334, 0.7514666666666666, 0.91465],
                [0.383, 0.6853166666666667, 0.7351],
                [0.4503333333333333, 0.76865, 0.5149],
                [0.617, 0.8146833333333333, 0.7649],
                [0.5496666666666666, 0.73135, 0.9851],
                [0.3731333333333333, 0.7223333333333333, 0.74505],
                [0.4602, 0.8056666666666666, 0.50495],
                [0.6268666666666667, 0.7776666666666667, 0.75495],
                [0.5398, 0.6943333333333334, 0.99505],
                [0.38273333333333337, 0.7306166666666667, 0.7754],
                [0.45059999999999995, 0.8139500000000001, 0.9746],
                [0.6172666666666666, 0.7693833333333333, 0.7246],
                [0.5494, 0.6860499999999999, 0.5254],
                [0.4012, 0.7019000000000001, 0.7949999999999999],
                [0.4321333333333333, 0.7852333333333333, 0.9550000000000001],
                [0.5988, 0.7980999999999999, 0.705],
                [0.5678666666666666, 0.7147666666666667, 0.545],
                [0.4122333333333333, 0.83175, 0.7846500000000001],
                [0.42110000000000003, 0.7484166666666666, 0.9653499999999999],
                [0.5877666666666667, 0.66825, 0.71535],
                [0.5789, 0.7515833333333334, 0.53465],
                [0.4031666666666667, 0.8233, 0.7544],
                [0.43016666666666664, 0.7399666666666667, 0.9956],
                [0.5968333333333333, 0.6767, 0.7456],
                [0.5698333333333333, 0.7600333333333333, 0.5044],
                [0.4091666666666667, 0.7104, 0.82785],
                [0.42416666666666664, 0.7937333333333333, 0.92215],
                [0.5908333333333333, 0.7896000000000001, 0.67215],
                [0.5758333333333333, 0.7062666666666666, 0.57785],
                [0.49773333333333336, 0.8764333333333334, 0.131],
                [0.33559999999999995, 0.9597666666666668, 0.119],
                [0.5022666666666668, 0.9569, 0.369],
                [0.6644, 0.8735666666666666, 0.381],
                [0.46659999999999996, 0.8685166666666667, 0.14955],
                [0.36673333333333336, 0.95185, 0.10045],
                [0.5334, 0.9648166666666667, 0.35045],
                [0.6332666666666666, 0.8814833333333333, 0.39955],
                [0.4373666666666667, 0.8964833333333333, 0.15005],
                [0.39596666666666663, 0.9798166666666667, 0.09995],
                [0.5626333333333333, 0.9368500000000001, 0.34995],
                [0.6040333333333333, 0.8535166666666667, 0.40005],
                [0.40693333333333337, 0.8880333333333333, 0.16495],
                [0.42639999999999995, 0.9713666666666666, 0.08505],
                [0.5930666666666666, 0.9453, 0.33505],
                [0.5736, 0.8619666666666667, 0.41495],
                [0.4069666666666667, 0.8516, 0.17945],
                [0.4263666666666667, 0.9349333333333334, 0.07055],
                [0.5930333333333334, 0.9817333333333332, 0.32055],
                [0.5736333333333333, 0.8984, 0.42945],
                [0.43696666666666667, 0.9902166666666666, 0.18015],
                [0.3963666666666667, 0.9068833333333334, 0.06985],
                [0.5630333333333334, 0.8431166666666666, 0.31985],
                [0.6036333333333334, 0.92645, 0.43015],
                [0.4667333333333333, 0.9985333333333334, 0.16465],
                [0.36660000000000004, 0.9152, 0.08535],
                [0.5332666666666667, 0.8348, 0.33535],
                [0.6334, 0.9181333333333334, 0.41465],
                [0.383, 0.8519833333333334, 0.2351],
                [0.4503333333333333, 0.9353166666666667, 0.0149],
                [0.617, 0.98135, 0.2649],
                [0.5496666666666666, 0.8980166666666666, 0.4851],
                [0.3731333333333333, 0.8889999999999999, 0.24505],
                [0.4602, 0.9723333333333333, 0.00495],
                [0.6268666666666667, 0.9443333333333334, 0.25495],
                [0.5398, 0.8610000000000001, 0.49505],
                [0.38273333333333337, 0.8972833333333333, 0.2754],
                [0.45059999999999995, 0.9806166666666667, 0.4746],
                [0.6172666666666666, 0.9360499999999999, 0.2246],
                [0.5494, 0.8527166666666667, 0.0254],
                [0.4012, 0.8685666666666667, 0.295],
                [0.4321333333333333, 0.9519000000000001, 0.455],
                [0.5988, 0.9647666666666667, 0.205],
                [0.5678666666666666, 0.8814333333333333, 0.045],
                [0.4122333333333333, 0.9984166666666666, 0.28465],
                [0.42110000000000003, 0.9150833333333334, 0.46535],
                [0.5877666666666667, 0.8349166666666666, 0.21535],
                [0.5789, 0.91825, 0.03465],
                [0.4031666666666667, 0.9899666666666667, 0.2544],
                [0.43016666666666664, 0.9066333333333333, 0.4956],
                [0.5968333333333333, 0.8433666666666667, 0.2456],
                [0.5698333333333333, 0.9267, 0.0044],
                [0.4091666666666667, 0.8770666666666668, 0.32785],
                [0.42416666666666664, 0.9603999999999999, 0.42215],
                [0.5908333333333333, 0.9562666666666667, 0.17215],
                [0.5758333333333333, 0.8729333333333332, 0.07785],
                [0.49773333333333336, 0.8764333333333334, 0.631],
                [0.33559999999999995, 0.9597666666666668, 0.619],
                [0.5022666666666668, 0.9569, 0.869],
                [0.6644, 0.8735666666666666, 0.881],
                [0.46659999999999996, 0.8685166666666667, 0.64955],
                [0.36673333333333336, 0.95185, 0.60045],
                [0.5334, 0.9648166666666667, 0.8504499999999999],
                [0.6332666666666666, 0.8814833333333333, 0.8995500000000001],
                [0.4373666666666667, 0.8964833333333333, 0.65005],
                [0.39596666666666663, 0.9798166666666667, 0.59995],
                [0.5626333333333333, 0.9368500000000001, 0.84995],
                [0.6040333333333333, 0.8535166666666667, 0.90005],
                [0.40693333333333337, 0.8880333333333333, 0.66495],
                [0.42639999999999995, 0.9713666666666666, 0.58505],
                [0.5930666666666666, 0.9453, 0.8350500000000001],
                [0.5736, 0.8619666666666667, 0.9149499999999999],
                [0.4069666666666667, 0.8516, 0.67945],
                [0.4263666666666667, 0.9349333333333334, 0.57055],
                [0.5930333333333334, 0.9817333333333332, 0.82055],
                [0.5736333333333333, 0.8984, 0.92945],
                [0.43696666666666667, 0.9902166666666666, 0.68015],
                [0.3963666666666667, 0.9068833333333334, 0.56985],
                [0.5630333333333334, 0.8431166666666666, 0.81985],
                [0.6036333333333334, 0.92645, 0.93015],
                [0.4667333333333333, 0.9985333333333334, 0.66465],
                [0.36660000000000004, 0.9152, 0.58535],
                [0.5332666666666667, 0.8348, 0.83535],
                [0.6334, 0.9181333333333334, 0.91465],
                [0.383, 0.8519833333333334, 0.7351],
                [0.4503333333333333, 0.9353166666666667, 0.5149],
                [0.617, 0.98135, 0.7649],
                [0.5496666666666666, 0.8980166666666666, 0.9851],
                [0.3731333333333333, 0.8889999999999999, 0.74505],
                [0.4602, 0.9723333333333333, 0.50495],
                [0.6268666666666667, 0.9443333333333334, 0.75495],
                [0.5398, 0.8610000000000001, 0.99505],
                [0.38273333333333337, 0.8972833333333333, 0.7754],
                [0.45059999999999995, 0.9806166666666667, 0.9746],
                [0.6172666666666666, 0.9360499999999999, 0.7246],
                [0.5494, 0.8527166666666667, 0.5254],
                [0.4012, 0.8685666666666667, 0.7949999999999999],
                [0.4321333333333333, 0.9519000000000001, 0.9550000000000001],
                [0.5988, 0.9647666666666667, 0.705],
                [0.5678666666666666, 0.8814333333333333, 0.545],
                [0.4122333333333333, 0.9984166666666666, 0.7846500000000001],
                [0.42110000000000003, 0.9150833333333334, 0.9653499999999999],
                [0.5877666666666667, 0.8349166666666666, 0.71535],
                [0.5789, 0.91825, 0.53465],
                [0.4031666666666667, 0.9899666666666667, 0.7544],
                [0.43016666666666664, 0.9066333333333333, 0.9956],
                [0.5968333333333333, 0.8433666666666667, 0.7456],
                [0.5698333333333333, 0.9267, 0.5044],
                [0.4091666666666667, 0.8770666666666668, 0.82785],
                [0.42416666666666664, 0.9603999999999999, 0.92215],
                [0.5908333333333333, 0.9562666666666667, 0.67215],
                [0.5758333333333333, 0.8729333333333332, 0.57785],
                [0.8310666666666666, 0.0431, 0.131],
                [0.6689333333333334, 0.12643333333333334, 0.119],
                [0.8356, 0.12356666666666666, 0.369],
                [0.9977333333333332, 0.040233333333333336, 0.381],
                [0.7999333333333333, 0.03518333333333334, 0.14955],
                [0.7000666666666667, 0.11851666666666666, 0.10045],
                [0.8667333333333334, 0.13148333333333334, 0.35045],
                [0.9666, 0.04815, 0.39955],
                [0.7707, 0.06315, 0.15005],
                [0.7293, 0.14648333333333333, 0.09995],
                [0.8959666666666667, 0.10351666666666666, 0.34995],
                [0.9373666666666667, 0.020183333333333334, 0.40005],
                [0.7402666666666667, 0.0547, 0.16495],
                [0.7597333333333333, 0.13803333333333334, 0.08505],
                [0.9264, 0.11196666666666666, 0.33505],
                [0.9069333333333334, 0.028633333333333334, 0.41495],
                [0.7403, 0.018266666666666667, 0.17945],
                [0.7597, 0.10160000000000001, 0.07055],
                [0.9263666666666667, 0.1484, 0.32055],
                [0.9069666666666666, 0.06506666666666668, 0.42945],
                [0.7703000000000001, 0.15688333333333335, 0.18015],
                [0.7296999999999999, 0.07355, 0.06985],
                [0.8963666666666666, 0.009783333333333333, 0.31985],
                [0.9369666666666667, 0.09311666666666667, 0.43015],
                [0.8000666666666666, 0.16519999999999999, 0.16465],
                [0.6999333333333334, 0.08186666666666667, 0.08535],
                [0.8666, 0.0014666666666666667, 0.33535],
                [0.9667333333333333, 0.0848, 0.41465],
                [0.7163333333333334, 0.01865, 0.2351],
                [0.7836666666666666, 0.10198333333333333, 0.0149],
                [0.9503333333333334, 0.14801666666666666, 0.2649],
                [0.883, 0.06468333333333333, 0.4851],
                [0.7064666666666667, 0.05566666666666667, 0.24505],
                [0.7935333333333333, 0.13899999999999998, 0.00495],
                [0.9602, 0.111, 0.25495],
                [0.8731333333333332, 0.02766666666666667, 0.49505],
                [0.7160666666666667, 0.06394999999999999, 0.2754],
                [0.7839333333333333, 0.14728333333333335, 0.4746],
                [0.9506, 0.10271666666666666, 0.2246],
                [0.8827333333333334, 0.019383333333333332, 0.0254],
                [0.7345333333333333, 0.03523333333333333, 0.295],
                [0.7654666666666667, 0.11856666666666667, 0.455],
                [0.9321333333333334, 0.13143333333333332, 0.205],
                [0.9011999999999999, 0.048100000000000004, 0.045],
                [0.7455666666666666, 0.16508333333333333, 0.28465],
                [0.7544333333333334, 0.08175, 0.46535],
                [0.9211, 0.0015833333333333333, 0.21535],
                [0.9122333333333333, 0.08491666666666665, 0.03465],
                [0.7364999999999999, 0.15663333333333332, 0.2544],
                [0.7635, 0.0733, 0.4956],
                [0.9301666666666666, 0.010033333333333333, 0.2456],
                [0.9031666666666668, 0.09336666666666667, 0.0044],
                [0.7425, 0.04373333333333334, 0.32785],
                [0.7575, 0.12706666666666666, 0.42215],
                [0.9241666666666667, 0.12293333333333334, 0.17215],
                [0.9091666666666667, 0.0396, 0.07785],
                [0.8310666666666666, 0.0431, 0.631],
                [0.6689333333333334, 0.12643333333333334, 0.619],
                [0.8356, 0.12356666666666666, 0.869],
                [0.9977333333333332, 0.040233333333333336, 0.881],
                [0.7999333333333333, 0.03518333333333334, 0.64955],
                [0.7000666666666667, 0.11851666666666666, 0.60045],
                [0.8667333333333334, 0.13148333333333334, 0.8504499999999999],
                [0.9666, 0.04815, 0.8995500000000001],
                [0.7707, 0.06315, 0.65005],
                [0.7293, 0.14648333333333333, 0.59995],
                [0.8959666666666667, 0.10351666666666666, 0.84995],
                [0.9373666666666667, 0.020183333333333334, 0.90005],
                [0.7402666666666667, 0.0547, 0.66495],
                [0.7597333333333333, 0.13803333333333334, 0.58505],
                [0.9264, 0.11196666666666666, 0.8350500000000001],
                [0.9069333333333334, 0.028633333333333334, 0.9149499999999999],
                [0.7403, 0.018266666666666667, 0.67945],
                [0.7597, 0.10160000000000001, 0.57055],
                [0.9263666666666667, 0.1484, 0.82055],
                [0.9069666666666666, 0.06506666666666668, 0.92945],
                [0.7703000000000001, 0.15688333333333335, 0.68015],
                [0.7296999999999999, 0.07355, 0.56985],
                [0.8963666666666666, 0.009783333333333333, 0.81985],
                [0.9369666666666667, 0.09311666666666667, 0.93015],
                [0.8000666666666666, 0.16519999999999999, 0.66465],
                [0.6999333333333334, 0.08186666666666667, 0.58535],
                [0.8666, 0.0014666666666666667, 0.83535],
                [0.9667333333333333, 0.0848, 0.91465],
                [0.7163333333333334, 0.01865, 0.7351],
                [0.7836666666666666, 0.10198333333333333, 0.5149],
                [0.9503333333333334, 0.14801666666666666, 0.7649],
                [0.883, 0.06468333333333333, 0.9851],
                [0.7064666666666667, 0.05566666666666667, 0.74505],
                [0.7935333333333333, 0.13899999999999998, 0.50495],
                [0.9602, 0.111, 0.75495],
                [0.8731333333333332, 0.02766666666666667, 0.99505],
                [0.7160666666666667, 0.06394999999999999, 0.7754],
                [0.7839333333333333, 0.14728333333333335, 0.9746],
                [0.9506, 0.10271666666666666, 0.7246],
                [0.8827333333333334, 0.019383333333333332, 0.5254],
                [0.7345333333333333, 0.03523333333333333, 0.7949999999999999],
                [0.7654666666666667, 0.11856666666666667, 0.9550000000000001],
                [0.9321333333333334, 0.13143333333333332, 0.705],
                [0.9011999999999999, 0.048100000000000004, 0.545],
                [0.7455666666666666, 0.16508333333333333, 0.7846500000000001],
                [0.7544333333333334, 0.08175, 0.9653499999999999],
                [0.9211, 0.0015833333333333333, 0.71535],
                [0.9122333333333333, 0.08491666666666665, 0.53465],
                [0.7364999999999999, 0.15663333333333332, 0.7544],
                [0.7635, 0.0733, 0.9956],
                [0.9301666666666666, 0.010033333333333333, 0.7456],
                [0.9031666666666668, 0.09336666666666667, 0.5044],
                [0.7425, 0.04373333333333334, 0.82785],
                [0.7575, 0.12706666666666666, 0.92215],
                [0.9241666666666667, 0.12293333333333334, 0.67215],
                [0.9091666666666667, 0.0396, 0.57785],
                [0.8310666666666666, 0.20976666666666666, 0.131],
                [0.6689333333333334, 0.29309999999999997, 0.119],
                [0.8356, 0.29023333333333334, 0.369],
                [0.9977333333333332, 0.2069, 0.381],
                [0.7999333333333333, 0.20185, 0.14955],
                [0.7000666666666667, 0.28518333333333334, 0.10045],
                [0.8667333333333334, 0.29814999999999997, 0.35045],
                [0.9666, 0.21481666666666666, 0.39955],
                [0.7707, 0.22981666666666667, 0.15005],
                [0.7293, 0.31315, 0.09995],
                [0.8959666666666667, 0.27018333333333333, 0.34995],
                [0.9373666666666667, 0.18685, 0.40005],
                [0.7402666666666667, 0.22136666666666668, 0.16495],
                [0.7597333333333333, 0.3047, 0.08505],
                [0.9264, 0.27863333333333334, 0.33505],
                [0.9069333333333334, 0.1953, 0.41495],
                [0.7403, 0.1849333333333333, 0.17945],
                [0.7597, 0.26826666666666665, 0.07055],
                [0.9263666666666667, 0.31506666666666666, 0.32055],
                [0.9069666666666666, 0.23173333333333335, 0.42945],
                [0.7703000000000001, 0.32355, 0.18015],
                [0.7296999999999999, 0.24021666666666666, 0.06985],
                [0.8963666666666666, 0.17645, 0.31985],
                [0.9369666666666667, 0.2597833333333333, 0.43015],
                [0.8000666666666666, 0.3318666666666667, 0.16465],
                [0.6999333333333334, 0.24853333333333336, 0.08535],
                [0.8666, 0.16813333333333333, 0.33535],
                [0.9667333333333333, 0.25146666666666667, 0.41465],
                [0.7163333333333334, 0.18531666666666666, 0.2351],
                [0.7836666666666666, 0.26865, 0.0149],
                [0.9503333333333334, 0.31468333333333337, 0.2649],
                [0.883, 0.23135000000000003, 0.4851],
                [0.7064666666666667, 0.22233333333333336, 0.24505],
                [0.7935333333333333, 0.3056666666666667, 0.00495],
                [0.9602, 0.2776666666666667, 0.25495],
                [0.8731333333333332, 0.19433333333333333, 0.49505],
                [0.7160666666666667, 0.23061666666666666, 0.2754],
                [0.7839333333333333, 0.31395, 0.4746],
                [0.9506, 0.2693833333333333, 0.2246],
                [0.8827333333333334, 0.18605000000000002, 0.0254],
                [0.7345333333333333, 0.2019, 0.295],
                [0.7654666666666667, 0.28523333333333334, 0.455],
                [0.9321333333333334, 0.2981, 0.205],
                [0.9011999999999999, 0.21476666666666666, 0.045],
                [0.7455666666666666, 0.33175, 0.28465],
                [0.7544333333333334, 0.24841666666666665, 0.46535],
                [0.9211, 0.16825, 0.21535],
                [0.9122333333333333, 0.2515833333333333, 0.03465],
                [0.7364999999999999, 0.3233, 0.2544],
                [0.7635, 0.23996666666666666, 0.4956],
                [0.9301666666666666, 0.1767, 0.2456],
                [0.9031666666666668, 0.26003333333333334, 0.0044],
                [0.7425, 0.2104, 0.32785],
                [0.7575, 0.29373333333333335, 0.42215],
                [0.9241666666666667, 0.2896, 0.17215],
                [0.9091666666666667, 0.20626666666666668, 0.07785],
                [0.8310666666666666, 0.20976666666666666, 0.631],
                [0.6689333333333334, 0.29309999999999997, 0.619],
                [0.8356, 0.29023333333333334, 0.869],
                [0.9977333333333332, 0.2069, 0.881],
                [0.7999333333333333, 0.20185, 0.64955],
                [0.7000666666666667, 0.28518333333333334, 0.60045],
                [0.8667333333333334, 0.29814999999999997, 0.8504499999999999],
                [0.9666, 0.21481666666666666, 0.8995500000000001],
                [0.7707, 0.22981666666666667, 0.65005],
                [0.7293, 0.31315, 0.59995],
                [0.8959666666666667, 0.27018333333333333, 0.84995],
                [0.9373666666666667, 0.18685, 0.90005],
                [0.7402666666666667, 0.22136666666666668, 0.66495],
                [0.7597333333333333, 0.3047, 0.58505],
                [0.9264, 0.27863333333333334, 0.8350500000000001],
                [0.9069333333333334, 0.1953, 0.9149499999999999],
                [0.7403, 0.1849333333333333, 0.67945],
                [0.7597, 0.26826666666666665, 0.57055],
                [0.9263666666666667, 0.31506666666666666, 0.82055],
                [0.9069666666666666, 0.23173333333333335, 0.92945],
                [0.7703000000000001, 0.32355, 0.68015],
                [0.7296999999999999, 0.24021666666666666, 0.56985],
                [0.8963666666666666, 0.17645, 0.81985],
                [0.9369666666666667, 0.2597833333333333, 0.93015],
                [0.8000666666666666, 0.3318666666666667, 0.66465],
                [0.6999333333333334, 0.24853333333333336, 0.58535],
                [0.8666, 0.16813333333333333, 0.83535],
                [0.9667333333333333, 0.25146666666666667, 0.91465],
                [0.7163333333333334, 0.18531666666666666, 0.7351],
                [0.7836666666666666, 0.26865, 0.5149],
                [0.9503333333333334, 0.31468333333333337, 0.7649],
                [0.883, 0.23135000000000003, 0.9851],
                [0.7064666666666667, 0.22233333333333336, 0.74505],
                [0.7935333333333333, 0.3056666666666667, 0.50495],
                [0.9602, 0.2776666666666667, 0.75495],
                [0.8731333333333332, 0.19433333333333333, 0.99505],
                [0.7160666666666667, 0.23061666666666666, 0.7754],
                [0.7839333333333333, 0.31395, 0.9746],
                [0.9506, 0.2693833333333333, 0.7246],
                [0.8827333333333334, 0.18605000000000002, 0.5254],
                [0.7345333333333333, 0.2019, 0.7949999999999999],
                [0.7654666666666667, 0.28523333333333334, 0.9550000000000001],
                [0.9321333333333334, 0.2981, 0.705],
                [0.9011999999999999, 0.21476666666666666, 0.545],
                [0.7455666666666666, 0.33175, 0.7846500000000001],
                [0.7544333333333334, 0.24841666666666665, 0.9653499999999999],
                [0.9211, 0.16825, 0.71535],
                [0.9122333333333333, 0.2515833333333333, 0.53465],
                [0.7364999999999999, 0.3233, 0.7544],
                [0.7635, 0.23996666666666666, 0.9956],
                [0.9301666666666666, 0.1767, 0.7456],
                [0.9031666666666668, 0.26003333333333334, 0.5044],
                [0.7425, 0.2104, 0.82785],
                [0.7575, 0.29373333333333335, 0.92215],
                [0.9241666666666667, 0.2896, 0.67215],
                [0.9091666666666667, 0.20626666666666668, 0.57785],
                [0.8310666666666666, 0.37643333333333334, 0.131],
                [0.6689333333333334, 0.45976666666666666, 0.119],
                [0.8356, 0.45690000000000003, 0.369],
                [0.9977333333333332, 0.37356666666666666, 0.381],
                [0.7999333333333333, 0.36851666666666666, 0.14955],
                [0.7000666666666667, 0.45185000000000003, 0.10045],
                [0.8667333333333334, 0.46481666666666666, 0.35045],
                [0.9666, 0.38148333333333334, 0.39955],
                [0.7707, 0.3964833333333333, 0.15005],
                [0.7293, 0.4798166666666666, 0.09995],
                [0.8959666666666667, 0.43685, 0.34995],
                [0.9373666666666667, 0.3535166666666667, 0.40005],
                [0.7402666666666667, 0.3880333333333333, 0.16495],
                [0.7597333333333333, 0.47136666666666666, 0.08505],
                [0.9264, 0.44530000000000003, 0.33505],
                [0.9069333333333334, 0.3619666666666667, 0.41495],
                [0.7403, 0.35159999999999997, 0.17945],
                [0.7597, 0.43493333333333334, 0.07055],
                [0.9263666666666667, 0.48173333333333335, 0.32055],
                [0.9069666666666666, 0.39840000000000003, 0.42945],
                [0.7703000000000001, 0.4902166666666667, 0.18015],
                [0.7296999999999999, 0.4068833333333333, 0.06985],
                [0.8963666666666666, 0.3431166666666667, 0.31985],
                [0.9369666666666667, 0.42645, 0.43015],
                [0.8000666666666666, 0.49853333333333333, 0.16465],
                [0.6999333333333334, 0.4152, 0.08535],
                [0.8666, 0.3348, 0.33535],
                [0.9667333333333333, 0.4181333333333333, 0.41465],
                [0.7163333333333334, 0.3519833333333333, 0.2351],
                [0.7836666666666666, 0.43531666666666663, 0.0149],
                [0.9503333333333334, 0.48135, 0.2649],
                [0.883, 0.3980166666666667, 0.4851],
                [0.7064666666666667, 0.389, 0.24505],
                [0.7935333333333333, 0.4723333333333333, 0.00495],
                [0.9602, 0.4443333333333333, 0.25495],
                [0.8731333333333332, 0.361, 0.49505],
                [0.7160666666666667, 0.3972833333333334, 0.2754],
                [0.7839333333333333, 0.4806166666666667, 0.4746],
                [0.9506, 0.43605, 0.2246],
                [0.8827333333333334, 0.3527166666666666, 0.0254],
                [0.7345333333333333, 0.36856666666666665, 0.295],
                [0.7654666666666667, 0.4519, 0.455],
                [0.9321333333333334, 0.4647666666666666, 0.205],
                [0.9011999999999999, 0.38143333333333335, 0.045],
                [0.7455666666666666, 0.4984166666666667, 0.28465],
                [0.7544333333333334, 0.4150833333333333, 0.46535],
                [0.9211, 0.3349166666666667, 0.21535],
                [0.9122333333333333, 0.41825, 0.03465],
                [0.7364999999999999, 0.48996666666666666, 0.2544],
                [0.7635, 0.40663333333333335, 0.4956],
                [0.9301666666666666, 0.34336666666666665, 0.2456],
                [0.9031666666666668, 0.4267, 0.0044],
                [0.7425, 0.37706666666666666, 0.32785],
                [0.7575, 0.4604, 0.42215],
                [0.9241666666666667, 0.45626666666666665, 0.17215],
                [0.9091666666666667, 0.37293333333333334, 0.07785],
                [0.8310666666666666, 0.37643333333333334, 0.631],
                [0.6689333333333334, 0.45976666666666666, 0.619],
                [0.8356, 0.45690000000000003, 0.869],
                [0.9977333333333332, 0.37356666666666666, 0.881],
                [0.7999333333333333, 0.36851666666666666, 0.64955],
                [0.7000666666666667, 0.45185000000000003, 0.60045],
                [0.8667333333333334, 0.46481666666666666, 0.8504499999999999],
                [0.9666, 0.38148333333333334, 0.8995500000000001],
                [0.7707, 0.3964833333333333, 0.65005],
                [0.7293, 0.4798166666666666, 0.59995],
                [0.8959666666666667, 0.43685, 0.84995],
                [0.9373666666666667, 0.3535166666666667, 0.90005],
                [0.7402666666666667, 0.3880333333333333, 0.66495],
                [0.7597333333333333, 0.47136666666666666, 0.58505],
                [0.9264, 0.44530000000000003, 0.8350500000000001],
                [0.9069333333333334, 0.3619666666666667, 0.9149499999999999],
                [0.7403, 0.35159999999999997, 0.67945],
                [0.7597, 0.43493333333333334, 0.57055],
                [0.9263666666666667, 0.48173333333333335, 0.82055],
                [0.9069666666666666, 0.39840000000000003, 0.92945],
                [0.7703000000000001, 0.4902166666666667, 0.68015],
                [0.7296999999999999, 0.4068833333333333, 0.56985],
                [0.8963666666666666, 0.3431166666666667, 0.81985],
                [0.9369666666666667, 0.42645, 0.93015],
                [0.8000666666666666, 0.49853333333333333, 0.66465],
                [0.6999333333333334, 0.4152, 0.58535],
                [0.8666, 0.3348, 0.83535],
                [0.9667333333333333, 0.4181333333333333, 0.91465],
                [0.7163333333333334, 0.3519833333333333, 0.7351],
                [0.7836666666666666, 0.43531666666666663, 0.5149],
                [0.9503333333333334, 0.48135, 0.7649],
                [0.883, 0.3980166666666667, 0.9851],
                [0.7064666666666667, 0.389, 0.74505],
                [0.7935333333333333, 0.4723333333333333, 0.50495],
                [0.9602, 0.4443333333333333, 0.75495],
                [0.8731333333333332, 0.361, 0.99505],
                [0.7160666666666667, 0.3972833333333334, 0.7754],
                [0.7839333333333333, 0.4806166666666667, 0.9746],
                [0.9506, 0.43605, 0.7246],
                [0.8827333333333334, 0.3527166666666666, 0.5254],
                [0.7345333333333333, 0.36856666666666665, 0.7949999999999999],
                [0.7654666666666667, 0.4519, 0.9550000000000001],
                [0.9321333333333334, 0.4647666666666666, 0.705],
                [0.9011999999999999, 0.38143333333333335, 0.545],
                [0.7455666666666666, 0.4984166666666667, 0.7846500000000001],
                [0.7544333333333334, 0.4150833333333333, 0.9653499999999999],
                [0.9211, 0.3349166666666667, 0.71535],
                [0.9122333333333333, 0.41825, 0.53465],
                [0.7364999999999999, 0.48996666666666666, 0.7544],
                [0.7635, 0.40663333333333335, 0.9956],
                [0.9301666666666666, 0.34336666666666665, 0.7456],
                [0.9031666666666668, 0.4267, 0.5044],
                [0.7425, 0.37706666666666666, 0.82785],
                [0.7575, 0.4604, 0.92215],
                [0.9241666666666667, 0.45626666666666665, 0.67215],
                [0.9091666666666667, 0.37293333333333334, 0.57785],
                [0.8310666666666666, 0.5431, 0.131],
                [0.6689333333333334, 0.6264333333333333, 0.119],
                [0.8356, 0.6235666666666667, 0.369],
                [0.9977333333333332, 0.5402333333333333, 0.381],
                [0.7999333333333333, 0.5351833333333333, 0.14955],
                [0.7000666666666667, 0.6185166666666667, 0.10045],
                [0.8667333333333334, 0.6314833333333333, 0.35045],
                [0.9666, 0.54815, 0.39955],
                [0.7707, 0.5631499999999999, 0.15005],
                [0.7293, 0.6464833333333333, 0.09995],
                [0.8959666666666667, 0.6035166666666667, 0.34995],
                [0.9373666666666667, 0.5201833333333333, 0.40005],
                [0.7402666666666667, 0.5547, 0.16495],
                [0.7597333333333333, 0.6380333333333333, 0.08505],
                [0.9264, 0.6119666666666667, 0.33505],
                [0.9069333333333334, 0.5286333333333334, 0.41495],
                [0.7403, 0.5182666666666667, 0.17945],
                [0.7597, 0.6016, 0.07055],
                [0.9263666666666667, 0.6484, 0.32055],
                [0.9069666666666666, 0.5650666666666667, 0.42945],
                [0.7703000000000001, 0.6568833333333334, 0.18015],
                [0.7296999999999999, 0.57355, 0.06985],
                [0.8963666666666666, 0.5097833333333334, 0.31985],
                [0.9369666666666667, 0.5931166666666666, 0.43015],
                [0.8000666666666666, 0.6652, 0.16465],
                [0.6999333333333334, 0.5818666666666666, 0.08535],
                [0.8666, 0.5014666666666666, 0.33535],
                [0.9667333333333333, 0.5848, 0.41465],
                [0.7163333333333334, 0.5186499999999999, 0.2351],
                [0.7836666666666666, 0.6019833333333333, 0.0149],
                [0.9503333333333334, 0.6480166666666667, 0.2649],
                [0.883, 0.5646833333333333, 0.4851],
                [0.7064666666666667, 0.5556666666666666, 0.24505],
                [0.7935333333333333, 0.639, 0.00495],
                [0.9602, 0.611, 0.25495],
                [0.8731333333333332, 0.5276666666666666, 0.49505],
                [0.7160666666666667, 0.5639500000000001, 0.2754],
                [0.7839333333333333, 0.6472833333333333, 0.4746],
                [0.9506, 0.6027166666666667, 0.2246],
                [0.8827333333333334, 0.5193833333333333, 0.0254],
                [0.7345333333333333, 0.5352333333333333, 0.295],
                [0.7654666666666667, 0.6185666666666667, 0.455],
                [0.9321333333333334, 0.6314333333333333, 0.205],
                [0.9011999999999999, 0.5481, 0.045],
                [0.7455666666666666, 0.6650833333333334, 0.28465],
                [0.7544333333333334, 0.58175, 0.46535],
                [0.9211, 0.5015833333333334, 0.21535],
                [0.9122333333333333, 0.5849166666666666, 0.03465],
                [0.7364999999999999, 0.6566333333333333, 0.2544],
                [0.7635, 0.5733, 0.4956],
                [0.9301666666666666, 0.5100333333333333, 0.2456],
                [0.9031666666666668, 0.5933666666666667, 0.0044],
                [0.7425, 0.5437333333333333, 0.32785],
                [0.7575, 0.6270666666666667, 0.42215],
                [0.9241666666666667, 0.6229333333333333, 0.17215],
                [0.9091666666666667, 0.5396, 0.07785],
                [0.8310666666666666, 0.5431, 0.631],
                [0.6689333333333334, 0.6264333333333333, 0.619],
                [0.8356, 0.6235666666666667, 0.869],
                [0.9977333333333332, 0.5402333333333333, 0.881],
                [0.7999333333333333, 0.5351833333333333, 0.64955],
                [0.7000666666666667, 0.6185166666666667, 0.60045],
                [0.8667333333333334, 0.6314833333333333, 0.8504499999999999],
                [0.9666, 0.54815, 0.8995500000000001],
                [0.7707, 0.5631499999999999, 0.65005],
                [0.7293, 0.6464833333333333, 0.59995],
                [0.8959666666666667, 0.6035166666666667, 0.84995],
                [0.9373666666666667, 0.5201833333333333, 0.90005],
                [0.7402666666666667, 0.5547, 0.66495],
                [0.7597333333333333, 0.6380333333333333, 0.58505],
                [0.9264, 0.6119666666666667, 0.8350500000000001],
                [0.9069333333333334, 0.5286333333333334, 0.9149499999999999],
                [0.7403, 0.5182666666666667, 0.67945],
                [0.7597, 0.6016, 0.57055],
                [0.9263666666666667, 0.6484, 0.82055],
                [0.9069666666666666, 0.5650666666666667, 0.92945],
                [0.7703000000000001, 0.6568833333333334, 0.68015],
                [0.7296999999999999, 0.57355, 0.56985],
                [0.8963666666666666, 0.5097833333333334, 0.81985],
                [0.9369666666666667, 0.5931166666666666, 0.93015],
                [0.8000666666666666, 0.6652, 0.66465],
                [0.6999333333333334, 0.5818666666666666, 0.58535],
                [0.8666, 0.5014666666666666, 0.83535],
                [0.9667333333333333, 0.5848, 0.91465],
                [0.7163333333333334, 0.5186499999999999, 0.7351],
                [0.7836666666666666, 0.6019833333333333, 0.5149],
                [0.9503333333333334, 0.6480166666666667, 0.7649],
                [0.883, 0.5646833333333333, 0.9851],
                [0.7064666666666667, 0.5556666666666666, 0.74505],
                [0.7935333333333333, 0.639, 0.50495],
                [0.9602, 0.611, 0.75495],
                [0.8731333333333332, 0.5276666666666666, 0.99505],
                [0.7160666666666667, 0.5639500000000001, 0.7754],
                [0.7839333333333333, 0.6472833333333333, 0.9746],
                [0.9506, 0.6027166666666667, 0.7246],
                [0.8827333333333334, 0.5193833333333333, 0.5254],
                [0.7345333333333333, 0.5352333333333333, 0.7949999999999999],
                [0.7654666666666667, 0.6185666666666667, 0.9550000000000001],
                [0.9321333333333334, 0.6314333333333333, 0.705],
                [0.9011999999999999, 0.5481, 0.545],
                [0.7455666666666666, 0.6650833333333334, 0.7846500000000001],
                [0.7544333333333334, 0.58175, 0.9653499999999999],
                [0.9211, 0.5015833333333334, 0.71535],
                [0.9122333333333333, 0.5849166666666666, 0.53465],
                [0.7364999999999999, 0.6566333333333333, 0.7544],
                [0.7635, 0.5733, 0.9956],
                [0.9301666666666666, 0.5100333333333333, 0.7456],
                [0.9031666666666668, 0.5933666666666667, 0.5044],
                [0.7425, 0.5437333333333333, 0.82785],
                [0.7575, 0.6270666666666667, 0.92215],
                [0.9241666666666667, 0.6229333333333333, 0.67215],
                [0.9091666666666667, 0.5396, 0.57785],
                [0.8310666666666666, 0.7097666666666668, 0.131],
                [0.6689333333333334, 0.7931, 0.119],
                [0.8356, 0.7902333333333332, 0.369],
                [0.9977333333333332, 0.7069, 0.381],
                [0.7999333333333333, 0.70185, 0.14955],
                [0.7000666666666667, 0.7851833333333333, 0.10045],
                [0.8667333333333334, 0.79815, 0.35045],
                [0.9666, 0.7148166666666667, 0.39955],
                [0.7707, 0.7298166666666667, 0.15005],
                [0.7293, 0.8131499999999999, 0.09995],
                [0.8959666666666667, 0.7701833333333333, 0.34995],
                [0.9373666666666667, 0.6868500000000001, 0.40005],
                [0.7402666666666667, 0.7213666666666666, 0.16495],
                [0.7597333333333333, 0.8047, 0.08505],
                [0.9264, 0.7786333333333334, 0.33505],
                [0.9069333333333334, 0.6953, 0.41495],
                [0.7403, 0.6849333333333334, 0.17945],
                [0.7597, 0.7682666666666668, 0.07055],
                [0.9263666666666667, 0.8150666666666666, 0.32055],
                [0.9069666666666666, 0.7317333333333332, 0.42945],
                [0.7703000000000001, 0.82355, 0.18015],
                [0.7296999999999999, 0.7402166666666666, 0.06985],
                [0.8963666666666666, 0.67645, 0.31985],
                [0.9369666666666667, 0.7597833333333334, 0.43015],
                [0.8000666666666666, 0.8318666666666666, 0.16465],
                [0.6999333333333334, 0.7485333333333334, 0.08535],
                [0.8666, 0.6681333333333334, 0.33535],
                [0.9667333333333333, 0.7514666666666666, 0.41465],
                [0.7163333333333334, 0.6853166666666667, 0.2351],
                [0.7836666666666666, 0.76865, 0.0149],
                [0.9503333333333334, 0.8146833333333333, 0.2649],
                [0.883, 0.73135, 0.4851],
                [0.7064666666666667, 0.7223333333333333, 0.24505],
                [0.7935333333333333, 0.8056666666666666, 0.00495],
                [0.9602, 0.7776666666666667, 0.25495],
                [0.8731333333333332, 0.6943333333333334, 0.49505],
                [0.7160666666666667, 0.7306166666666667, 0.2754],
                [0.7839333333333333, 0.8139500000000001, 0.4746],
                [0.9506, 0.7693833333333333, 0.2246],
                [0.8827333333333334, 0.6860499999999999, 0.0254],
                [0.7345333333333333, 0.7019000000000001, 0.295],
                [0.7654666666666667, 0.7852333333333333, 0.455],
                [0.9321333333333334, 0.7980999999999999, 0.205],
                [0.9011999999999999, 0.7147666666666667, 0.045],
                [0.7455666666666666, 0.83175, 0.28465],
                [0.7544333333333334, 0.7484166666666666, 0.46535],
                [0.9211, 0.66825, 0.21535],
                [0.9122333333333333, 0.7515833333333334, 0.03465],
                [0.7364999999999999, 0.8233, 0.2544],
                [0.7635, 0.7399666666666667, 0.4956],
                [0.9301666666666666, 0.6767, 0.2456],
                [0.9031666666666668, 0.7600333333333333, 0.0044],
                [0.7425, 0.7104, 0.32785],
                [0.7575, 0.7937333333333333, 0.42215],
                [0.9241666666666667, 0.7896000000000001, 0.17215],
                [0.9091666666666667, 0.7062666666666666, 0.07785],
                [0.8310666666666666, 0.7097666666666668, 0.631],
                [0.6689333333333334, 0.7931, 0.619],
                [0.8356, 0.7902333333333332, 0.869],
                [0.9977333333333332, 0.7069, 0.881],
                [0.7999333333333333, 0.70185, 0.64955],
                [0.7000666666666667, 0.7851833333333333, 0.60045],
                [0.8667333333333334, 0.79815, 0.8504499999999999],
                [0.9666, 0.7148166666666667, 0.8995500000000001],
                [0.7707, 0.7298166666666667, 0.65005],
                [0.7293, 0.8131499999999999, 0.59995],
                [0.8959666666666667, 0.7701833333333333, 0.84995],
                [0.9373666666666667, 0.6868500000000001, 0.90005],
                [0.7402666666666667, 0.7213666666666666, 0.66495],
                [0.7597333333333333, 0.8047, 0.58505],
                [0.9264, 0.7786333333333334, 0.8350500000000001],
                [0.9069333333333334, 0.6953, 0.9149499999999999],
                [0.7403, 0.6849333333333334, 0.67945],
                [0.7597, 0.7682666666666668, 0.57055],
                [0.9263666666666667, 0.8150666666666666, 0.82055],
                [0.9069666666666666, 0.7317333333333332, 0.92945],
                [0.7703000000000001, 0.82355, 0.68015],
                [0.7296999999999999, 0.7402166666666666, 0.56985],
                [0.8963666666666666, 0.67645, 0.81985],
                [0.9369666666666667, 0.7597833333333334, 0.93015],
                [0.8000666666666666, 0.8318666666666666, 0.66465],
                [0.6999333333333334, 0.7485333333333334, 0.58535],
                [0.8666, 0.6681333333333334, 0.83535],
                [0.9667333333333333, 0.7514666666666666, 0.91465],
                [0.7163333333333334, 0.6853166666666667, 0.7351],
                [0.7836666666666666, 0.76865, 0.5149],
                [0.9503333333333334, 0.8146833333333333, 0.7649],
                [0.883, 0.73135, 0.9851],
                [0.7064666666666667, 0.7223333333333333, 0.74505],
                [0.7935333333333333, 0.8056666666666666, 0.50495],
                [0.9602, 0.7776666666666667, 0.75495],
                [0.8731333333333332, 0.6943333333333334, 0.99505],
                [0.7160666666666667, 0.7306166666666667, 0.7754],
                [0.7839333333333333, 0.8139500000000001, 0.9746],
                [0.9506, 0.7693833333333333, 0.7246],
                [0.8827333333333334, 0.6860499999999999, 0.5254],
                [0.7345333333333333, 0.7019000000000001, 0.7949999999999999],
                [0.7654666666666667, 0.7852333333333333, 0.9550000000000001],
                [0.9321333333333334, 0.7980999999999999, 0.705],
                [0.9011999999999999, 0.7147666666666667, 0.545],
                [0.7455666666666666, 0.83175, 0.7846500000000001],
                [0.7544333333333334, 0.7484166666666666, 0.9653499999999999],
                [0.9211, 0.66825, 0.71535],
                [0.9122333333333333, 0.7515833333333334, 0.53465],
                [0.7364999999999999, 0.8233, 0.7544],
                [0.7635, 0.7399666666666667, 0.9956],
                [0.9301666666666666, 0.6767, 0.7456],
                [0.9031666666666668, 0.7600333333333333, 0.5044],
                [0.7425, 0.7104, 0.82785],
                [0.7575, 0.7937333333333333, 0.92215],
                [0.9241666666666667, 0.7896000000000001, 0.67215],
                [0.9091666666666667, 0.7062666666666666, 0.57785],
                [0.8310666666666666, 0.8764333333333334, 0.131],
                [0.6689333333333334, 0.9597666666666668, 0.119],
                [0.8356, 0.9569, 0.369],
                [0.9977333333333332, 0.8735666666666666, 0.381],
                [0.7999333333333333, 0.8685166666666667, 0.14955],
                [0.7000666666666667, 0.95185, 0.10045],
                [0.8667333333333334, 0.9648166666666667, 0.35045],
                [0.9666, 0.8814833333333333, 0.39955],
                [0.7707, 0.8964833333333333, 0.15005],
                [0.7293, 0.9798166666666667, 0.09995],
                [0.8959666666666667, 0.9368500000000001, 0.34995],
                [0.9373666666666667, 0.8535166666666667, 0.40005],
                [0.7402666666666667, 0.8880333333333333, 0.16495],
                [0.7597333333333333, 0.9713666666666666, 0.08505],
                [0.9264, 0.9453, 0.33505],
                [0.9069333333333334, 0.8619666666666667, 0.41495],
                [0.7403, 0.8516, 0.17945],
                [0.7597, 0.9349333333333334, 0.07055],
                [0.9263666666666667, 0.9817333333333332, 0.32055],
                [0.9069666666666666, 0.8984, 0.42945],
                [0.7703000000000001, 0.9902166666666666, 0.18015],
                [0.7296999999999999, 0.9068833333333334, 0.06985],
                [0.8963666666666666, 0.8431166666666666, 0.31985],
                [0.9369666666666667, 0.92645, 0.43015],
                [0.8000666666666666, 0.9985333333333334, 0.16465],
                [0.6999333333333334, 0.9152, 0.08535],
                [0.8666, 0.8348, 0.33535],
                [0.9667333333333333, 0.9181333333333334, 0.41465],
                [0.7163333333333334, 0.8519833333333334, 0.2351],
                [0.7836666666666666, 0.9353166666666667, 0.0149],
                [0.9503333333333334, 0.98135, 0.2649],
                [0.883, 0.8980166666666666, 0.4851],
                [0.7064666666666667, 0.8889999999999999, 0.24505],
                [0.7935333333333333, 0.9723333333333333, 0.00495],
                [0.9602, 0.9443333333333334, 0.25495],
                [0.8731333333333332, 0.8610000000000001, 0.49505],
                [0.7160666666666667, 0.8972833333333333, 0.2754],
                [0.7839333333333333, 0.9806166666666667, 0.4746],
                [0.9506, 0.9360499999999999, 0.2246],
                [0.8827333333333334, 0.8527166666666667, 0.0254],
                [0.7345333333333333, 0.8685666666666667, 0.295],
                [0.7654666666666667, 0.9519000000000001, 0.455],
                [0.9321333333333334, 0.9647666666666667, 0.205],
                [0.9011999999999999, 0.8814333333333333, 0.045],
                [0.7455666666666666, 0.9984166666666666, 0.28465],
                [0.7544333333333334, 0.9150833333333334, 0.46535],
                [0.9211, 0.8349166666666666, 0.21535],
                [0.9122333333333333, 0.91825, 0.03465],
                [0.7364999999999999, 0.9899666666666667, 0.2544],
                [0.7635, 0.9066333333333333, 0.4956],
                [0.9301666666666666, 0.8433666666666667, 0.2456],
                [0.9031666666666668, 0.9267, 0.0044],
                [0.7425, 0.8770666666666668, 0.32785],
                [0.7575, 0.9603999999999999, 0.42215],
                [0.9241666666666667, 0.9562666666666667, 0.17215],
                [0.9091666666666667, 0.8729333333333332, 0.07785],
                [0.8310666666666666, 0.8764333333333334, 0.631],
                [0.6689333333333334, 0.9597666666666668, 0.619],
                [0.8356, 0.9569, 0.869],
                [0.9977333333333332, 0.8735666666666666, 0.881],
                [0.7999333333333333, 0.8685166666666667, 0.64955],
                [0.7000666666666667, 0.95185, 0.60045],
                [0.8667333333333334, 0.9648166666666667, 0.8504499999999999],
                [0.9666, 0.8814833333333333, 0.8995500000000001],
                [0.7707, 0.8964833333333333, 0.65005],
                [0.7293, 0.9798166666666667, 0.59995],
                [0.8959666666666667, 0.9368500000000001, 0.84995],
                [0.9373666666666667, 0.8535166666666667, 0.90005],
                [0.7402666666666667, 0.8880333333333333, 0.66495],
                [0.7597333333333333, 0.9713666666666666, 0.58505],
                [0.9264, 0.9453, 0.8350500000000001],
                [0.9069333333333334, 0.8619666666666667, 0.9149499999999999],
                [0.7403, 0.8516, 0.67945],
                [0.7597, 0.9349333333333334, 0.57055],
                [0.9263666666666667, 0.9817333333333332, 0.82055],
                [0.9069666666666666, 0.8984, 0.92945],
                [0.7703000000000001, 0.9902166666666666, 0.68015],
                [0.7296999999999999, 0.9068833333333334, 0.56985],
                [0.8963666666666666, 0.8431166666666666, 0.81985],
                [0.9369666666666667, 0.92645, 0.93015],
                [0.8000666666666666, 0.9985333333333334, 0.66465],
                [0.6999333333333334, 0.9152, 0.58535],
                [0.8666, 0.8348, 0.83535],
                [0.9667333333333333, 0.9181333333333334, 0.91465],
                [0.7163333333333334, 0.8519833333333334, 0.7351],
                [0.7836666666666666, 0.9353166666666667, 0.5149],
                [0.9503333333333334, 0.98135, 0.7649],
                [0.883, 0.8980166666666666, 0.9851],
                [0.7064666666666667, 0.8889999999999999, 0.74505],
                [0.7935333333333333, 0.9723333333333333, 0.50495],
                [0.9602, 0.9443333333333334, 0.75495],
                [0.8731333333333332, 0.8610000000000001, 0.99505],
                [0.7160666666666667, 0.8972833333333333, 0.7754],
                [0.7839333333333333, 0.9806166666666667, 0.9746],
                [0.9506, 0.9360499999999999, 0.7246],
                [0.8827333333333334, 0.8527166666666667, 0.5254],
                [0.7345333333333333, 0.8685666666666667, 0.7949999999999999],
                [0.7654666666666667, 0.9519000000000001, 0.9550000000000001],
                [0.9321333333333334, 0.9647666666666667, 0.705],
                [0.9011999999999999, 0.8814333333333333, 0.545],
                [0.7455666666666666, 0.9984166666666666, 0.7846500000000001],
                [0.7544333333333334, 0.9150833333333334, 0.9653499999999999],
                [0.9211, 0.8349166666666666, 0.71535],
                [0.9122333333333333, 0.91825, 0.53465],
                [0.7364999999999999, 0.9899666666666667, 0.7544],
                [0.7635, 0.9066333333333333, 0.9956],
                [0.9301666666666666, 0.8433666666666667, 0.7456],
                [0.9031666666666668, 0.9267, 0.5044],
                [0.7425, 0.8770666666666668, 0.82785],
                [0.7575, 0.9603999999999999, 0.92215],
                [0.9241666666666667, 0.9562666666666667, 0.67215],
                [0.9091666666666667, 0.8729333333333332, 0.57785],
                [0.2124, 0.12388500000000001, 0.110585],
                [0.2876, 0.040551666666666666, 0.139415],
                [0.12093333333333334, 0.04278166666666666, 0.389415],
                [0.04573333333333333, 0.126115, 0.360585],
                [0.2124, 0.12388500000000001, 0.610585],
                [0.2876, 0.040551666666666666, 0.6394150000000001],
                [0.12093333333333334, 0.04278166666666666, 0.8894150000000001],
                [0.04573333333333333, 0.126115, 0.8605849999999999],
                [0.2124, 0.2905516666666667, 0.110585],
                [0.2876, 0.2072183333333333, 0.139415],
                [0.12093333333333334, 0.20944833333333332, 0.389415],
                [0.04573333333333333, 0.29278166666666666, 0.360585],
                [0.2124, 0.2905516666666667, 0.610585],
                [0.2876, 0.2072183333333333, 0.6394150000000001],
                [0.12093333333333334, 0.20944833333333332, 0.8894150000000001],
                [0.04573333333333333, 0.29278166666666666, 0.8605849999999999],
                [0.2124, 0.45721833333333334, 0.110585],
                [0.2876, 0.373885, 0.139415],
                [0.12093333333333334, 0.376115, 0.389415],
                [0.04573333333333333, 0.4594483333333333, 0.360585],
                [0.2124, 0.45721833333333334, 0.610585],
                [0.2876, 0.373885, 0.6394150000000001],
                [0.12093333333333334, 0.376115, 0.8894150000000001],
                [0.04573333333333333, 0.4594483333333333, 0.8605849999999999],
                [0.2124, 0.623885, 0.110585],
                [0.2876, 0.5405516666666667, 0.139415],
                [0.12093333333333334, 0.5427816666666666, 0.389415],
                [0.04573333333333333, 0.626115, 0.360585],
                [0.2124, 0.623885, 0.610585],
                [0.2876, 0.5405516666666667, 0.6394150000000001],
                [0.12093333333333334, 0.5427816666666666, 0.8894150000000001],
                [0.04573333333333333, 0.626115, 0.8605849999999999],
                [0.2124, 0.7905516666666667, 0.110585],
                [0.2876, 0.7072183333333334, 0.139415],
                [0.12093333333333334, 0.7094483333333333, 0.389415],
                [0.04573333333333333, 0.7927816666666666, 0.360585],
                [0.2124, 0.7905516666666667, 0.610585],
                [0.2876, 0.7072183333333334, 0.6394150000000001],
                [0.12093333333333334, 0.7094483333333333, 0.8894150000000001],
                [0.04573333333333333, 0.7927816666666666, 0.8605849999999999],
                [0.2124, 0.9572183333333334, 0.110585],
                [0.2876, 0.873885, 0.139415],
                [0.12093333333333334, 0.876115, 0.389415],
                [0.04573333333333333, 0.9594483333333333, 0.360585],
                [0.2124, 0.9572183333333334, 0.610585],
                [0.2876, 0.873885, 0.6394150000000001],
                [0.12093333333333334, 0.876115, 0.8894150000000001],
                [0.04573333333333333, 0.9594483333333333, 0.8605849999999999],
                [0.5457333333333333, 0.12388500000000001, 0.110585],
                [0.6209333333333333, 0.040551666666666666, 0.139415],
                [0.45426666666666665, 0.04278166666666666, 0.389415],
                [0.37906666666666666, 0.126115, 0.360585],
                [0.5457333333333333, 0.12388500000000001, 0.610585],
                [0.6209333333333333, 0.040551666666666666, 0.6394150000000001],
                [0.45426666666666665, 0.04278166666666666, 0.8894150000000001],
                [0.37906666666666666, 0.126115, 0.8605849999999999],
                [0.5457333333333333, 0.2905516666666667, 0.110585],
                [0.6209333333333333, 0.2072183333333333, 0.139415],
                [0.45426666666666665, 0.20944833333333332, 0.389415],
                [0.37906666666666666, 0.29278166666666666, 0.360585],
                [0.5457333333333333, 0.2905516666666667, 0.610585],
                [0.6209333333333333, 0.2072183333333333, 0.6394150000000001],
                [0.45426666666666665, 0.20944833333333332, 0.8894150000000001],
                [0.37906666666666666, 0.29278166666666666, 0.8605849999999999],
                [0.5457333333333333, 0.45721833333333334, 0.110585],
                [0.6209333333333333, 0.373885, 0.139415],
                [0.45426666666666665, 0.376115, 0.389415],
                [0.37906666666666666, 0.4594483333333333, 0.360585],
                [0.5457333333333333, 0.45721833333333334, 0.610585],
                [0.6209333333333333, 0.373885, 0.6394150000000001],
                [0.45426666666666665, 0.376115, 0.8894150000000001],
                [0.37906666666666666, 0.4594483333333333, 0.8605849999999999],
                [0.5457333333333333, 0.623885, 0.110585],
                [0.6209333333333333, 0.5405516666666667, 0.139415],
                [0.45426666666666665, 0.5427816666666666, 0.389415],
                [0.37906666666666666, 0.626115, 0.360585],
                [0.5457333333333333, 0.623885, 0.610585],
                [0.6209333333333333, 0.5405516666666667, 0.6394150000000001],
                [0.45426666666666665, 0.5427816666666666, 0.8894150000000001],
                [0.37906666666666666, 0.626115, 0.8605849999999999],
                [0.5457333333333333, 0.7905516666666667, 0.110585],
                [0.6209333333333333, 0.7072183333333334, 0.139415],
                [0.45426666666666665, 0.7094483333333333, 0.389415],
                [0.37906666666666666, 0.7927816666666666, 0.360585],
                [0.5457333333333333, 0.7905516666666667, 0.610585],
                [0.6209333333333333, 0.7072183333333334, 0.6394150000000001],
                [0.45426666666666665, 0.7094483333333333, 0.8894150000000001],
                [0.37906666666666666, 0.7927816666666666, 0.8605849999999999],
                [0.5457333333333333, 0.9572183333333334, 0.110585],
                [0.6209333333333333, 0.873885, 0.139415],
                [0.45426666666666665, 0.876115, 0.389415],
                [0.37906666666666666, 0.9594483333333333, 0.360585],
                [0.5457333333333333, 0.9572183333333334, 0.610585],
                [0.6209333333333333, 0.873885, 0.6394150000000001],
                [0.45426666666666665, 0.876115, 0.8894150000000001],
                [0.37906666666666666, 0.9594483333333333, 0.8605849999999999],
                [0.8790666666666667, 0.12388500000000001, 0.110585],
                [0.9542666666666667, 0.040551666666666666, 0.139415],
                [0.7876, 0.04278166666666666, 0.389415],
                [0.7124, 0.126115, 0.360585],
                [0.8790666666666667, 0.12388500000000001, 0.610585],
                [0.9542666666666667, 0.040551666666666666, 0.6394150000000001],
                [0.7876, 0.04278166666666666, 0.8894150000000001],
                [0.7124, 0.126115, 0.8605849999999999],
                [0.8790666666666667, 0.2905516666666667, 0.110585],
                [0.9542666666666667, 0.2072183333333333, 0.139415],
                [0.7876, 0.20944833333333332, 0.389415],
                [0.7124, 0.29278166666666666, 0.360585],
                [0.8790666666666667, 0.2905516666666667, 0.610585],
                [0.9542666666666667, 0.2072183333333333, 0.6394150000000001],
                [0.7876, 0.20944833333333332, 0.8894150000000001],
                [0.7124, 0.29278166666666666, 0.8605849999999999],
                [0.8790666666666667, 0.45721833333333334, 0.110585],
                [0.9542666666666667, 0.373885, 0.139415],
                [0.7876, 0.376115, 0.389415],
                [0.7124, 0.4594483333333333, 0.360585],
                [0.8790666666666667, 0.45721833333333334, 0.610585],
                [0.9542666666666667, 0.373885, 0.6394150000000001],
                [0.7876, 0.376115, 0.8894150000000001],
                [0.7124, 0.4594483333333333, 0.8605849999999999],
                [0.8790666666666667, 0.623885, 0.110585],
                [0.9542666666666667, 0.5405516666666667, 0.139415],
                [0.7876, 0.5427816666666666, 0.389415],
                [0.7124, 0.626115, 0.360585],
                [0.8790666666666667, 0.623885, 0.610585],
                [0.9542666666666667, 0.5405516666666667, 0.6394150000000001],
                [0.7876, 0.5427816666666666, 0.8894150000000001],
                [0.7124, 0.626115, 0.8605849999999999],
                [0.8790666666666667, 0.7905516666666667, 0.110585],
                [0.9542666666666667, 0.7072183333333334, 0.139415],
                [0.7876, 0.7094483333333333, 0.389415],
                [0.7124, 0.7927816666666666, 0.360585],
                [0.8790666666666667, 0.7905516666666667, 0.610585],
                [0.9542666666666667, 0.7072183333333334, 0.6394150000000001],
                [0.7876, 0.7094483333333333, 0.8894150000000001],
                [0.7124, 0.7927816666666666, 0.8605849999999999],
                [0.8790666666666667, 0.9572183333333334, 0.110585],
                [0.9542666666666667, 0.873885, 0.139415],
                [0.7876, 0.876115, 0.389415],
                [0.7124, 0.9594483333333333, 0.360585],
                [0.8790666666666667, 0.9572183333333334, 0.610585],
                [0.9542666666666667, 0.873885, 0.6394150000000001],
                [0.7876, 0.876115, 0.8894150000000001],
                [0.7124, 0.9594483333333333, 0.8605849999999999],
                [0.10473333333333333, 0.08811666666666666, 0.1402],
                [0.06193333333333333, 0.004783333333333333, 0.1098],
                [0.2286, 0.07855, 0.3598],
                [0.27140000000000003, 0.16188333333333335, 0.3902],
                [0.053233333333333334, 0.07355, 0.1652],
                [0.11343333333333333, 0.15688333333333335, 0.0848],
                [0.2801, 0.09311666666666667, 0.3348],
                [0.21989999999999998, 0.009783333333333333, 0.4152],
                [0.10366666666666667, 0.13263333333333333, 0.191],
                [0.063, 0.049300000000000004, 0.059],
                [0.22966666666666666, 0.03403333333333333, 0.309],
                [0.27033333333333337, 0.11736666666666667, 0.441],
                [0.15363333333333332, 0.14628333333333335, 0.1643],
                [0.013033333333333334, 0.06294999999999999, 0.0857],
                [0.1797, 0.020383333333333333, 0.3357],
                [0.3203, 0.10371666666666667, 0.4143],
                [0.026766666666666664, 0.07503333333333333, 0.2315],
                [0.1399, 0.15836666666666668, 0.0185],
                [0.30656666666666665, 0.09163333333333333, 0.2685],
                [0.19343333333333335, 0.0083, 0.4815],
                [0.04326666666666667, 0.08935, 0.2826],
                [0.1234, 0.006016666666666667, 0.4674],
                [0.29006666666666664, 0.07731666666666666, 0.2174],
                [0.20993333333333333, 0.16065, 0.0326],
                [0.09250000000000001, 0.14598333333333333, 0.2981],
                [0.07416666666666667, 0.06265, 0.4519],
                [0.24083333333333334, 0.020683333333333335, 0.2019],
                [0.25916666666666666, 0.10401666666666666, 0.0481],
                [0.07723333333333333, 0.13183333333333333, 0.24695],
                [0.08943333333333332, 0.048499999999999995, 0.00305],
                [0.2561, 0.034833333333333334, 0.25305],
                [0.2439, 0.11816666666666666, 0.49695],
                [0.10473333333333333, 0.08811666666666666, 0.6402],
                [0.06193333333333333, 0.004783333333333333, 0.6098],
                [0.2286, 0.07855, 0.8598],
                [0.27140000000000003, 0.16188333333333335, 0.8902],
                [0.053233333333333334, 0.07355, 0.6652],
                [0.11343333333333333, 0.15688333333333335, 0.5848],
                [0.2801, 0.09311666666666667, 0.8348],
                [0.21989999999999998, 0.009783333333333333, 0.9152],
                [0.10366666666666667, 0.13263333333333333, 0.6910000000000001],
                [0.063, 0.049300000000000004, 0.5589999999999999],
                [0.22966666666666666, 0.03403333333333333, 0.8089999999999999],
                [0.27033333333333337, 0.11736666666666667, 0.9410000000000001],
                [0.15363333333333332, 0.14628333333333335, 0.6643],
                [0.013033333333333334, 0.06294999999999999, 0.5857],
                [0.1797, 0.020383333333333333, 0.8357],
                [0.3203, 0.10371666666666667, 0.9143],
                [0.026766666666666664, 0.07503333333333333, 0.7315],
                [0.1399, 0.15836666666666668, 0.5185],
                [0.30656666666666665, 0.09163333333333333, 0.7685],
                [0.19343333333333335, 0.0083, 0.9815],
                [0.04326666666666667, 0.08935, 0.7826],
                [0.1234, 0.006016666666666667, 0.9674],
                [0.29006666666666664, 0.07731666666666666, 0.7174],
                [0.20993333333333333, 0.16065, 0.5326],
                [0.09250000000000001, 0.14598333333333333, 0.7981],
                [0.07416666666666667, 0.06265, 0.9519],
                [0.24083333333333334, 0.020683333333333335, 0.7019],
                [0.25916666666666666, 0.10401666666666666, 0.5481],
                [0.07723333333333333, 0.13183333333333333, 0.74695],
                [0.08943333333333332, 0.048499999999999995, 0.50305],
                [0.2561, 0.034833333333333334, 0.75305],
                [0.2439, 0.11816666666666666, 0.99695],
                [0.10473333333333333, 0.2547833333333333, 0.1402],
                [0.06193333333333333, 0.17145, 0.1098],
                [0.2286, 0.24521666666666667, 0.3598],
                [0.27140000000000003, 0.32855, 0.3902],
                [0.053233333333333334, 0.24021666666666666, 0.1652],
                [0.11343333333333333, 0.32355, 0.0848],
                [0.2801, 0.2597833333333333, 0.3348],
                [0.21989999999999998, 0.17645, 0.4152],
                [0.10366666666666667, 0.29929999999999995, 0.191],
                [0.063, 0.21596666666666667, 0.059],
                [0.22966666666666666, 0.2007, 0.309],
                [0.27033333333333337, 0.28403333333333336, 0.441],
                [0.15363333333333332, 0.31295, 0.1643],
                [0.013033333333333334, 0.22961666666666666, 0.0857],
                [0.1797, 0.18705000000000002, 0.3357],
                [0.3203, 0.27038333333333336, 0.4143],
                [0.026766666666666664, 0.2417, 0.2315],
                [0.1399, 0.32503333333333334, 0.0185],
                [0.30656666666666665, 0.2583, 0.2685],
                [0.19343333333333335, 0.1749666666666667, 0.4815],
                [0.04326666666666667, 0.25601666666666667, 0.2826],
                [0.1234, 0.17268333333333333, 0.4674],
                [0.29006666666666664, 0.24398333333333333, 0.2174],
                [0.20993333333333333, 0.32731666666666664, 0.0326],
                [0.09250000000000001, 0.31265000000000004, 0.2981],
                [0.07416666666666667, 0.2293166666666667, 0.4519],
                [0.24083333333333334, 0.18735000000000002, 0.2019],
                [0.25916666666666666, 0.27068333333333333, 0.0481],
                [0.07723333333333333, 0.2985, 0.24695],
                [0.08943333333333332, 0.21516666666666664, 0.00305],
                [0.2561, 0.2015, 0.25305],
                [0.2439, 0.2848333333333333, 0.49695],
                [0.10473333333333333, 0.2547833333333333, 0.6402],
                [0.06193333333333333, 0.17145, 0.6098],
                [0.2286, 0.24521666666666667, 0.8598],
                [0.27140000000000003, 0.32855, 0.8902],
                [0.053233333333333334, 0.24021666666666666, 0.6652],
                [0.11343333333333333, 0.32355, 0.5848],
                [0.2801, 0.2597833333333333, 0.8348],
                [0.21989999999999998, 0.17645, 0.9152],
                [0.10366666666666667, 0.29929999999999995, 0.6910000000000001],
                [0.063, 0.21596666666666667, 0.5589999999999999],
                [0.22966666666666666, 0.2007, 0.8089999999999999],
                [0.27033333333333337, 0.28403333333333336, 0.9410000000000001],
                [0.15363333333333332, 0.31295, 0.6643],
                [0.013033333333333334, 0.22961666666666666, 0.5857],
                [0.1797, 0.18705000000000002, 0.8357],
                [0.3203, 0.27038333333333336, 0.9143],
                [0.026766666666666664, 0.2417, 0.7315],
                [0.1399, 0.32503333333333334, 0.5185],
                [0.30656666666666665, 0.2583, 0.7685],
                [0.19343333333333335, 0.1749666666666667, 0.9815],
                [0.04326666666666667, 0.25601666666666667, 0.7826],
                [0.1234, 0.17268333333333333, 0.9674],
                [0.29006666666666664, 0.24398333333333333, 0.7174],
                [0.20993333333333333, 0.32731666666666664, 0.5326],
                [0.09250000000000001, 0.31265000000000004, 0.7981],
                [0.07416666666666667, 0.2293166666666667, 0.9519],
                [0.24083333333333334, 0.18735000000000002, 0.7019],
                [0.25916666666666666, 0.27068333333333333, 0.5481],
                [0.07723333333333333, 0.2985, 0.74695],
                [0.08943333333333332, 0.21516666666666664, 0.50305],
                [0.2561, 0.2015, 0.75305],
                [0.2439, 0.2848333333333333, 0.99695],
                [0.10473333333333333, 0.42144999999999994, 0.1402],
                [0.06193333333333333, 0.3381166666666667, 0.1098],
                [0.2286, 0.4118833333333333, 0.3598],
                [0.27140000000000003, 0.4952166666666667, 0.3902],
                [0.053233333333333334, 0.4068833333333333, 0.1652],
                [0.11343333333333333, 0.4902166666666667, 0.0848],
                [0.2801, 0.42645, 0.3348],
                [0.21989999999999998, 0.3431166666666667, 0.4152],
                [0.10366666666666667, 0.46596666666666664, 0.191],
                [0.063, 0.3826333333333333, 0.059],
                [0.22966666666666666, 0.3673666666666667, 0.309],
                [0.27033333333333337, 0.45070000000000005, 0.441],
                [0.15363333333333332, 0.47961666666666664, 0.1643],
                [0.013033333333333334, 0.3962833333333333, 0.0857],
                [0.1797, 0.3537166666666667, 0.3357],
                [0.3203, 0.43705, 0.4143],
                [0.026766666666666664, 0.4083666666666667, 0.2315],
                [0.1399, 0.4917, 0.0185],
                [0.30656666666666665, 0.42496666666666666, 0.2685],
                [0.19343333333333335, 0.3416333333333333, 0.4815],
                [0.04326666666666667, 0.42268333333333336, 0.2826],
                [0.1234, 0.33935, 0.4674],
                [0.29006666666666664, 0.41064999999999996, 0.2174],
                [0.20993333333333333, 0.4939833333333333, 0.0326],
                [0.09250000000000001, 0.47931666666666667, 0.2981],
                [0.07416666666666667, 0.39598333333333335, 0.4519],
                [0.24083333333333334, 0.35401666666666665, 0.2019],
                [0.25916666666666666, 0.43734999999999996, 0.0481],
                [0.07723333333333333, 0.4651666666666667, 0.24695],
                [0.08943333333333332, 0.3818333333333333, 0.00305],
                [0.2561, 0.3681666666666667, 0.25305],
                [0.2439, 0.4515, 0.49695],
                [0.10473333333333333, 0.42144999999999994, 0.6402],
                [0.06193333333333333, 0.3381166666666667, 0.6098],
                [0.2286, 0.4118833333333333, 0.8598],
                [0.27140000000000003, 0.4952166666666667, 0.8902],
                [0.053233333333333334, 0.4068833333333333, 0.6652],
                [0.11343333333333333, 0.4902166666666667, 0.5848],
                [0.2801, 0.42645, 0.8348],
                [0.21989999999999998, 0.3431166666666667, 0.9152],
                [0.10366666666666667, 0.46596666666666664, 0.6910000000000001],
                [0.063, 0.3826333333333333, 0.5589999999999999],
                [0.22966666666666666, 0.3673666666666667, 0.8089999999999999],
                [0.27033333333333337, 0.45070000000000005, 0.9410000000000001],
                [0.15363333333333332, 0.47961666666666664, 0.6643],
                [0.013033333333333334, 0.3962833333333333, 0.5857],
                [0.1797, 0.3537166666666667, 0.8357],
                [0.3203, 0.43705, 0.9143],
                [0.026766666666666664, 0.4083666666666667, 0.7315],
                [0.1399, 0.4917, 0.5185],
                [0.30656666666666665, 0.42496666666666666, 0.7685],
                [0.19343333333333335, 0.3416333333333333, 0.9815],
                [0.04326666666666667, 0.42268333333333336, 0.7826],
                [0.1234, 0.33935, 0.9674],
                [0.29006666666666664, 0.41064999999999996, 0.7174],
                [0.20993333333333333, 0.4939833333333333, 0.5326],
                [0.09250000000000001, 0.47931666666666667, 0.7981],
                [0.07416666666666667, 0.39598333333333335, 0.9519],
                [0.24083333333333334, 0.35401666666666665, 0.7019],
                [0.25916666666666666, 0.43734999999999996, 0.5481],
                [0.07723333333333333, 0.4651666666666667, 0.74695],
                [0.08943333333333332, 0.3818333333333333, 0.50305],
                [0.2561, 0.3681666666666667, 0.75305],
                [0.2439, 0.4515, 0.99695],
                [0.10473333333333333, 0.5881166666666666, 0.1402],
                [0.06193333333333333, 0.5047833333333334, 0.1098],
                [0.2286, 0.57855, 0.3598],
                [0.27140000000000003, 0.6618833333333334, 0.3902],
                [0.053233333333333334, 0.57355, 0.1652],
                [0.11343333333333333, 0.6568833333333334, 0.0848],
                [0.2801, 0.5931166666666666, 0.3348],
                [0.21989999999999998, 0.5097833333333334, 0.4152],
                [0.10366666666666667, 0.6326333333333333, 0.191],
                [0.063, 0.5493, 0.059],
                [0.22966666666666666, 0.5340333333333334, 0.309],
                [0.27033333333333337, 0.6173666666666667, 0.441],
                [0.15363333333333332, 0.6462833333333333, 0.1643],
                [0.013033333333333334, 0.56295, 0.0857],
                [0.1797, 0.5203833333333333, 0.3357],
                [0.3203, 0.6037166666666667, 0.4143],
                [0.026766666666666664, 0.5750333333333334, 0.2315],
                [0.1399, 0.6583666666666667, 0.0185],
                [0.30656666666666665, 0.5916333333333333, 0.2685],
                [0.19343333333333335, 0.5083, 0.4815],
                [0.04326666666666667, 0.58935, 0.2826],
                [0.1234, 0.5060166666666667, 0.4674],
                [0.29006666666666664, 0.5773166666666666, 0.2174],
                [0.20993333333333333, 0.66065, 0.0326],
                [0.09250000000000001, 0.6459833333333334, 0.2981],
                [0.07416666666666667, 0.56265, 0.4519],
                [0.24083333333333334, 0.5206833333333333, 0.2019],
                [0.25916666666666666, 0.6040166666666666, 0.0481],
                [0.07723333333333333, 0.6318333333333334, 0.24695],
                [0.08943333333333332, 0.5485, 0.00305],
                [0.2561, 0.5348333333333334, 0.25305],
                [0.2439, 0.6181666666666666, 0.49695],
                [0.10473333333333333, 0.5881166666666666, 0.6402],
                [0.06193333333333333, 0.5047833333333334, 0.6098],
                [0.2286, 0.57855, 0.8598],
                [0.27140000000000003, 0.6618833333333334, 0.8902],
                [0.053233333333333334, 0.57355, 0.6652],
                [0.11343333333333333, 0.6568833333333334, 0.5848],
                [0.2801, 0.5931166666666666, 0.8348],
                [0.21989999999999998, 0.5097833333333334, 0.9152],
                [0.10366666666666667, 0.6326333333333333, 0.6910000000000001],
                [0.063, 0.5493, 0.5589999999999999],
                [0.22966666666666666, 0.5340333333333334, 0.8089999999999999],
                [0.27033333333333337, 0.6173666666666667, 0.9410000000000001],
                [0.15363333333333332, 0.6462833333333333, 0.6643],
                [0.013033333333333334, 0.56295, 0.5857],
                [0.1797, 0.5203833333333333, 0.8357],
                [0.3203, 0.6037166666666667, 0.9143],
                [0.026766666666666664, 0.5750333333333334, 0.7315],
                [0.1399, 0.6583666666666667, 0.5185],
                [0.30656666666666665, 0.5916333333333333, 0.7685],
                [0.19343333333333335, 0.5083, 0.9815],
                [0.04326666666666667, 0.58935, 0.7826],
                [0.1234, 0.5060166666666667, 0.9674],
                [0.29006666666666664, 0.5773166666666666, 0.7174],
                [0.20993333333333333, 0.66065, 0.5326],
                [0.09250000000000001, 0.6459833333333334, 0.7981],
                [0.07416666666666667, 0.56265, 0.9519],
                [0.24083333333333334, 0.5206833333333333, 0.7019],
                [0.25916666666666666, 0.6040166666666666, 0.5481],
                [0.07723333333333333, 0.6318333333333334, 0.74695],
                [0.08943333333333332, 0.5485, 0.50305],
                [0.2561, 0.5348333333333334, 0.75305],
                [0.2439, 0.6181666666666666, 0.99695],
                [0.10473333333333333, 0.7547833333333333, 0.1402],
                [0.06193333333333333, 0.67145, 0.1098],
                [0.2286, 0.7452166666666667, 0.3598],
                [0.27140000000000003, 0.82855, 0.3902],
                [0.053233333333333334, 0.7402166666666666, 0.1652],
                [0.11343333333333333, 0.82355, 0.0848],
                [0.2801, 0.7597833333333334, 0.3348],
                [0.21989999999999998, 0.67645, 0.4152],
                [0.10366666666666667, 0.7993, 0.191],
                [0.063, 0.7159666666666666, 0.059],
                [0.22966666666666666, 0.7007, 0.309],
                [0.27033333333333337, 0.7840333333333334, 0.441],
                [0.15363333333333332, 0.81295, 0.1643],
                [0.013033333333333334, 0.7296166666666667, 0.0857],
                [0.1797, 0.68705, 0.3357],
                [0.3203, 0.7703833333333333, 0.4143],
                [0.026766666666666664, 0.7416999999999999, 0.2315],
                [0.1399, 0.8250333333333333, 0.0185],
                [0.30656666666666665, 0.7583000000000001, 0.2685],
                [0.19343333333333335, 0.6749666666666667, 0.4815],
                [0.04326666666666667, 0.7560166666666667, 0.2826],
                [0.1234, 0.6726833333333334, 0.4674],
                [0.29006666666666664, 0.7439833333333333, 0.2174],
                [0.20993333333333333, 0.8273166666666666, 0.0326],
                [0.09250000000000001, 0.81265, 0.2981],
                [0.07416666666666667, 0.7293166666666666, 0.4519],
                [0.24083333333333334, 0.68735, 0.2019],
                [0.25916666666666666, 0.7706833333333334, 0.0481],
                [0.07723333333333333, 0.7985000000000001, 0.24695],
                [0.08943333333333332, 0.7151666666666667, 0.00305],
                [0.2561, 0.7014999999999999, 0.25305],
                [0.2439, 0.7848333333333333, 0.49695],
                [0.10473333333333333, 0.7547833333333333, 0.6402],
                [0.06193333333333333, 0.67145, 0.6098],
                [0.2286, 0.7452166666666667, 0.8598],
                [0.27140000000000003, 0.82855, 0.8902],
                [0.053233333333333334, 0.7402166666666666, 0.6652],
                [0.11343333333333333, 0.82355, 0.5848],
                [0.2801, 0.7597833333333334, 0.8348],
                [0.21989999999999998, 0.67645, 0.9152],
                [0.10366666666666667, 0.7993, 0.6910000000000001],
                [0.063, 0.7159666666666666, 0.5589999999999999],
                [0.22966666666666666, 0.7007, 0.8089999999999999],
                [0.27033333333333337, 0.7840333333333334, 0.9410000000000001],
                [0.15363333333333332, 0.81295, 0.6643],
                [0.013033333333333334, 0.7296166666666667, 0.5857],
                [0.1797, 0.68705, 0.8357],
                [0.3203, 0.7703833333333333, 0.9143],
                [0.026766666666666664, 0.7416999999999999, 0.7315],
                [0.1399, 0.8250333333333333, 0.5185],
                [0.30656666666666665, 0.7583000000000001, 0.7685],
                [0.19343333333333335, 0.6749666666666667, 0.9815],
                [0.04326666666666667, 0.7560166666666667, 0.7826],
                [0.1234, 0.6726833333333334, 0.9674],
                [0.29006666666666664, 0.7439833333333333, 0.7174],
                [0.20993333333333333, 0.8273166666666666, 0.5326],
                [0.09250000000000001, 0.81265, 0.7981],
                [0.07416666666666667, 0.7293166666666666, 0.9519],
                [0.24083333333333334, 0.68735, 0.7019],
                [0.25916666666666666, 0.7706833333333334, 0.5481],
                [0.07723333333333333, 0.7985000000000001, 0.74695],
                [0.08943333333333332, 0.7151666666666667, 0.50305],
                [0.2561, 0.7014999999999999, 0.75305],
                [0.2439, 0.7848333333333333, 0.99695],
                [0.10473333333333333, 0.92145, 0.1402],
                [0.06193333333333333, 0.8381166666666666, 0.1098],
                [0.2286, 0.9118833333333334, 0.3598],
                [0.27140000000000003, 0.9952166666666667, 0.3902],
                [0.053233333333333334, 0.9068833333333334, 0.1652],
                [0.11343333333333333, 0.9902166666666666, 0.0848],
                [0.2801, 0.92645, 0.3348],
                [0.21989999999999998, 0.8431166666666666, 0.4152],
                [0.10366666666666667, 0.9659666666666666, 0.191],
                [0.063, 0.8826333333333333, 0.059],
                [0.22966666666666666, 0.8673666666666667, 0.309],
                [0.27033333333333337, 0.9507, 0.441],
                [0.15363333333333332, 0.9796166666666667, 0.1643],
                [0.013033333333333334, 0.8962833333333333, 0.0857],
                [0.1797, 0.8537166666666667, 0.3357],
                [0.3203, 0.93705, 0.4143],
                [0.026766666666666664, 0.9083666666666667, 0.2315],
                [0.1399, 0.9916999999999999, 0.0185],
                [0.30656666666666665, 0.9249666666666667, 0.2685],
                [0.19343333333333335, 0.8416333333333333, 0.4815],
                [0.04326666666666667, 0.9226833333333334, 0.2826],
                [0.1234, 0.83935, 0.4674],
                [0.29006666666666664, 0.91065, 0.2174],
                [0.20993333333333333, 0.9939833333333333, 0.0326],
                [0.09250000000000001, 0.9793166666666666, 0.2981],
                [0.07416666666666667, 0.8959833333333332, 0.4519],
                [0.24083333333333334, 0.8540166666666668, 0.2019],
                [0.25916666666666666, 0.93735, 0.0481],
                [0.07723333333333333, 0.9651666666666667, 0.24695],
                [0.08943333333333332, 0.8818333333333334, 0.00305],
                [0.2561, 0.8681666666666666, 0.25305],
                [0.2439, 0.9514999999999999, 0.49695],
                [0.10473333333333333, 0.92145, 0.6402],
                [0.06193333333333333, 0.8381166666666666, 0.6098],
                [0.2286, 0.9118833333333334, 0.8598],
                [0.27140000000000003, 0.9952166666666667, 0.8902],
                [0.053233333333333334, 0.9068833333333334, 0.6652],
                [0.11343333333333333, 0.9902166666666666, 0.5848],
                [0.2801, 0.92645, 0.8348],
                [0.21989999999999998, 0.8431166666666666, 0.9152],
                [0.10366666666666667, 0.9659666666666666, 0.6910000000000001],
                [0.063, 0.8826333333333333, 0.5589999999999999],
                [0.22966666666666666, 0.8673666666666667, 0.8089999999999999],
                [0.27033333333333337, 0.9507, 0.9410000000000001],
                [0.15363333333333332, 0.9796166666666667, 0.6643],
                [0.013033333333333334, 0.8962833333333333, 0.5857],
                [0.1797, 0.8537166666666667, 0.8357],
                [0.3203, 0.93705, 0.9143],
                [0.026766666666666664, 0.9083666666666667, 0.7315],
                [0.1399, 0.9916999999999999, 0.5185],
                [0.30656666666666665, 0.9249666666666667, 0.7685],
                [0.19343333333333335, 0.8416333333333333, 0.9815],
                [0.04326666666666667, 0.9226833333333334, 0.7826],
                [0.1234, 0.83935, 0.9674],
                [0.29006666666666664, 0.91065, 0.7174],
                [0.20993333333333333, 0.9939833333333333, 0.5326],
                [0.09250000000000001, 0.9793166666666666, 0.7981],
                [0.07416666666666667, 0.8959833333333332, 0.9519],
                [0.24083333333333334, 0.8540166666666668, 0.7019],
                [0.25916666666666666, 0.93735, 0.5481],
                [0.07723333333333333, 0.9651666666666667, 0.74695],
                [0.08943333333333332, 0.8818333333333334, 0.50305],
                [0.2561, 0.8681666666666666, 0.75305],
                [0.2439, 0.9514999999999999, 0.99695],
                [0.43806666666666666, 0.08811666666666666, 0.1402],
                [0.39526666666666666, 0.004783333333333333, 0.1098],
                [0.5619333333333333, 0.07855, 0.3598],
                [0.6047333333333333, 0.16188333333333335, 0.3902],
                [0.38656666666666667, 0.07355, 0.1652],
                [0.4467666666666667, 0.15688333333333335, 0.0848],
                [0.6134333333333334, 0.09311666666666667, 0.3348],
                [0.5532333333333334, 0.009783333333333333, 0.4152],
                [0.437, 0.13263333333333333, 0.191],
                [0.39633333333333337, 0.049300000000000004, 0.059],
                [0.5630000000000001, 0.03403333333333333, 0.309],
                [0.6036666666666667, 0.11736666666666667, 0.441],
                [0.4869666666666667, 0.14628333333333335, 0.1643],
                [0.34636666666666666, 0.06294999999999999, 0.0857],
                [0.5130333333333333, 0.020383333333333333, 0.3357],
                [0.6536333333333334, 0.10371666666666667, 0.4143],
                [0.36010000000000003, 0.07503333333333333, 0.2315],
                [0.47323333333333334, 0.15836666666666668, 0.0185],
                [0.6399, 0.09163333333333333, 0.2685],
                [0.5267666666666667, 0.0083, 0.4815],
                [0.3766, 0.08935, 0.2826],
                [0.4567333333333334, 0.006016666666666667, 0.4674],
                [0.6234000000000001, 0.07731666666666666, 0.2174],
                [0.5432666666666667, 0.16065, 0.0326],
                [0.42583333333333334, 0.14598333333333333, 0.2981],
                [0.4075, 0.06265, 0.4519],
                [0.5741666666666667, 0.020683333333333335, 0.2019],
                [0.5924999999999999, 0.10401666666666666, 0.0481],
                [0.4105666666666667, 0.13183333333333333, 0.24695],
                [0.4227666666666667, 0.048499999999999995, 0.00305],
                [0.5894333333333334, 0.034833333333333334, 0.25305],
                [0.5772333333333334, 0.11816666666666666, 0.49695],
                [0.43806666666666666, 0.08811666666666666, 0.6402],
                [0.39526666666666666, 0.004783333333333333, 0.6098],
                [0.5619333333333333, 0.07855, 0.8598],
                [0.6047333333333333, 0.16188333333333335, 0.8902],
                [0.38656666666666667, 0.07355, 0.6652],
                [0.4467666666666667, 0.15688333333333335, 0.5848],
                [0.6134333333333334, 0.09311666666666667, 0.8348],
                [0.5532333333333334, 0.009783333333333333, 0.9152],
                [0.437, 0.13263333333333333, 0.6910000000000001],
                [0.39633333333333337, 0.049300000000000004, 0.5589999999999999],
                [0.5630000000000001, 0.03403333333333333, 0.8089999999999999],
                [0.6036666666666667, 0.11736666666666667, 0.9410000000000001],
                [0.4869666666666667, 0.14628333333333335, 0.6643],
                [0.34636666666666666, 0.06294999999999999, 0.5857],
                [0.5130333333333333, 0.020383333333333333, 0.8357],
                [0.6536333333333334, 0.10371666666666667, 0.9143],
                [0.36010000000000003, 0.07503333333333333, 0.7315],
                [0.47323333333333334, 0.15836666666666668, 0.5185],
                [0.6399, 0.09163333333333333, 0.7685],
                [0.5267666666666667, 0.0083, 0.9815],
                [0.3766, 0.08935, 0.7826],
                [0.4567333333333334, 0.006016666666666667, 0.9674],
                [0.6234000000000001, 0.07731666666666666, 0.7174],
                [0.5432666666666667, 0.16065, 0.5326],
                [0.42583333333333334, 0.14598333333333333, 0.7981],
                [0.4075, 0.06265, 0.9519],
                [0.5741666666666667, 0.020683333333333335, 0.7019],
                [0.5924999999999999, 0.10401666666666666, 0.5481],
                [0.4105666666666667, 0.13183333333333333, 0.74695],
                [0.4227666666666667, 0.048499999999999995, 0.50305],
                [0.5894333333333334, 0.034833333333333334, 0.75305],
                [0.5772333333333334, 0.11816666666666666, 0.99695],
                [0.43806666666666666, 0.2547833333333333, 0.1402],
                [0.39526666666666666, 0.17145, 0.1098],
                [0.5619333333333333, 0.24521666666666667, 0.3598],
                [0.6047333333333333, 0.32855, 0.3902],
                [0.38656666666666667, 0.24021666666666666, 0.1652],
                [0.4467666666666667, 0.32355, 0.0848],
                [0.6134333333333334, 0.2597833333333333, 0.3348],
                [0.5532333333333334, 0.17645, 0.4152],
                [0.437, 0.29929999999999995, 0.191],
                [0.39633333333333337, 0.21596666666666667, 0.059],
                [0.5630000000000001, 0.2007, 0.309],
                [0.6036666666666667, 0.28403333333333336, 0.441],
                [0.4869666666666667, 0.31295, 0.1643],
                [0.34636666666666666, 0.22961666666666666, 0.0857],
                [0.5130333333333333, 0.18705000000000002, 0.3357],
                [0.6536333333333334, 0.27038333333333336, 0.4143],
                [0.36010000000000003, 0.2417, 0.2315],
                [0.47323333333333334, 0.32503333333333334, 0.0185],
                [0.6399, 0.2583, 0.2685],
                [0.5267666666666667, 0.1749666666666667, 0.4815],
                [0.3766, 0.25601666666666667, 0.2826],
                [0.4567333333333334, 0.17268333333333333, 0.4674],
                [0.6234000000000001, 0.24398333333333333, 0.2174],
                [0.5432666666666667, 0.32731666666666664, 0.0326],
                [0.42583333333333334, 0.31265000000000004, 0.2981],
                [0.4075, 0.2293166666666667, 0.4519],
                [0.5741666666666667, 0.18735000000000002, 0.2019],
                [0.5924999999999999, 0.27068333333333333, 0.0481],
                [0.4105666666666667, 0.2985, 0.24695],
                [0.4227666666666667, 0.21516666666666664, 0.00305],
                [0.5894333333333334, 0.2015, 0.25305],
                [0.5772333333333334, 0.2848333333333333, 0.49695],
                [0.43806666666666666, 0.2547833333333333, 0.6402],
                [0.39526666666666666, 0.17145, 0.6098],
                [0.5619333333333333, 0.24521666666666667, 0.8598],
                [0.6047333333333333, 0.32855, 0.8902],
                [0.38656666666666667, 0.24021666666666666, 0.6652],
                [0.4467666666666667, 0.32355, 0.5848],
                [0.6134333333333334, 0.2597833333333333, 0.8348],
                [0.5532333333333334, 0.17645, 0.9152],
                [0.437, 0.29929999999999995, 0.6910000000000001],
                [0.39633333333333337, 0.21596666666666667, 0.5589999999999999],
                [0.5630000000000001, 0.2007, 0.8089999999999999],
                [0.6036666666666667, 0.28403333333333336, 0.9410000000000001],
                [0.4869666666666667, 0.31295, 0.6643],
                [0.34636666666666666, 0.22961666666666666, 0.5857],
                [0.5130333333333333, 0.18705000000000002, 0.8357],
                [0.6536333333333334, 0.27038333333333336, 0.9143],
                [0.36010000000000003, 0.2417, 0.7315],
                [0.47323333333333334, 0.32503333333333334, 0.5185],
                [0.6399, 0.2583, 0.7685],
                [0.5267666666666667, 0.1749666666666667, 0.9815],
                [0.3766, 0.25601666666666667, 0.7826],
                [0.4567333333333334, 0.17268333333333333, 0.9674],
                [0.6234000000000001, 0.24398333333333333, 0.7174],
                [0.5432666666666667, 0.32731666666666664, 0.5326],
                [0.42583333333333334, 0.31265000000000004, 0.7981],
                [0.4075, 0.2293166666666667, 0.9519],
                [0.5741666666666667, 0.18735000000000002, 0.7019],
                [0.5924999999999999, 0.27068333333333333, 0.5481],
                [0.4105666666666667, 0.2985, 0.74695],
                [0.4227666666666667, 0.21516666666666664, 0.50305],
                [0.5894333333333334, 0.2015, 0.75305],
                [0.5772333333333334, 0.2848333333333333, 0.99695],
                [0.43806666666666666, 0.42144999999999994, 0.1402],
                [0.39526666666666666, 0.3381166666666667, 0.1098],
                [0.5619333333333333, 0.4118833333333333, 0.3598],
                [0.6047333333333333, 0.4952166666666667, 0.3902],
                [0.38656666666666667, 0.4068833333333333, 0.1652],
                [0.4467666666666667, 0.4902166666666667, 0.0848],
                [0.6134333333333334, 0.42645, 0.3348],
                [0.5532333333333334, 0.3431166666666667, 0.4152],
                [0.437, 0.46596666666666664, 0.191],
                [0.39633333333333337, 0.3826333333333333, 0.059],
                [0.5630000000000001, 0.3673666666666667, 0.309],
                [0.6036666666666667, 0.45070000000000005, 0.441],
                [0.4869666666666667, 0.47961666666666664, 0.1643],
                [0.34636666666666666, 0.3962833333333333, 0.0857],
                [0.5130333333333333, 0.3537166666666667, 0.3357],
                [0.6536333333333334, 0.43705, 0.4143],
                [0.36010000000000003, 0.4083666666666667, 0.2315],
                [0.47323333333333334, 0.4917, 0.0185],
                [0.6399, 0.42496666666666666, 0.2685],
                [0.5267666666666667, 0.3416333333333333, 0.4815],
                [0.3766, 0.42268333333333336, 0.2826],
                [0.4567333333333334, 0.33935, 0.4674],
                [0.6234000000000001, 0.41064999999999996, 0.2174],
                [0.5432666666666667, 0.4939833333333333, 0.0326],
                [0.42583333333333334, 0.47931666666666667, 0.2981],
                [0.4075, 0.39598333333333335, 0.4519],
                [0.5741666666666667, 0.35401666666666665, 0.2019],
                [0.5924999999999999, 0.43734999999999996, 0.0481],
                [0.4105666666666667, 0.4651666666666667, 0.24695],
                [0.4227666666666667, 0.3818333333333333, 0.00305],
                [0.5894333333333334, 0.3681666666666667, 0.25305],
                [0.5772333333333334, 0.4515, 0.49695],
                [0.43806666666666666, 0.42144999999999994, 0.6402],
                [0.39526666666666666, 0.3381166666666667, 0.6098],
                [0.5619333333333333, 0.4118833333333333, 0.8598],
                [0.6047333333333333, 0.4952166666666667, 0.8902],
                [0.38656666666666667, 0.4068833333333333, 0.6652],
                [0.4467666666666667, 0.4902166666666667, 0.5848],
                [0.6134333333333334, 0.42645, 0.8348],
                [0.5532333333333334, 0.3431166666666667, 0.9152],
                [0.437, 0.46596666666666664, 0.6910000000000001],
                [0.39633333333333337, 0.3826333333333333, 0.5589999999999999],
                [0.5630000000000001, 0.3673666666666667, 0.8089999999999999],
                [0.6036666666666667, 0.45070000000000005, 0.9410000000000001],
                [0.4869666666666667, 0.47961666666666664, 0.6643],
                [0.34636666666666666, 0.3962833333333333, 0.5857],
                [0.5130333333333333, 0.3537166666666667, 0.8357],
                [0.6536333333333334, 0.43705, 0.9143],
                [0.36010000000000003, 0.4083666666666667, 0.7315],
                [0.47323333333333334, 0.4917, 0.5185],
                [0.6399, 0.42496666666666666, 0.7685],
                [0.5267666666666667, 0.3416333333333333, 0.9815],
                [0.3766, 0.42268333333333336, 0.7826],
                [0.4567333333333334, 0.33935, 0.9674],
                [0.6234000000000001, 0.41064999999999996, 0.7174],
                [0.5432666666666667, 0.4939833333333333, 0.5326],
                [0.42583333333333334, 0.47931666666666667, 0.7981],
                [0.4075, 0.39598333333333335, 0.9519],
                [0.5741666666666667, 0.35401666666666665, 0.7019],
                [0.5924999999999999, 0.43734999999999996, 0.5481],
                [0.4105666666666667, 0.4651666666666667, 0.74695],
                [0.4227666666666667, 0.3818333333333333, 0.50305],
                [0.5894333333333334, 0.3681666666666667, 0.75305],
                [0.5772333333333334, 0.4515, 0.99695],
                [0.43806666666666666, 0.5881166666666666, 0.1402],
                [0.39526666666666666, 0.5047833333333334, 0.1098],
                [0.5619333333333333, 0.57855, 0.3598],
                [0.6047333333333333, 0.6618833333333334, 0.3902],
                [0.38656666666666667, 0.57355, 0.1652],
                [0.4467666666666667, 0.6568833333333334, 0.0848],
                [0.6134333333333334, 0.5931166666666666, 0.3348],
                [0.5532333333333334, 0.5097833333333334, 0.4152],
                [0.437, 0.6326333333333333, 0.191],
                [0.39633333333333337, 0.5493, 0.059],
                [0.5630000000000001, 0.5340333333333334, 0.309],
                [0.6036666666666667, 0.6173666666666667, 0.441],
                [0.4869666666666667, 0.6462833333333333, 0.1643],
                [0.34636666666666666, 0.56295, 0.0857],
                [0.5130333333333333, 0.5203833333333333, 0.3357],
                [0.6536333333333334, 0.6037166666666667, 0.4143],
                [0.36010000000000003, 0.5750333333333334, 0.2315],
                [0.47323333333333334, 0.6583666666666667, 0.0185],
                [0.6399, 0.5916333333333333, 0.2685],
                [0.5267666666666667, 0.5083, 0.4815],
                [0.3766, 0.58935, 0.2826],
                [0.4567333333333334, 0.5060166666666667, 0.4674],
                [0.6234000000000001, 0.5773166666666666, 0.2174],
                [0.5432666666666667, 0.66065, 0.0326],
                [0.42583333333333334, 0.6459833333333334, 0.2981],
                [0.4075, 0.56265, 0.4519],
                [0.5741666666666667, 0.5206833333333333, 0.2019],
                [0.5924999999999999, 0.6040166666666666, 0.0481],
                [0.4105666666666667, 0.6318333333333334, 0.24695],
                [0.4227666666666667, 0.5485, 0.00305],
                [0.5894333333333334, 0.5348333333333334, 0.25305],
                [0.5772333333333334, 0.6181666666666666, 0.49695],
                [0.43806666666666666, 0.5881166666666666, 0.6402],
                [0.39526666666666666, 0.5047833333333334, 0.6098],
                [0.5619333333333333, 0.57855, 0.8598],
                [0.6047333333333333, 0.6618833333333334, 0.8902],
                [0.38656666666666667, 0.57355, 0.6652],
                [0.4467666666666667, 0.6568833333333334, 0.5848],
                [0.6134333333333334, 0.5931166666666666, 0.8348],
                [0.5532333333333334, 0.5097833333333334, 0.9152],
                [0.437, 0.6326333333333333, 0.6910000000000001],
                [0.39633333333333337, 0.5493, 0.5589999999999999],
                [0.5630000000000001, 0.5340333333333334, 0.8089999999999999],
                [0.6036666666666667, 0.6173666666666667, 0.9410000000000001],
                [0.4869666666666667, 0.6462833333333333, 0.6643],
                [0.34636666666666666, 0.56295, 0.5857],
                [0.5130333333333333, 0.5203833333333333, 0.8357],
                [0.6536333333333334, 0.6037166666666667, 0.9143],
                [0.36010000000000003, 0.5750333333333334, 0.7315],
                [0.47323333333333334, 0.6583666666666667, 0.5185],
                [0.6399, 0.5916333333333333, 0.7685],
                [0.5267666666666667, 0.5083, 0.9815],
                [0.3766, 0.58935, 0.7826],
                [0.4567333333333334, 0.5060166666666667, 0.9674],
                [0.6234000000000001, 0.5773166666666666, 0.7174],
                [0.5432666666666667, 0.66065, 0.5326],
                [0.42583333333333334, 0.6459833333333334, 0.7981],
                [0.4075, 0.56265, 0.9519],
                [0.5741666666666667, 0.5206833333333333, 0.7019],
                [0.5924999999999999, 0.6040166666666666, 0.5481],
                [0.4105666666666667, 0.6318333333333334, 0.74695],
                [0.4227666666666667, 0.5485, 0.50305],
                [0.5894333333333334, 0.5348333333333334, 0.75305],
                [0.5772333333333334, 0.6181666666666666, 0.99695],
                [0.43806666666666666, 0.7547833333333333, 0.1402],
                [0.39526666666666666, 0.67145, 0.1098],
                [0.5619333333333333, 0.7452166666666667, 0.3598],
                [0.6047333333333333, 0.82855, 0.3902],
                [0.38656666666666667, 0.7402166666666666, 0.1652],
                [0.4467666666666667, 0.82355, 0.0848],
                [0.6134333333333334, 0.7597833333333334, 0.3348],
                [0.5532333333333334, 0.67645, 0.4152],
                [0.437, 0.7993, 0.191],
                [0.39633333333333337, 0.7159666666666666, 0.059],
                [0.5630000000000001, 0.7007, 0.309],
                [0.6036666666666667, 0.7840333333333334, 0.441],
                [0.4869666666666667, 0.81295, 0.1643],
                [0.34636666666666666, 0.7296166666666667, 0.0857],
                [0.5130333333333333, 0.68705, 0.3357],
                [0.6536333333333334, 0.7703833333333333, 0.4143],
                [0.36010000000000003, 0.7416999999999999, 0.2315],
                [0.47323333333333334, 0.8250333333333333, 0.0185],
                [0.6399, 0.7583000000000001, 0.2685],
                [0.5267666666666667, 0.6749666666666667, 0.4815],
                [0.3766, 0.7560166666666667, 0.2826],
                [0.4567333333333334, 0.6726833333333334, 0.4674],
                [0.6234000000000001, 0.7439833333333333, 0.2174],
                [0.5432666666666667, 0.8273166666666666, 0.0326],
                [0.42583333333333334, 0.81265, 0.2981],
                [0.4075, 0.7293166666666666, 0.4519],
                [0.5741666666666667, 0.68735, 0.2019],
                [0.5924999999999999, 0.7706833333333334, 0.0481],
                [0.4105666666666667, 0.7985000000000001, 0.24695],
                [0.4227666666666667, 0.7151666666666667, 0.00305],
                [0.5894333333333334, 0.7014999999999999, 0.25305],
                [0.5772333333333334, 0.7848333333333333, 0.49695],
                [0.43806666666666666, 0.7547833333333333, 0.6402],
                [0.39526666666666666, 0.67145, 0.6098],
                [0.5619333333333333, 0.7452166666666667, 0.8598],
                [0.6047333333333333, 0.82855, 0.8902],
                [0.38656666666666667, 0.7402166666666666, 0.6652],
                [0.4467666666666667, 0.82355, 0.5848],
                [0.6134333333333334, 0.7597833333333334, 0.8348],
                [0.5532333333333334, 0.67645, 0.9152],
                [0.437, 0.7993, 0.6910000000000001],
                [0.39633333333333337, 0.7159666666666666, 0.5589999999999999],
                [0.5630000000000001, 0.7007, 0.8089999999999999],
                [0.6036666666666667, 0.7840333333333334, 0.9410000000000001],
                [0.4869666666666667, 0.81295, 0.6643],
                [0.34636666666666666, 0.7296166666666667, 0.5857],
                [0.5130333333333333, 0.68705, 0.8357],
                [0.6536333333333334, 0.7703833333333333, 0.9143],
                [0.36010000000000003, 0.7416999999999999, 0.7315],
                [0.47323333333333334, 0.8250333333333333, 0.5185],
                [0.6399, 0.7583000000000001, 0.7685],
                [0.5267666666666667, 0.6749666666666667, 0.9815],
                [0.3766, 0.7560166666666667, 0.7826],
                [0.4567333333333334, 0.6726833333333334, 0.9674],
                [0.6234000000000001, 0.7439833333333333, 0.7174],
                [0.5432666666666667, 0.8273166666666666, 0.5326],
                [0.42583333333333334, 0.81265, 0.7981],
                [0.4075, 0.7293166666666666, 0.9519],
                [0.5741666666666667, 0.68735, 0.7019],
                [0.5924999999999999, 0.7706833333333334, 0.5481],
                [0.4105666666666667, 0.7985000000000001, 0.74695],
                [0.4227666666666667, 0.7151666666666667, 0.50305],
                [0.5894333333333334, 0.7014999999999999, 0.75305],
                [0.5772333333333334, 0.7848333333333333, 0.99695],
                [0.43806666666666666, 0.92145, 0.1402],
                [0.39526666666666666, 0.8381166666666666, 0.1098],
                [0.5619333333333333, 0.9118833333333334, 0.3598],
                [0.6047333333333333, 0.9952166666666667, 0.3902],
                [0.38656666666666667, 0.9068833333333334, 0.1652],
                [0.4467666666666667, 0.9902166666666666, 0.0848],
                [0.6134333333333334, 0.92645, 0.3348],
                [0.5532333333333334, 0.8431166666666666, 0.4152],
                [0.437, 0.9659666666666666, 0.191],
                [0.39633333333333337, 0.8826333333333333, 0.059],
                [0.5630000000000001, 0.8673666666666667, 0.309],
                [0.6036666666666667, 0.9507, 0.441],
                [0.4869666666666667, 0.9796166666666667, 0.1643],
                [0.34636666666666666, 0.8962833333333333, 0.0857],
                [0.5130333333333333, 0.8537166666666667, 0.3357],
                [0.6536333333333334, 0.93705, 0.4143],
                [0.36010000000000003, 0.9083666666666667, 0.2315],
                [0.47323333333333334, 0.9916999999999999, 0.0185],
                [0.6399, 0.9249666666666667, 0.2685],
                [0.5267666666666667, 0.8416333333333333, 0.4815],
                [0.3766, 0.9226833333333334, 0.2826],
                [0.4567333333333334, 0.83935, 0.4674],
                [0.6234000000000001, 0.91065, 0.2174],
                [0.5432666666666667, 0.9939833333333333, 0.0326],
                [0.42583333333333334, 0.9793166666666666, 0.2981],
                [0.4075, 0.8959833333333332, 0.4519],
                [0.5741666666666667, 0.8540166666666668, 0.2019],
                [0.5924999999999999, 0.93735, 0.0481],
                [0.4105666666666667, 0.9651666666666667, 0.24695],
                [0.4227666666666667, 0.8818333333333334, 0.00305],
                [0.5894333333333334, 0.8681666666666666, 0.25305],
                [0.5772333333333334, 0.9514999999999999, 0.49695],
                [0.43806666666666666, 0.92145, 0.6402],
                [0.39526666666666666, 0.8381166666666666, 0.6098],
                [0.5619333333333333, 0.9118833333333334, 0.8598],
                [0.6047333333333333, 0.9952166666666667, 0.8902],
                [0.38656666666666667, 0.9068833333333334, 0.6652],
                [0.4467666666666667, 0.9902166666666666, 0.5848],
                [0.6134333333333334, 0.92645, 0.8348],
                [0.5532333333333334, 0.8431166666666666, 0.9152],
                [0.437, 0.9659666666666666, 0.6910000000000001],
                [0.39633333333333337, 0.8826333333333333, 0.5589999999999999],
                [0.5630000000000001, 0.8673666666666667, 0.8089999999999999],
                [0.6036666666666667, 0.9507, 0.9410000000000001],
                [0.4869666666666667, 0.9796166666666667, 0.6643],
                [0.34636666666666666, 0.8962833333333333, 0.5857],
                [0.5130333333333333, 0.8537166666666667, 0.8357],
                [0.6536333333333334, 0.93705, 0.9143],
                [0.36010000000000003, 0.9083666666666667, 0.7315],
                [0.47323333333333334, 0.9916999999999999, 0.5185],
                [0.6399, 0.9249666666666667, 0.7685],
                [0.5267666666666667, 0.8416333333333333, 0.9815],
                [0.3766, 0.9226833333333334, 0.7826],
                [0.4567333333333334, 0.83935, 0.9674],
                [0.6234000000000001, 0.91065, 0.7174],
                [0.5432666666666667, 0.9939833333333333, 0.5326],
                [0.42583333333333334, 0.9793166666666666, 0.7981],
                [0.4075, 0.8959833333333332, 0.9519],
                [0.5741666666666667, 0.8540166666666668, 0.7019],
                [0.5924999999999999, 0.93735, 0.5481],
                [0.4105666666666667, 0.9651666666666667, 0.74695],
                [0.4227666666666667, 0.8818333333333334, 0.50305],
                [0.5894333333333334, 0.8681666666666666, 0.75305],
                [0.5772333333333334, 0.9514999999999999, 0.99695],
                [0.7714, 0.08811666666666666, 0.1402],
                [0.7286, 0.004783333333333333, 0.1098],
                [0.8952666666666667, 0.07855, 0.3598],
                [0.9380666666666667, 0.16188333333333335, 0.3902],
                [0.7199, 0.07355, 0.1652],
                [0.7801, 0.15688333333333335, 0.0848],
                [0.9467666666666666, 0.09311666666666667, 0.3348],
                [0.8865666666666666, 0.009783333333333333, 0.4152],
                [0.7703333333333333, 0.13263333333333333, 0.191],
                [0.7296666666666667, 0.049300000000000004, 0.059],
                [0.8963333333333333, 0.03403333333333333, 0.309],
                [0.9369999999999999, 0.11736666666666667, 0.441],
                [0.8203, 0.14628333333333335, 0.1643],
                [0.6797, 0.06294999999999999, 0.0857],
                [0.8463666666666666, 0.020383333333333333, 0.3357],
                [0.9869666666666667, 0.10371666666666667, 0.4143],
                [0.6934333333333332, 0.07503333333333333, 0.2315],
                [0.8065666666666668, 0.15836666666666668, 0.0185],
                [0.9732333333333333, 0.09163333333333333, 0.2685],
                [0.8601000000000001, 0.0083, 0.4815],
                [0.7099333333333333, 0.08935, 0.2826],
                [0.7900666666666667, 0.006016666666666667, 0.4674],
                [0.9567333333333333, 0.07731666666666666, 0.2174],
                [0.8765999999999999, 0.16065, 0.0326],
                [0.7591666666666667, 0.14598333333333333, 0.2981],
                [0.7408333333333333, 0.06265, 0.4519],
                [0.9075000000000001, 0.020683333333333335, 0.2019],
                [0.9258333333333333, 0.10401666666666666, 0.0481],
                [0.7439, 0.13183333333333333, 0.24695],
                [0.7561, 0.048499999999999995, 0.00305],
                [0.9227666666666666, 0.034833333333333334, 0.25305],
                [0.9105666666666666, 0.11816666666666666, 0.49695],
                [0.7714, 0.08811666666666666, 0.6402],
                [0.7286, 0.004783333333333333, 0.6098],
                [0.8952666666666667, 0.07855, 0.8598],
                [0.9380666666666667, 0.16188333333333335, 0.8902],
                [0.7199, 0.07355, 0.6652],
                [0.7801, 0.15688333333333335, 0.5848],
                [0.9467666666666666, 0.09311666666666667, 0.8348],
                [0.8865666666666666, 0.009783333333333333, 0.9152],
                [0.7703333333333333, 0.13263333333333333, 0.6910000000000001],
                [0.7296666666666667, 0.049300000000000004, 0.5589999999999999],
                [0.8963333333333333, 0.03403333333333333, 0.8089999999999999],
                [0.9369999999999999, 0.11736666666666667, 0.9410000000000001],
                [0.8203, 0.14628333333333335, 0.6643],
                [0.6797, 0.06294999999999999, 0.5857],
                [0.8463666666666666, 0.020383333333333333, 0.8357],
                [0.9869666666666667, 0.10371666666666667, 0.9143],
                [0.6934333333333332, 0.07503333333333333, 0.7315],
                [0.8065666666666668, 0.15836666666666668, 0.5185],
                [0.9732333333333333, 0.09163333333333333, 0.7685],
                [0.8601000000000001, 0.0083, 0.9815],
                [0.7099333333333333, 0.08935, 0.7826],
                [0.7900666666666667, 0.006016666666666667, 0.9674],
                [0.9567333333333333, 0.07731666666666666, 0.7174],
                [0.8765999999999999, 0.16065, 0.5326],
                [0.7591666666666667, 0.14598333333333333, 0.7981],
                [0.7408333333333333, 0.06265, 0.9519],
                [0.9075000000000001, 0.020683333333333335, 0.7019],
                [0.9258333333333333, 0.10401666666666666, 0.5481],
                [0.7439, 0.13183333333333333, 0.74695],
                [0.7561, 0.048499999999999995, 0.50305],
                [0.9227666666666666, 0.034833333333333334, 0.75305],
                [0.9105666666666666, 0.11816666666666666, 0.99695],
                [0.7714, 0.2547833333333333, 0.1402],
                [0.7286, 0.17145, 0.1098],
                [0.8952666666666667, 0.24521666666666667, 0.3598],
                [0.9380666666666667, 0.32855, 0.3902],
                [0.7199, 0.24021666666666666, 0.1652],
                [0.7801, 0.32355, 0.0848],
                [0.9467666666666666, 0.2597833333333333, 0.3348],
                [0.8865666666666666, 0.17645, 0.4152],
                [0.7703333333333333, 0.29929999999999995, 0.191],
                [0.7296666666666667, 0.21596666666666667, 0.059],
                [0.8963333333333333, 0.2007, 0.309],
                [0.9369999999999999, 0.28403333333333336, 0.441],
                [0.8203, 0.31295, 0.1643],
                [0.6797, 0.22961666666666666, 0.0857],
                [0.8463666666666666, 0.18705000000000002, 0.3357],
                [0.9869666666666667, 0.27038333333333336, 0.4143],
                [0.6934333333333332, 0.2417, 0.2315],
                [0.8065666666666668, 0.32503333333333334, 0.0185],
                [0.9732333333333333, 0.2583, 0.2685],
                [0.8601000000000001, 0.1749666666666667, 0.4815],
                [0.7099333333333333, 0.25601666666666667, 0.2826],
                [0.7900666666666667, 0.17268333333333333, 0.4674],
                [0.9567333333333333, 0.24398333333333333, 0.2174],
                [0.8765999999999999, 0.32731666666666664, 0.0326],
                [0.7591666666666667, 0.31265000000000004, 0.2981],
                [0.7408333333333333, 0.2293166666666667, 0.4519],
                [0.9075000000000001, 0.18735000000000002, 0.2019],
                [0.9258333333333333, 0.27068333333333333, 0.0481],
                [0.7439, 0.2985, 0.24695],
                [0.7561, 0.21516666666666664, 0.00305],
                [0.9227666666666666, 0.2015, 0.25305],
                [0.9105666666666666, 0.2848333333333333, 0.49695],
                [0.7714, 0.2547833333333333, 0.6402],
                [0.7286, 0.17145, 0.6098],
                [0.8952666666666667, 0.24521666666666667, 0.8598],
                [0.9380666666666667, 0.32855, 0.8902],
                [0.7199, 0.24021666666666666, 0.6652],
                [0.7801, 0.32355, 0.5848],
                [0.9467666666666666, 0.2597833333333333, 0.8348],
                [0.8865666666666666, 0.17645, 0.9152],
                [0.7703333333333333, 0.29929999999999995, 0.6910000000000001],
                [0.7296666666666667, 0.21596666666666667, 0.5589999999999999],
                [0.8963333333333333, 0.2007, 0.8089999999999999],
                [0.9369999999999999, 0.28403333333333336, 0.9410000000000001],
                [0.8203, 0.31295, 0.6643],
                [0.6797, 0.22961666666666666, 0.5857],
                [0.8463666666666666, 0.18705000000000002, 0.8357],
                [0.9869666666666667, 0.27038333333333336, 0.9143],
                [0.6934333333333332, 0.2417, 0.7315],
                [0.8065666666666668, 0.32503333333333334, 0.5185],
                [0.9732333333333333, 0.2583, 0.7685],
                [0.8601000000000001, 0.1749666666666667, 0.9815],
                [0.7099333333333333, 0.25601666666666667, 0.7826],
                [0.7900666666666667, 0.17268333333333333, 0.9674],
                [0.9567333333333333, 0.24398333333333333, 0.7174],
                [0.8765999999999999, 0.32731666666666664, 0.5326],
                [0.7591666666666667, 0.31265000000000004, 0.7981],
                [0.7408333333333333, 0.2293166666666667, 0.9519],
                [0.9075000000000001, 0.18735000000000002, 0.7019],
                [0.9258333333333333, 0.27068333333333333, 0.5481],
                [0.7439, 0.2985, 0.74695],
                [0.7561, 0.21516666666666664, 0.50305],
                [0.9227666666666666, 0.2015, 0.75305],
                [0.9105666666666666, 0.2848333333333333, 0.99695],
                [0.7714, 0.42144999999999994, 0.1402],
                [0.7286, 0.3381166666666667, 0.1098],
                [0.8952666666666667, 0.4118833333333333, 0.3598],
                [0.9380666666666667, 0.4952166666666667, 0.3902],
                [0.7199, 0.4068833333333333, 0.1652],
                [0.7801, 0.4902166666666667, 0.0848],
                [0.9467666666666666, 0.42645, 0.3348],
                [0.8865666666666666, 0.3431166666666667, 0.4152],
                [0.7703333333333333, 0.46596666666666664, 0.191],
                [0.7296666666666667, 0.3826333333333333, 0.059],
                [0.8963333333333333, 0.3673666666666667, 0.309],
                [0.9369999999999999, 0.45070000000000005, 0.441],
                [0.8203, 0.47961666666666664, 0.1643],
                [0.6797, 0.3962833333333333, 0.0857],
                [0.8463666666666666, 0.3537166666666667, 0.3357],
                [0.9869666666666667, 0.43705, 0.4143],
                [0.6934333333333332, 0.4083666666666667, 0.2315],
                [0.8065666666666668, 0.4917, 0.0185],
                [0.9732333333333333, 0.42496666666666666, 0.2685],
                [0.8601000000000001, 0.3416333333333333, 0.4815],
                [0.7099333333333333, 0.42268333333333336, 0.2826],
                [0.7900666666666667, 0.33935, 0.4674],
                [0.9567333333333333, 0.41064999999999996, 0.2174],
                [0.8765999999999999, 0.4939833333333333, 0.0326],
                [0.7591666666666667, 0.47931666666666667, 0.2981],
                [0.7408333333333333, 0.39598333333333335, 0.4519],
                [0.9075000000000001, 0.35401666666666665, 0.2019],
                [0.9258333333333333, 0.43734999999999996, 0.0481],
                [0.7439, 0.4651666666666667, 0.24695],
                [0.7561, 0.3818333333333333, 0.00305],
                [0.9227666666666666, 0.3681666666666667, 0.25305],
                [0.9105666666666666, 0.4515, 0.49695],
                [0.7714, 0.42144999999999994, 0.6402],
                [0.7286, 0.3381166666666667, 0.6098],
                [0.8952666666666667, 0.4118833333333333, 0.8598],
                [0.9380666666666667, 0.4952166666666667, 0.8902],
                [0.7199, 0.4068833333333333, 0.6652],
                [0.7801, 0.4902166666666667, 0.5848],
                [0.9467666666666666, 0.42645, 0.8348],
                [0.8865666666666666, 0.3431166666666667, 0.9152],
                [0.7703333333333333, 0.46596666666666664, 0.6910000000000001],
                [0.7296666666666667, 0.3826333333333333, 0.5589999999999999],
                [0.8963333333333333, 0.3673666666666667, 0.8089999999999999],
                [0.9369999999999999, 0.45070000000000005, 0.9410000000000001],
                [0.8203, 0.47961666666666664, 0.6643],
                [0.6797, 0.3962833333333333, 0.5857],
                [0.8463666666666666, 0.3537166666666667, 0.8357],
                [0.9869666666666667, 0.43705, 0.9143],
                [0.6934333333333332, 0.4083666666666667, 0.7315],
                [0.8065666666666668, 0.4917, 0.5185],
                [0.9732333333333333, 0.42496666666666666, 0.7685],
                [0.8601000000000001, 0.3416333333333333, 0.9815],
                [0.7099333333333333, 0.42268333333333336, 0.7826],
                [0.7900666666666667, 0.33935, 0.9674],
                [0.9567333333333333, 0.41064999999999996, 0.7174],
                [0.8765999999999999, 0.4939833333333333, 0.5326],
                [0.7591666666666667, 0.47931666666666667, 0.7981],
                [0.7408333333333333, 0.39598333333333335, 0.9519],
                [0.9075000000000001, 0.35401666666666665, 0.7019],
                [0.9258333333333333, 0.43734999999999996, 0.5481],
                [0.7439, 0.4651666666666667, 0.74695],
                [0.7561, 0.3818333333333333, 0.50305],
                [0.9227666666666666, 0.3681666666666667, 0.75305],
                [0.9105666666666666, 0.4515, 0.99695],
                [0.7714, 0.5881166666666666, 0.1402],
                [0.7286, 0.5047833333333334, 0.1098],
                [0.8952666666666667, 0.57855, 0.3598],
                [0.9380666666666667, 0.6618833333333334, 0.3902],
                [0.7199, 0.57355, 0.1652],
                [0.7801, 0.6568833333333334, 0.0848],
                [0.9467666666666666, 0.5931166666666666, 0.3348],
                [0.8865666666666666, 0.5097833333333334, 0.4152],
                [0.7703333333333333, 0.6326333333333333, 0.191],
                [0.7296666666666667, 0.5493, 0.059],
                [0.8963333333333333, 0.5340333333333334, 0.309],
                [0.9369999999999999, 0.6173666666666667, 0.441],
                [0.8203, 0.6462833333333333, 0.1643],
                [0.6797, 0.56295, 0.0857],
                [0.8463666666666666, 0.5203833333333333, 0.3357],
                [0.9869666666666667, 0.6037166666666667, 0.4143],
                [0.6934333333333332, 0.5750333333333334, 0.2315],
                [0.8065666666666668, 0.6583666666666667, 0.0185],
                [0.9732333333333333, 0.5916333333333333, 0.2685],
                [0.8601000000000001, 0.5083, 0.4815],
                [0.7099333333333333, 0.58935, 0.2826],
                [0.7900666666666667, 0.5060166666666667, 0.4674],
                [0.9567333333333333, 0.5773166666666666, 0.2174],
                [0.8765999999999999, 0.66065, 0.0326],
                [0.7591666666666667, 0.6459833333333334, 0.2981],
                [0.7408333333333333, 0.56265, 0.4519],
                [0.9075000000000001, 0.5206833333333333, 0.2019],
                [0.9258333333333333, 0.6040166666666666, 0.0481],
                [0.7439, 0.6318333333333334, 0.24695],
                [0.7561, 0.5485, 0.00305],
                [0.9227666666666666, 0.5348333333333334, 0.25305],
                [0.9105666666666666, 0.6181666666666666, 0.49695],
                [0.7714, 0.5881166666666666, 0.6402],
                [0.7286, 0.5047833333333334, 0.6098],
                [0.8952666666666667, 0.57855, 0.8598],
                [0.9380666666666667, 0.6618833333333334, 0.8902],
                [0.7199, 0.57355, 0.6652],
                [0.7801, 0.6568833333333334, 0.5848],
                [0.9467666666666666, 0.5931166666666666, 0.8348],
                [0.8865666666666666, 0.5097833333333334, 0.9152],
                [0.7703333333333333, 0.6326333333333333, 0.6910000000000001],
                [0.7296666666666667, 0.5493, 0.5589999999999999],
                [0.8963333333333333, 0.5340333333333334, 0.8089999999999999],
                [0.9369999999999999, 0.6173666666666667, 0.9410000000000001],
                [0.8203, 0.6462833333333333, 0.6643],
                [0.6797, 0.56295, 0.5857],
                [0.8463666666666666, 0.5203833333333333, 0.8357],
                [0.9869666666666667, 0.6037166666666667, 0.9143],
                [0.6934333333333332, 0.5750333333333334, 0.7315],
                [0.8065666666666668, 0.6583666666666667, 0.5185],
                [0.9732333333333333, 0.5916333333333333, 0.7685],
                [0.8601000000000001, 0.5083, 0.9815],
                [0.7099333333333333, 0.58935, 0.7826],
                [0.7900666666666667, 0.5060166666666667, 0.9674],
                [0.9567333333333333, 0.5773166666666666, 0.7174],
                [0.8765999999999999, 0.66065, 0.5326],
                [0.7591666666666667, 0.6459833333333334, 0.7981],
                [0.7408333333333333, 0.56265, 0.9519],
                [0.9075000000000001, 0.5206833333333333, 0.7019],
                [0.9258333333333333, 0.6040166666666666, 0.5481],
                [0.7439, 0.6318333333333334, 0.74695],
                [0.7561, 0.5485, 0.50305],
                [0.9227666666666666, 0.5348333333333334, 0.75305],
                [0.9105666666666666, 0.6181666666666666, 0.99695],
                [0.7714, 0.7547833333333333, 0.1402],
                [0.7286, 0.67145, 0.1098],
                [0.8952666666666667, 0.7452166666666667, 0.3598],
                [0.9380666666666667, 0.82855, 0.3902],
                [0.7199, 0.7402166666666666, 0.1652],
                [0.7801, 0.82355, 0.0848],
                [0.9467666666666666, 0.7597833333333334, 0.3348],
                [0.8865666666666666, 0.67645, 0.4152],
                [0.7703333333333333, 0.7993, 0.191],
                [0.7296666666666667, 0.7159666666666666, 0.059],
                [0.8963333333333333, 0.7007, 0.309],
                [0.9369999999999999, 0.7840333333333334, 0.441],
                [0.8203, 0.81295, 0.1643],
                [0.6797, 0.7296166666666667, 0.0857],
                [0.8463666666666666, 0.68705, 0.3357],
                [0.9869666666666667, 0.7703833333333333, 0.4143],
                [0.6934333333333332, 0.7416999999999999, 0.2315],
                [0.8065666666666668, 0.8250333333333333, 0.0185],
                [0.9732333333333333, 0.7583000000000001, 0.2685],
                [0.8601000000000001, 0.6749666666666667, 0.4815],
                [0.7099333333333333, 0.7560166666666667, 0.2826],
                [0.7900666666666667, 0.6726833333333334, 0.4674],
                [0.9567333333333333, 0.7439833333333333, 0.2174],
                [0.8765999999999999, 0.8273166666666666, 0.0326],
                [0.7591666666666667, 0.81265, 0.2981],
                [0.7408333333333333, 0.7293166666666666, 0.4519],
                [0.9075000000000001, 0.68735, 0.2019],
                [0.9258333333333333, 0.7706833333333334, 0.0481],
                [0.7439, 0.7985000000000001, 0.24695],
                [0.7561, 0.7151666666666667, 0.00305],
                [0.9227666666666666, 0.7014999999999999, 0.25305],
                [0.9105666666666666, 0.7848333333333333, 0.49695],
                [0.7714, 0.7547833333333333, 0.6402],
                [0.7286, 0.67145, 0.6098],
                [0.8952666666666667, 0.7452166666666667, 0.8598],
                [0.9380666666666667, 0.82855, 0.8902],
                [0.7199, 0.7402166666666666, 0.6652],
                [0.7801, 0.82355, 0.5848],
                [0.9467666666666666, 0.7597833333333334, 0.8348],
                [0.8865666666666666, 0.67645, 0.9152],
                [0.7703333333333333, 0.7993, 0.6910000000000001],
                [0.7296666666666667, 0.7159666666666666, 0.5589999999999999],
                [0.8963333333333333, 0.7007, 0.8089999999999999],
                [0.9369999999999999, 0.7840333333333334, 0.9410000000000001],
                [0.8203, 0.81295, 0.6643],
                [0.6797, 0.7296166666666667, 0.5857],
                [0.8463666666666666, 0.68705, 0.8357],
                [0.9869666666666667, 0.7703833333333333, 0.9143],
                [0.6934333333333332, 0.7416999999999999, 0.7315],
                [0.8065666666666668, 0.8250333333333333, 0.5185],
                [0.9732333333333333, 0.7583000000000001, 0.7685],
                [0.8601000000000001, 0.6749666666666667, 0.9815],
                [0.7099333333333333, 0.7560166666666667, 0.7826],
                [0.7900666666666667, 0.6726833333333334, 0.9674],
                [0.9567333333333333, 0.7439833333333333, 0.7174],
                [0.8765999999999999, 0.8273166666666666, 0.5326],
                [0.7591666666666667, 0.81265, 0.7981],
                [0.7408333333333333, 0.7293166666666666, 0.9519],
                [0.9075000000000001, 0.68735, 0.7019],
                [0.9258333333333333, 0.7706833333333334, 0.5481],
                [0.7439, 0.7985000000000001, 0.74695],
                [0.7561, 0.7151666666666667, 0.50305],
                [0.9227666666666666, 0.7014999999999999, 0.75305],
                [0.9105666666666666, 0.7848333333333333, 0.99695],
                [0.7714, 0.92145, 0.1402],
                [0.7286, 0.8381166666666666, 0.1098],
                [0.8952666666666667, 0.9118833333333334, 0.3598],
                [0.9380666666666667, 0.9952166666666667, 0.3902],
                [0.7199, 0.9068833333333334, 0.1652],
                [0.7801, 0.9902166666666666, 0.0848],
                [0.9467666666666666, 0.92645, 0.3348],
                [0.8865666666666666, 0.8431166666666666, 0.4152],
                [0.7703333333333333, 0.9659666666666666, 0.191],
                [0.7296666666666667, 0.8826333333333333, 0.059],
                [0.8963333333333333, 0.8673666666666667, 0.309],
                [0.9369999999999999, 0.9507, 0.441],
                [0.8203, 0.9796166666666667, 0.1643],
                [0.6797, 0.8962833333333333, 0.0857],
                [0.8463666666666666, 0.8537166666666667, 0.3357],
                [0.9869666666666667, 0.93705, 0.4143],
                [0.6934333333333332, 0.9083666666666667, 0.2315],
                [0.8065666666666668, 0.9916999999999999, 0.0185],
                [0.9732333333333333, 0.9249666666666667, 0.2685],
                [0.8601000000000001, 0.8416333333333333, 0.4815],
                [0.7099333333333333, 0.9226833333333334, 0.2826],
                [0.7900666666666667, 0.83935, 0.4674],
                [0.9567333333333333, 0.91065, 0.2174],
                [0.8765999999999999, 0.9939833333333333, 0.0326],
                [0.7591666666666667, 0.9793166666666666, 0.2981],
                [0.7408333333333333, 0.8959833333333332, 0.4519],
                [0.9075000000000001, 0.8540166666666668, 0.2019],
                [0.9258333333333333, 0.93735, 0.0481],
                [0.7439, 0.9651666666666667, 0.24695],
                [0.7561, 0.8818333333333334, 0.00305],
                [0.9227666666666666, 0.8681666666666666, 0.25305],
                [0.9105666666666666, 0.9514999999999999, 0.49695],
                [0.7714, 0.92145, 0.6402],
                [0.7286, 0.8381166666666666, 0.6098],
                [0.8952666666666667, 0.9118833333333334, 0.8598],
                [0.9380666666666667, 0.9952166666666667, 0.8902],
                [0.7199, 0.9068833333333334, 0.6652],
                [0.7801, 0.9902166666666666, 0.5848],
                [0.9467666666666666, 0.92645, 0.8348],
                [0.8865666666666666, 0.8431166666666666, 0.9152],
                [0.7703333333333333, 0.9659666666666666, 0.6910000000000001],
                [0.7296666666666667, 0.8826333333333333, 0.5589999999999999],
                [0.8963333333333333, 0.8673666666666667, 0.8089999999999999],
                [0.9369999999999999, 0.9507, 0.9410000000000001],
                [0.8203, 0.9796166666666667, 0.6643],
                [0.6797, 0.8962833333333333, 0.5857],
                [0.8463666666666666, 0.8537166666666667, 0.8357],
                [0.9869666666666667, 0.93705, 0.9143],
                [0.6934333333333332, 0.9083666666666667, 0.7315],
                [0.8065666666666668, 0.9916999999999999, 0.5185],
                [0.9732333333333333, 0.9249666666666667, 0.7685],
                [0.8601000000000001, 0.8416333333333333, 0.9815],
                [0.7099333333333333, 0.9226833333333334, 0.7826],
                [0.7900666666666667, 0.83935, 0.9674],
                [0.9567333333333333, 0.91065, 0.7174],
                [0.8765999999999999, 0.9939833333333333, 0.5326],
                [0.7591666666666667, 0.9793166666666666, 0.7981],
                [0.7408333333333333, 0.8959833333333332, 0.9519],
                [0.9075000000000001, 0.8540166666666668, 0.7019],
                [0.9258333333333333, 0.93735, 0.5481],
                [0.7439, 0.9651666666666667, 0.74695],
                [0.7561, 0.8818333333333334, 0.50305],
                [0.9227666666666666, 0.8681666666666666, 0.75305],
                [0.9105666666666666, 0.9514999999999999, 0.99695],
                [0.17079999999999998, 0.07925, 0.126105],
                [0.3292, 0.16258333333333333, 0.123895],
                [0.16253333333333334, 0.08741666666666666, 0.373895],
                [0.0041333333333333335, 0.004083333333333334, 0.376105],
                [0.18030000000000002, 0.013466666666666667, 0.12186],
                [0.3197, 0.0968, 0.12814],
                [0.15303333333333333, 0.1532, 0.37814],
                [0.013633333333333332, 0.06986666666666667, 0.37186],
                [0.0018000000000000002, 0.032633333333333334, 0.18562],
                [0.16486666666666666, 0.11596666666666666, 0.06438],
                [0.33153333333333335, 0.13403333333333334, 0.31438],
                [0.16846666666666665, 0.0507, 0.43562],
                [0.028866666666666665, 0.12993333333333332, 0.19527],
                [0.1378, 0.0466, 0.05473],
                [0.30446666666666666, 0.03673333333333333, 0.30473],
                [0.19553333333333334, 0.12006666666666667, 0.44527],
                [0.07903333333333333, 0.07946666666666667, 0.336325],
                [0.08763333333333334, 0.1628, 0.413675],
                [0.2543, 0.0872, 0.163675],
                [0.2457, 0.0038666666666666663, 0.086325],
                [0.08020000000000001, 0.014166666666666668, 0.34533],
                [0.08646666666666668, 0.09749999999999999, 0.40467],
                [0.2531333333333333, 0.1525, 0.15467],
                [0.24686666666666668, 0.06916666666666667, 0.09533],
                [0.17079999999999998, 0.07925, 0.626105],
                [0.3292, 0.16258333333333333, 0.623895],
                [0.16253333333333334, 0.08741666666666666, 0.873895],
                [0.0041333333333333335, 0.004083333333333334, 0.876105],
                [0.18030000000000002, 0.013466666666666667, 0.62186],
                [0.3197, 0.0968, 0.62814],
                [0.15303333333333333, 0.1532, 0.8781399999999999],
                [0.013633333333333332, 0.06986666666666667, 0.8718600000000001],
                [0.0018000000000000002, 0.032633333333333334, 0.68562],
                [0.16486666666666666, 0.11596666666666666, 0.56438],
                [0.33153333333333335, 0.13403333333333334, 0.81438],
                [0.16846666666666665, 0.0507, 0.93562],
                [0.028866666666666665, 0.12993333333333332, 0.69527],
                [0.1378, 0.0466, 0.55473],
                [0.30446666666666666, 0.03673333333333333, 0.80473],
                [0.19553333333333334, 0.12006666666666667, 0.94527],
                [0.07903333333333333, 0.07946666666666667, 0.836325],
                [0.08763333333333334, 0.1628, 0.913675],
                [0.2543, 0.0872, 0.663675],
                [0.2457, 0.0038666666666666663, 0.586325],
                [0.08020000000000001, 0.014166666666666668, 0.84533],
                [0.08646666666666668, 0.09749999999999999, 0.90467],
                [0.2531333333333333, 0.1525, 0.65467],
                [0.24686666666666668, 0.06916666666666667, 0.59533],
                [0.17079999999999998, 0.24591666666666667, 0.126105],
                [0.3292, 0.32925, 0.123895],
                [0.16253333333333334, 0.2540833333333333, 0.373895],
                [0.0041333333333333335, 0.17074999999999999, 0.376105],
                [0.18030000000000002, 0.18013333333333334, 0.12186],
                [0.3197, 0.2634666666666667, 0.12814],
                [0.15303333333333333, 0.3198666666666667, 0.37814],
                [0.013633333333333332, 0.23653333333333335, 0.37186],
                [0.0018000000000000002, 0.1993, 0.18562],
                [0.16486666666666666, 0.28263333333333335, 0.06438],
                [0.33153333333333335, 0.3007, 0.31438],
                [0.16846666666666665, 0.21736666666666668, 0.43562],
                [0.028866666666666665, 0.2966, 0.19527],
                [0.1378, 0.2132666666666667, 0.05473],
                [0.30446666666666666, 0.2034, 0.30473],
                [0.19553333333333334, 0.28673333333333334, 0.44527],
                [0.07903333333333333, 0.24613333333333332, 0.336325],
                [0.08763333333333334, 0.32946666666666663, 0.413675],
                [0.2543, 0.2538666666666667, 0.163675],
                [0.2457, 0.17053333333333334, 0.086325],
                [0.08020000000000001, 0.18083333333333332, 0.34533],
                [0.08646666666666668, 0.26416666666666666, 0.40467],
                [0.2531333333333333, 0.31916666666666665, 0.15467],
                [0.24686666666666668, 0.23583333333333334, 0.09533],
                [0.17079999999999998, 0.24591666666666667, 0.626105],
                [0.3292, 0.32925, 0.623895],
                [0.16253333333333334, 0.2540833333333333, 0.873895],
                [0.0041333333333333335, 0.17074999999999999, 0.876105],
                [0.18030000000000002, 0.18013333333333334, 0.62186],
                [0.3197, 0.2634666666666667, 0.62814],
                [0.15303333333333333, 0.3198666666666667, 0.8781399999999999],
                [0.013633333333333332, 0.23653333333333335, 0.8718600000000001],
                [0.0018000000000000002, 0.1993, 0.68562],
                [0.16486666666666666, 0.28263333333333335, 0.56438],
                [0.33153333333333335, 0.3007, 0.81438],
                [0.16846666666666665, 0.21736666666666668, 0.93562],
                [0.028866666666666665, 0.2966, 0.69527],
                [0.1378, 0.2132666666666667, 0.55473],
                [0.30446666666666666, 0.2034, 0.80473],
                [0.19553333333333334, 0.28673333333333334, 0.94527],
                [0.07903333333333333, 0.24613333333333332, 0.836325],
                [0.08763333333333334, 0.32946666666666663, 0.913675],
                [0.2543, 0.2538666666666667, 0.663675],
                [0.2457, 0.17053333333333334, 0.586325],
                [0.08020000000000001, 0.18083333333333332, 0.84533],
                [0.08646666666666668, 0.26416666666666666, 0.90467],
                [0.2531333333333333, 0.31916666666666665, 0.65467],
                [0.24686666666666668, 0.23583333333333334, 0.59533],
                [0.17079999999999998, 0.4125833333333333, 0.126105],
                [0.3292, 0.49591666666666673, 0.123895],
                [0.16253333333333334, 0.42074999999999996, 0.373895],
                [0.0041333333333333335, 0.3374166666666667, 0.376105],
                [0.18030000000000002, 0.3468, 0.12186],
                [0.3197, 0.4301333333333333, 0.12814],
                [0.15303333333333333, 0.4865333333333333, 0.37814],
                [0.013633333333333332, 0.4032, 0.37186],
                [0.0018000000000000002, 0.3659666666666667, 0.18562],
                [0.16486666666666666, 0.44930000000000003, 0.06438],
                [0.33153333333333335, 0.46736666666666665, 0.31438],
                [0.16846666666666665, 0.3840333333333333, 0.43562],
                [0.028866666666666665, 0.46326666666666666, 0.19527],
                [0.1378, 0.3799333333333333, 0.05473],
                [0.30446666666666666, 0.3700666666666667, 0.30473],
                [0.19553333333333334, 0.4534, 0.44527],
                [0.07903333333333333, 0.4128, 0.336325],
                [0.08763333333333334, 0.4961333333333333, 0.413675],
                [0.2543, 0.42053333333333337, 0.163675],
                [0.2457, 0.3372, 0.086325],
                [0.08020000000000001, 0.3475, 0.34533],
                [0.08646666666666668, 0.43083333333333335, 0.40467],
                [0.2531333333333333, 0.48583333333333334, 0.15467],
                [0.24686666666666668, 0.4025, 0.09533],
                [0.17079999999999998, 0.4125833333333333, 0.626105],
                [0.3292, 0.49591666666666673, 0.623895],
                [0.16253333333333334, 0.42074999999999996, 0.873895],
                [0.0041333333333333335, 0.3374166666666667, 0.876105],
                [0.18030000000000002, 0.3468, 0.62186],
                [0.3197, 0.4301333333333333, 0.62814],
                [0.15303333333333333, 0.4865333333333333, 0.8781399999999999],
                [0.013633333333333332, 0.4032, 0.8718600000000001],
                [0.0018000000000000002, 0.3659666666666667, 0.68562],
                [0.16486666666666666, 0.44930000000000003, 0.56438],
                [0.33153333333333335, 0.46736666666666665, 0.81438],
                [0.16846666666666665, 0.3840333333333333, 0.93562],
                [0.028866666666666665, 0.46326666666666666, 0.69527],
                [0.1378, 0.3799333333333333, 0.55473],
                [0.30446666666666666, 0.3700666666666667, 0.80473],
                [0.19553333333333334, 0.4534, 0.94527],
                [0.07903333333333333, 0.4128, 0.836325],
                [0.08763333333333334, 0.4961333333333333, 0.913675],
                [0.2543, 0.42053333333333337, 0.663675],
                [0.2457, 0.3372, 0.586325],
                [0.08020000000000001, 0.3475, 0.84533],
                [0.08646666666666668, 0.43083333333333335, 0.90467],
                [0.2531333333333333, 0.48583333333333334, 0.65467],
                [0.24686666666666668, 0.4025, 0.59533],
                [0.17079999999999998, 0.5792499999999999, 0.126105],
                [0.3292, 0.6625833333333334, 0.123895],
                [0.16253333333333334, 0.5874166666666666, 0.373895],
                [0.0041333333333333335, 0.5040833333333333, 0.376105],
                [0.18030000000000002, 0.5134666666666666, 0.12186],
                [0.3197, 0.5968, 0.12814],
                [0.15303333333333333, 0.6532, 0.37814],
                [0.013633333333333332, 0.5698666666666666, 0.37186],
                [0.0018000000000000002, 0.5326333333333334, 0.18562],
                [0.16486666666666666, 0.6159666666666667, 0.06438],
                [0.33153333333333335, 0.6340333333333333, 0.31438],
                [0.16846666666666665, 0.5507, 0.43562],
                [0.028866666666666665, 0.6299333333333333, 0.19527],
                [0.1378, 0.5466, 0.05473],
                [0.30446666666666666, 0.5367333333333334, 0.30473],
                [0.19553333333333334, 0.6200666666666667, 0.44527],
                [0.07903333333333333, 0.5794666666666667, 0.336325],
                [0.08763333333333334, 0.6628, 0.413675],
                [0.2543, 0.5872, 0.163675],
                [0.2457, 0.5038666666666667, 0.086325],
                [0.08020000000000001, 0.5141666666666667, 0.34533],
                [0.08646666666666668, 0.5975, 0.40467],
                [0.2531333333333333, 0.6525, 0.15467],
                [0.24686666666666668, 0.5691666666666667, 0.09533],
                [0.17079999999999998, 0.5792499999999999, 0.626105],
                [0.3292, 0.6625833333333334, 0.623895],
                [0.16253333333333334, 0.5874166666666666, 0.873895],
                [0.0041333333333333335, 0.5040833333333333, 0.876105],
                [0.18030000000000002, 0.5134666666666666, 0.62186],
                [0.3197, 0.5968, 0.62814],
                [0.15303333333333333, 0.6532, 0.8781399999999999],
                [0.013633333333333332, 0.5698666666666666, 0.8718600000000001],
                [0.0018000000000000002, 0.5326333333333334, 0.68562],
                [0.16486666666666666, 0.6159666666666667, 0.56438],
                [0.33153333333333335, 0.6340333333333333, 0.81438],
                [0.16846666666666665, 0.5507, 0.93562],
                [0.028866666666666665, 0.6299333333333333, 0.69527],
                [0.1378, 0.5466, 0.55473],
                [0.30446666666666666, 0.5367333333333334, 0.80473],
                [0.19553333333333334, 0.6200666666666667, 0.94527],
                [0.07903333333333333, 0.5794666666666667, 0.836325],
                [0.08763333333333334, 0.6628, 0.913675],
                [0.2543, 0.5872, 0.663675],
                [0.2457, 0.5038666666666667, 0.586325],
                [0.08020000000000001, 0.5141666666666667, 0.84533],
                [0.08646666666666668, 0.5975, 0.90467],
                [0.2531333333333333, 0.6525, 0.65467],
                [0.24686666666666668, 0.5691666666666667, 0.59533],
                [0.17079999999999998, 0.7459166666666667, 0.126105],
                [0.3292, 0.82925, 0.123895],
                [0.16253333333333334, 0.7540833333333333, 0.373895],
                [0.0041333333333333335, 0.67075, 0.376105],
                [0.18030000000000002, 0.6801333333333334, 0.12186],
                [0.3197, 0.7634666666666666, 0.12814],
                [0.15303333333333333, 0.8198666666666666, 0.37814],
                [0.013633333333333332, 0.7365333333333334, 0.37186],
                [0.0018000000000000002, 0.6993, 0.18562],
                [0.16486666666666666, 0.7826333333333334, 0.06438],
                [0.33153333333333335, 0.8007, 0.31438],
                [0.16846666666666665, 0.7173666666666666, 0.43562],
                [0.028866666666666665, 0.7966000000000001, 0.19527],
                [0.1378, 0.7132666666666667, 0.05473],
                [0.30446666666666666, 0.7033999999999999, 0.30473],
                [0.19553333333333334, 0.7867333333333333, 0.44527],
                [0.07903333333333333, 0.7461333333333333, 0.336325],
                [0.08763333333333334, 0.8294666666666667, 0.413675],
                [0.2543, 0.7538666666666667, 0.163675],
                [0.2457, 0.6705333333333333, 0.086325],
                [0.08020000000000001, 0.6808333333333333, 0.34533],
                [0.08646666666666668, 0.7641666666666667, 0.40467],
                [0.2531333333333333, 0.8191666666666667, 0.15467],
                [0.24686666666666668, 0.7358333333333333, 0.09533],
                [0.17079999999999998, 0.7459166666666667, 0.626105],
                [0.3292, 0.82925, 0.623895],
                [0.16253333333333334, 0.7540833333333333, 0.873895],
                [0.0041333333333333335, 0.67075, 0.876105],
                [0.18030000000000002, 0.6801333333333334, 0.62186],
                [0.3197, 0.7634666666666666, 0.62814],
                [0.15303333333333333, 0.8198666666666666, 0.8781399999999999],
                [0.013633333333333332, 0.7365333333333334, 0.8718600000000001],
                [0.0018000000000000002, 0.6993, 0.68562],
                [0.16486666666666666, 0.7826333333333334, 0.56438],
                [0.33153333333333335, 0.8007, 0.81438],
                [0.16846666666666665, 0.7173666666666666, 0.93562],
                [0.028866666666666665, 0.7966000000000001, 0.69527],
                [0.1378, 0.7132666666666667, 0.55473],
                [0.30446666666666666, 0.7033999999999999, 0.80473],
                [0.19553333333333334, 0.7867333333333333, 0.94527],
                [0.07903333333333333, 0.7461333333333333, 0.836325],
                [0.08763333333333334, 0.8294666666666667, 0.913675],
                [0.2543, 0.7538666666666667, 0.663675],
                [0.2457, 0.6705333333333333, 0.586325],
                [0.08020000000000001, 0.6808333333333333, 0.84533],
                [0.08646666666666668, 0.7641666666666667, 0.90467],
                [0.2531333333333333, 0.8191666666666667, 0.65467],
                [0.24686666666666668, 0.7358333333333333, 0.59533],
                [0.17079999999999998, 0.9125833333333334, 0.126105],
                [0.3292, 0.9959166666666667, 0.123895],
                [0.16253333333333334, 0.92075, 0.373895],
                [0.0041333333333333335, 0.8374166666666666, 0.376105],
                [0.18030000000000002, 0.8468, 0.12186],
                [0.3197, 0.9301333333333334, 0.12814],
                [0.15303333333333333, 0.9865333333333334, 0.37814],
                [0.013633333333333332, 0.9032, 0.37186],
                [0.0018000000000000002, 0.8659666666666667, 0.18562],
                [0.16486666666666666, 0.9493, 0.06438],
                [0.33153333333333335, 0.9673666666666666, 0.31438],
                [0.16846666666666665, 0.8840333333333333, 0.43562],
                [0.028866666666666665, 0.9632666666666667, 0.19527],
                [0.1378, 0.8799333333333333, 0.05473],
                [0.30446666666666666, 0.8700666666666667, 0.30473],
                [0.19553333333333334, 0.9533999999999999, 0.44527],
                [0.07903333333333333, 0.9128, 0.336325],
                [0.08763333333333334, 0.9961333333333333, 0.413675],
                [0.2543, 0.9205333333333333, 0.163675],
                [0.2457, 0.8372, 0.086325],
                [0.08020000000000001, 0.8475, 0.34533],
                [0.08646666666666668, 0.9308333333333333, 0.40467],
                [0.2531333333333333, 0.9858333333333333, 0.15467],
                [0.24686666666666668, 0.9025, 0.09533],
                [0.17079999999999998, 0.9125833333333334, 0.626105],
                [0.3292, 0.9959166666666667, 0.623895],
                [0.16253333333333334, 0.92075, 0.873895],
                [0.0041333333333333335, 0.8374166666666666, 0.876105],
                [0.18030000000000002, 0.8468, 0.62186],
                [0.3197, 0.9301333333333334, 0.62814],
                [0.15303333333333333, 0.9865333333333334, 0.8781399999999999],
                [0.013633333333333332, 0.9032, 0.8718600000000001],
                [0.0018000000000000002, 0.8659666666666667, 0.68562],
                [0.16486666666666666, 0.9493, 0.56438],
                [0.33153333333333335, 0.9673666666666666, 0.81438],
                [0.16846666666666665, 0.8840333333333333, 0.93562],
                [0.028866666666666665, 0.9632666666666667, 0.69527],
                [0.1378, 0.8799333333333333, 0.55473],
                [0.30446666666666666, 0.8700666666666667, 0.80473],
                [0.19553333333333334, 0.9533999999999999, 0.94527],
                [0.07903333333333333, 0.9128, 0.836325],
                [0.08763333333333334, 0.9961333333333333, 0.913675],
                [0.2543, 0.9205333333333333, 0.663675],
                [0.2457, 0.8372, 0.586325],
                [0.08020000000000001, 0.8475, 0.84533],
                [0.08646666666666668, 0.9308333333333333, 0.90467],
                [0.2531333333333333, 0.9858333333333333, 0.65467],
                [0.24686666666666668, 0.9025, 0.59533],
                [0.5041333333333333, 0.07925, 0.126105],
                [0.6625333333333333, 0.16258333333333333, 0.123895],
                [0.4958666666666667, 0.08741666666666666, 0.373895],
                [0.33746666666666664, 0.004083333333333334, 0.376105],
                [0.5136333333333334, 0.013466666666666667, 0.12186],
                [0.6530333333333332, 0.0968, 0.12814],
                [0.48636666666666667, 0.1532, 0.37814],
                [0.34696666666666665, 0.06986666666666667, 0.37186],
                [0.33513333333333334, 0.032633333333333334, 0.18562],
                [0.4982, 0.11596666666666666, 0.06438],
                [0.6648666666666667, 0.13403333333333334, 0.31438],
                [0.5017999999999999, 0.0507, 0.43562],
                [0.3622, 0.12993333333333332, 0.19527],
                [0.47113333333333335, 0.0466, 0.05473],
                [0.6378, 0.03673333333333333, 0.30473],
                [0.5288666666666667, 0.12006666666666667, 0.44527],
                [0.4123666666666667, 0.07946666666666667, 0.336325],
                [0.4209666666666667, 0.1628, 0.413675],
                [0.5876333333333333, 0.0872, 0.163675],
                [0.5790333333333333, 0.0038666666666666663, 0.086325],
                [0.4135333333333333, 0.014166666666666668, 0.34533],
                [0.4198, 0.09749999999999999, 0.40467],
                [0.5864666666666666, 0.1525, 0.15467],
                [0.5802, 0.06916666666666667, 0.09533],
                [0.5041333333333333, 0.07925, 0.626105],
                [0.6625333333333333, 0.16258333333333333, 0.623895],
                [0.4958666666666667, 0.08741666666666666, 0.873895],
                [0.33746666666666664, 0.004083333333333334, 0.876105],
                [0.5136333333333334, 0.013466666666666667, 0.62186],
                [0.6530333333333332, 0.0968, 0.62814],
                [0.48636666666666667, 0.1532, 0.8781399999999999],
                [0.34696666666666665, 0.06986666666666667, 0.8718600000000001],
                [0.33513333333333334, 0.032633333333333334, 0.68562],
                [0.4982, 0.11596666666666666, 0.56438],
                [0.6648666666666667, 0.13403333333333334, 0.81438],
                [0.5017999999999999, 0.0507, 0.93562],
                [0.3622, 0.12993333333333332, 0.69527],
                [0.47113333333333335, 0.0466, 0.55473],
                [0.6378, 0.03673333333333333, 0.80473],
                [0.5288666666666667, 0.12006666666666667, 0.94527],
                [0.4123666666666667, 0.07946666666666667, 0.836325],
                [0.4209666666666667, 0.1628, 0.913675],
                [0.5876333333333333, 0.0872, 0.663675],
                [0.5790333333333333, 0.0038666666666666663, 0.586325],
                [0.4135333333333333, 0.014166666666666668, 0.84533],
                [0.4198, 0.09749999999999999, 0.90467],
                [0.5864666666666666, 0.1525, 0.65467],
                [0.5802, 0.06916666666666667, 0.59533],
                [0.5041333333333333, 0.24591666666666667, 0.126105],
                [0.6625333333333333, 0.32925, 0.123895],
                [0.4958666666666667, 0.2540833333333333, 0.373895],
                [0.33746666666666664, 0.17074999999999999, 0.376105],
                [0.5136333333333334, 0.18013333333333334, 0.12186],
                [0.6530333333333332, 0.2634666666666667, 0.12814],
                [0.48636666666666667, 0.3198666666666667, 0.37814],
                [0.34696666666666665, 0.23653333333333335, 0.37186],
                [0.33513333333333334, 0.1993, 0.18562],
                [0.4982, 0.28263333333333335, 0.06438],
                [0.6648666666666667, 0.3007, 0.31438],
                [0.5017999999999999, 0.21736666666666668, 0.43562],
                [0.3622, 0.2966, 0.19527],
                [0.47113333333333335, 0.2132666666666667, 0.05473],
                [0.6378, 0.2034, 0.30473],
                [0.5288666666666667, 0.28673333333333334, 0.44527],
                [0.4123666666666667, 0.24613333333333332, 0.336325],
                [0.4209666666666667, 0.32946666666666663, 0.413675],
                [0.5876333333333333, 0.2538666666666667, 0.163675],
                [0.5790333333333333, 0.17053333333333334, 0.086325],
                [0.4135333333333333, 0.18083333333333332, 0.34533],
                [0.4198, 0.26416666666666666, 0.40467],
                [0.5864666666666666, 0.31916666666666665, 0.15467],
                [0.5802, 0.23583333333333334, 0.09533],
                [0.5041333333333333, 0.24591666666666667, 0.626105],
                [0.6625333333333333, 0.32925, 0.623895],
                [0.4958666666666667, 0.2540833333333333, 0.873895],
                [0.33746666666666664, 0.17074999999999999, 0.876105],
                [0.5136333333333334, 0.18013333333333334, 0.62186],
                [0.6530333333333332, 0.2634666666666667, 0.62814],
                [0.48636666666666667, 0.3198666666666667, 0.8781399999999999],
                [0.34696666666666665, 0.23653333333333335, 0.8718600000000001],
                [0.33513333333333334, 0.1993, 0.68562],
                [0.4982, 0.28263333333333335, 0.56438],
                [0.6648666666666667, 0.3007, 0.81438],
                [0.5017999999999999, 0.21736666666666668, 0.93562],
                [0.3622, 0.2966, 0.69527],
                [0.47113333333333335, 0.2132666666666667, 0.55473],
                [0.6378, 0.2034, 0.80473],
                [0.5288666666666667, 0.28673333333333334, 0.94527],
                [0.4123666666666667, 0.24613333333333332, 0.836325],
                [0.4209666666666667, 0.32946666666666663, 0.913675],
                [0.5876333333333333, 0.2538666666666667, 0.663675],
                [0.5790333333333333, 0.17053333333333334, 0.586325],
                [0.4135333333333333, 0.18083333333333332, 0.84533],
                [0.4198, 0.26416666666666666, 0.90467],
                [0.5864666666666666, 0.31916666666666665, 0.65467],
                [0.5802, 0.23583333333333334, 0.59533],
                [0.5041333333333333, 0.4125833333333333, 0.126105],
                [0.6625333333333333, 0.49591666666666673, 0.123895],
                [0.4958666666666667, 0.42074999999999996, 0.373895],
                [0.33746666666666664, 0.3374166666666667, 0.376105],
                [0.5136333333333334, 0.3468, 0.12186],
                [0.6530333333333332, 0.4301333333333333, 0.12814],
                [0.48636666666666667, 0.4865333333333333, 0.37814],
                [0.34696666666666665, 0.4032, 0.37186],
                [0.33513333333333334, 0.3659666666666667, 0.18562],
                [0.4982, 0.44930000000000003, 0.06438],
                [0.6648666666666667, 0.46736666666666665, 0.31438],
                [0.5017999999999999, 0.3840333333333333, 0.43562],
                [0.3622, 0.46326666666666666, 0.19527],
                [0.47113333333333335, 0.3799333333333333, 0.05473],
                [0.6378, 0.3700666666666667, 0.30473],
                [0.5288666666666667, 0.4534, 0.44527],
                [0.4123666666666667, 0.4128, 0.336325],
                [0.4209666666666667, 0.4961333333333333, 0.413675],
                [0.5876333333333333, 0.42053333333333337, 0.163675],
                [0.5790333333333333, 0.3372, 0.086325],
                [0.4135333333333333, 0.3475, 0.34533],
                [0.4198, 0.43083333333333335, 0.40467],
                [0.5864666666666666, 0.48583333333333334, 0.15467],
                [0.5802, 0.4025, 0.09533],
                [0.5041333333333333, 0.4125833333333333, 0.626105],
                [0.6625333333333333, 0.49591666666666673, 0.623895],
                [0.4958666666666667, 0.42074999999999996, 0.873895],
                [0.33746666666666664, 0.3374166666666667, 0.876105],
                [0.5136333333333334, 0.3468, 0.62186],
                [0.6530333333333332, 0.4301333333333333, 0.62814],
                [0.48636666666666667, 0.4865333333333333, 0.8781399999999999],
                [0.34696666666666665, 0.4032, 0.8718600000000001],
                [0.33513333333333334, 0.3659666666666667, 0.68562],
                [0.4982, 0.44930000000000003, 0.56438],
                [0.6648666666666667, 0.46736666666666665, 0.81438],
                [0.5017999999999999, 0.3840333333333333, 0.93562],
                [0.3622, 0.46326666666666666, 0.69527],
                [0.47113333333333335, 0.3799333333333333, 0.55473],
                [0.6378, 0.3700666666666667, 0.80473],
                [0.5288666666666667, 0.4534, 0.94527],
                [0.4123666666666667, 0.4128, 0.836325],
                [0.4209666666666667, 0.4961333333333333, 0.913675],
                [0.5876333333333333, 0.42053333333333337, 0.663675],
                [0.5790333333333333, 0.3372, 0.586325],
                [0.4135333333333333, 0.3475, 0.84533],
                [0.4198, 0.43083333333333335, 0.90467],
                [0.5864666666666666, 0.48583333333333334, 0.65467],
                [0.5802, 0.4025, 0.59533],
                [0.5041333333333333, 0.5792499999999999, 0.126105],
                [0.6625333333333333, 0.6625833333333334, 0.123895],
                [0.4958666666666667, 0.5874166666666666, 0.373895],
                [0.33746666666666664, 0.5040833333333333, 0.376105],
                [0.5136333333333334, 0.5134666666666666, 0.12186],
                [0.6530333333333332, 0.5968, 0.12814],
                [0.48636666666666667, 0.6532, 0.37814],
                [0.34696666666666665, 0.5698666666666666, 0.37186],
                [0.33513333333333334, 0.5326333333333334, 0.18562],
                [0.4982, 0.6159666666666667, 0.06438],
                [0.6648666666666667, 0.6340333333333333, 0.31438],
                [0.5017999999999999, 0.5507, 0.43562],
                [0.3622, 0.6299333333333333, 0.19527],
                [0.47113333333333335, 0.5466, 0.05473],
                [0.6378, 0.5367333333333334, 0.30473],
                [0.5288666666666667, 0.6200666666666667, 0.44527],
                [0.4123666666666667, 0.5794666666666667, 0.336325],
                [0.4209666666666667, 0.6628, 0.413675],
                [0.5876333333333333, 0.5872, 0.163675],
                [0.5790333333333333, 0.5038666666666667, 0.086325],
                [0.4135333333333333, 0.5141666666666667, 0.34533],
                [0.4198, 0.5975, 0.40467],
                [0.5864666666666666, 0.6525, 0.15467],
                [0.5802, 0.5691666666666667, 0.09533],
                [0.5041333333333333, 0.5792499999999999, 0.626105],
                [0.6625333333333333, 0.6625833333333334, 0.623895],
                [0.4958666666666667, 0.5874166666666666, 0.873895],
                [0.33746666666666664, 0.5040833333333333, 0.876105],
                [0.5136333333333334, 0.5134666666666666, 0.62186],
                [0.6530333333333332, 0.5968, 0.62814],
                [0.48636666666666667, 0.6532, 0.8781399999999999],
                [0.34696666666666665, 0.5698666666666666, 0.8718600000000001],
                [0.33513333333333334, 0.5326333333333334, 0.68562],
                [0.4982, 0.6159666666666667, 0.56438],
                [0.6648666666666667, 0.6340333333333333, 0.81438],
                [0.5017999999999999, 0.5507, 0.93562],
                [0.3622, 0.6299333333333333, 0.69527],
                [0.47113333333333335, 0.5466, 0.55473],
                [0.6378, 0.5367333333333334, 0.80473],
                [0.5288666666666667, 0.6200666666666667, 0.94527],
                [0.4123666666666667, 0.5794666666666667, 0.836325],
                [0.4209666666666667, 0.6628, 0.913675],
                [0.5876333333333333, 0.5872, 0.663675],
                [0.5790333333333333, 0.5038666666666667, 0.586325],
                [0.4135333333333333, 0.5141666666666667, 0.84533],
                [0.4198, 0.5975, 0.90467],
                [0.5864666666666666, 0.6525, 0.65467],
                [0.5802, 0.5691666666666667, 0.59533],
                [0.5041333333333333, 0.7459166666666667, 0.126105],
                [0.6625333333333333, 0.82925, 0.123895],
                [0.4958666666666667, 0.7540833333333333, 0.373895],
                [0.33746666666666664, 0.67075, 0.376105],
                [0.5136333333333334, 0.6801333333333334, 0.12186],
                [0.6530333333333332, 0.7634666666666666, 0.12814],
                [0.48636666666666667, 0.8198666666666666, 0.37814],
                [0.34696666666666665, 0.7365333333333334, 0.37186],
                [0.33513333333333334, 0.6993, 0.18562],
                [0.4982, 0.7826333333333334, 0.06438],
                [0.6648666666666667, 0.8007, 0.31438],
                [0.5017999999999999, 0.7173666666666666, 0.43562],
                [0.3622, 0.7966000000000001, 0.19527],
                [0.47113333333333335, 0.7132666666666667, 0.05473],
                [0.6378, 0.7033999999999999, 0.30473],
                [0.5288666666666667, 0.7867333333333333, 0.44527],
                [0.4123666666666667, 0.7461333333333333, 0.336325],
                [0.4209666666666667, 0.8294666666666667, 0.413675],
                [0.5876333333333333, 0.7538666666666667, 0.163675],
                [0.5790333333333333, 0.6705333333333333, 0.086325],
                [0.4135333333333333, 0.6808333333333333, 0.34533],
                [0.4198, 0.7641666666666667, 0.40467],
                [0.5864666666666666, 0.8191666666666667, 0.15467],
                [0.5802, 0.7358333333333333, 0.09533],
                [0.5041333333333333, 0.7459166666666667, 0.626105],
                [0.6625333333333333, 0.82925, 0.623895],
                [0.4958666666666667, 0.7540833333333333, 0.873895],
                [0.33746666666666664, 0.67075, 0.876105],
                [0.5136333333333334, 0.6801333333333334, 0.62186],
                [0.6530333333333332, 0.7634666666666666, 0.62814],
                [0.48636666666666667, 0.8198666666666666, 0.8781399999999999],
                [0.34696666666666665, 0.7365333333333334, 0.8718600000000001],
                [0.33513333333333334, 0.6993, 0.68562],
                [0.4982, 0.7826333333333334, 0.56438],
                [0.6648666666666667, 0.8007, 0.81438],
                [0.5017999999999999, 0.7173666666666666, 0.93562],
                [0.3622, 0.7966000000000001, 0.69527],
                [0.47113333333333335, 0.7132666666666667, 0.55473],
                [0.6378, 0.7033999999999999, 0.80473],
                [0.5288666666666667, 0.7867333333333333, 0.94527],
                [0.4123666666666667, 0.7461333333333333, 0.836325],
                [0.4209666666666667, 0.8294666666666667, 0.913675],
                [0.5876333333333333, 0.7538666666666667, 0.663675],
                [0.5790333333333333, 0.6705333333333333, 0.586325],
                [0.4135333333333333, 0.6808333333333333, 0.84533],
                [0.4198, 0.7641666666666667, 0.90467],
                [0.5864666666666666, 0.8191666666666667, 0.65467],
                [0.5802, 0.7358333333333333, 0.59533],
                [0.5041333333333333, 0.9125833333333334, 0.126105],
                [0.6625333333333333, 0.9959166666666667, 0.123895],
                [0.4958666666666667, 0.92075, 0.373895],
                [0.33746666666666664, 0.8374166666666666, 0.376105],
                [0.5136333333333334, 0.8468, 0.12186],
                [0.6530333333333332, 0.9301333333333334, 0.12814],
                [0.48636666666666667, 0.9865333333333334, 0.37814],
                [0.34696666666666665, 0.9032, 0.37186],
                [0.33513333333333334, 0.8659666666666667, 0.18562],
                [0.4982, 0.9493, 0.06438],
                [0.6648666666666667, 0.9673666666666666, 0.31438],
                [0.5017999999999999, 0.8840333333333333, 0.43562],
                [0.3622, 0.9632666666666667, 0.19527],
                [0.47113333333333335, 0.8799333333333333, 0.05473],
                [0.6378, 0.8700666666666667, 0.30473],
                [0.5288666666666667, 0.9533999999999999, 0.44527],
                [0.4123666666666667, 0.9128, 0.336325],
                [0.4209666666666667, 0.9961333333333333, 0.413675],
                [0.5876333333333333, 0.9205333333333333, 0.163675],
                [0.5790333333333333, 0.8372, 0.086325],
                [0.4135333333333333, 0.8475, 0.34533],
                [0.4198, 0.9308333333333333, 0.40467],
                [0.5864666666666666, 0.9858333333333333, 0.15467],
                [0.5802, 0.9025, 0.09533],
                [0.5041333333333333, 0.9125833333333334, 0.626105],
                [0.6625333333333333, 0.9959166666666667, 0.623895],
                [0.4958666666666667, 0.92075, 0.873895],
                [0.33746666666666664, 0.8374166666666666, 0.876105],
                [0.5136333333333334, 0.8468, 0.62186],
                [0.6530333333333332, 0.9301333333333334, 0.62814],
                [0.48636666666666667, 0.9865333333333334, 0.8781399999999999],
                [0.34696666666666665, 0.9032, 0.8718600000000001],
                [0.33513333333333334, 0.8659666666666667, 0.68562],
                [0.4982, 0.9493, 0.56438],
                [0.6648666666666667, 0.9673666666666666, 0.81438],
                [0.5017999999999999, 0.8840333333333333, 0.93562],
                [0.3622, 0.9632666666666667, 0.69527],
                [0.47113333333333335, 0.8799333333333333, 0.55473],
                [0.6378, 0.8700666666666667, 0.80473],
                [0.5288666666666667, 0.9533999999999999, 0.94527],
                [0.4123666666666667, 0.9128, 0.836325],
                [0.4209666666666667, 0.9961333333333333, 0.913675],
                [0.5876333333333333, 0.9205333333333333, 0.663675],
                [0.5790333333333333, 0.8372, 0.586325],
                [0.4135333333333333, 0.8475, 0.84533],
                [0.4198, 0.9308333333333333, 0.90467],
                [0.5864666666666666, 0.9858333333333333, 0.65467],
                [0.5802, 0.9025, 0.59533],
                [0.8374666666666667, 0.07925, 0.126105],
                [0.9958666666666667, 0.16258333333333333, 0.123895],
                [0.8292, 0.08741666666666666, 0.373895],
                [0.6708, 0.004083333333333334, 0.376105],
                [0.8469666666666668, 0.013466666666666667, 0.12186],
                [0.9863666666666666, 0.0968, 0.12814],
                [0.8197, 0.1532, 0.37814],
                [0.6803, 0.06986666666666667, 0.37186],
                [0.6684666666666667, 0.032633333333333334, 0.18562],
                [0.8315333333333333, 0.11596666666666666, 0.06438],
                [0.9982000000000001, 0.13403333333333334, 0.31438],
                [0.8351333333333333, 0.0507, 0.43562],
                [0.6955333333333332, 0.12993333333333332, 0.19527],
                [0.8044666666666668, 0.0466, 0.05473],
                [0.9711333333333334, 0.03673333333333333, 0.30473],
                [0.8622, 0.12006666666666667, 0.44527],
                [0.7456999999999999, 0.07946666666666667, 0.336325],
                [0.7543000000000001, 0.1628, 0.413675],
                [0.9209666666666667, 0.0872, 0.163675],
                [0.9123666666666667, 0.0038666666666666663, 0.086325],
                [0.7468666666666667, 0.014166666666666668, 0.34533],
                [0.7531333333333333, 0.09749999999999999, 0.40467],
                [0.9198, 0.1525, 0.15467],
                [0.9135333333333334, 0.06916666666666667, 0.09533],
                [0.8374666666666667, 0.07925, 0.626105],
                [0.9958666666666667, 0.16258333333333333, 0.623895],
                [0.8292, 0.08741666666666666, 0.873895],
                [0.6708, 0.004083333333333334, 0.876105],
                [0.8469666666666668, 0.013466666666666667, 0.62186],
                [0.9863666666666666, 0.0968, 0.62814],
                [0.8197, 0.1532, 0.8781399999999999],
                [0.6803, 0.06986666666666667, 0.8718600000000001],
                [0.6684666666666667, 0.032633333333333334, 0.68562],
                [0.8315333333333333, 0.11596666666666666, 0.56438],
                [0.9982000000000001, 0.13403333333333334, 0.81438],
                [0.8351333333333333, 0.0507, 0.93562],
                [0.6955333333333332, 0.12993333333333332, 0.69527],
                [0.8044666666666668, 0.0466, 0.55473],
                [0.9711333333333334, 0.03673333333333333, 0.80473],
                [0.8622, 0.12006666666666667, 0.94527],
                [0.7456999999999999, 0.07946666666666667, 0.836325],
                [0.7543000000000001, 0.1628, 0.913675],
                [0.9209666666666667, 0.0872, 0.663675],
                [0.9123666666666667, 0.0038666666666666663, 0.586325],
                [0.7468666666666667, 0.014166666666666668, 0.84533],
                [0.7531333333333333, 0.09749999999999999, 0.90467],
                [0.9198, 0.1525, 0.65467],
                [0.9135333333333334, 0.06916666666666667, 0.59533],
                [0.8374666666666667, 0.24591666666666667, 0.126105],
                [0.9958666666666667, 0.32925, 0.123895],
                [0.8292, 0.2540833333333333, 0.373895],
                [0.6708, 0.17074999999999999, 0.376105],
                [0.8469666666666668, 0.18013333333333334, 0.12186],
                [0.9863666666666666, 0.2634666666666667, 0.12814],
                [0.8197, 0.3198666666666667, 0.37814],
                [0.6803, 0.23653333333333335, 0.37186],
                [0.6684666666666667, 0.1993, 0.18562],
                [0.8315333333333333, 0.28263333333333335, 0.06438],
                [0.9982000000000001, 0.3007, 0.31438],
                [0.8351333333333333, 0.21736666666666668, 0.43562],
                [0.6955333333333332, 0.2966, 0.19527],
                [0.8044666666666668, 0.2132666666666667, 0.05473],
                [0.9711333333333334, 0.2034, 0.30473],
                [0.8622, 0.28673333333333334, 0.44527],
                [0.7456999999999999, 0.24613333333333332, 0.336325],
                [0.7543000000000001, 0.32946666666666663, 0.413675],
                [0.9209666666666667, 0.2538666666666667, 0.163675],
                [0.9123666666666667, 0.17053333333333334, 0.086325],
                [0.7468666666666667, 0.18083333333333332, 0.34533],
                [0.7531333333333333, 0.26416666666666666, 0.40467],
                [0.9198, 0.31916666666666665, 0.15467],
                [0.9135333333333334, 0.23583333333333334, 0.09533],
                [0.8374666666666667, 0.24591666666666667, 0.626105],
                [0.9958666666666667, 0.32925, 0.623895],
                [0.8292, 0.2540833333333333, 0.873895],
                [0.6708, 0.17074999999999999, 0.876105],
                [0.8469666666666668, 0.18013333333333334, 0.62186],
                [0.9863666666666666, 0.2634666666666667, 0.62814],
                [0.8197, 0.3198666666666667, 0.8781399999999999],
                [0.6803, 0.23653333333333335, 0.8718600000000001],
                [0.6684666666666667, 0.1993, 0.68562],
                [0.8315333333333333, 0.28263333333333335, 0.56438],
                [0.9982000000000001, 0.3007, 0.81438],
                [0.8351333333333333, 0.21736666666666668, 0.93562],
                [0.6955333333333332, 0.2966, 0.69527],
                [0.8044666666666668, 0.2132666666666667, 0.55473],
                [0.9711333333333334, 0.2034, 0.80473],
                [0.8622, 0.28673333333333334, 0.94527],
                [0.7456999999999999, 0.24613333333333332, 0.836325],
                [0.7543000000000001, 0.32946666666666663, 0.913675],
                [0.9209666666666667, 0.2538666666666667, 0.663675],
                [0.9123666666666667, 0.17053333333333334, 0.586325],
                [0.7468666666666667, 0.18083333333333332, 0.84533],
                [0.7531333333333333, 0.26416666666666666, 0.90467],
                [0.9198, 0.31916666666666665, 0.65467],
                [0.9135333333333334, 0.23583333333333334, 0.59533],
                [0.8374666666666667, 0.4125833333333333, 0.126105],
                [0.9958666666666667, 0.49591666666666673, 0.123895],
                [0.8292, 0.42074999999999996, 0.373895],
                [0.6708, 0.3374166666666667, 0.376105],
                [0.8469666666666668, 0.3468, 0.12186],
                [0.9863666666666666, 0.4301333333333333, 0.12814],
                [0.8197, 0.4865333333333333, 0.37814],
                [0.6803, 0.4032, 0.37186],
                [0.6684666666666667, 0.3659666666666667, 0.18562],
                [0.8315333333333333, 0.44930000000000003, 0.06438],
                [0.9982000000000001, 0.46736666666666665, 0.31438],
                [0.8351333333333333, 0.3840333333333333, 0.43562],
                [0.6955333333333332, 0.46326666666666666, 0.19527],
                [0.8044666666666668, 0.3799333333333333, 0.05473],
                [0.9711333333333334, 0.3700666666666667, 0.30473],
                [0.8622, 0.4534, 0.44527],
                [0.7456999999999999, 0.4128, 0.336325],
                [0.7543000000000001, 0.4961333333333333, 0.413675],
                [0.9209666666666667, 0.42053333333333337, 0.163675],
                [0.9123666666666667, 0.3372, 0.086325],
                [0.7468666666666667, 0.3475, 0.34533],
                [0.7531333333333333, 0.43083333333333335, 0.40467],
                [0.9198, 0.48583333333333334, 0.15467],
                [0.9135333333333334, 0.4025, 0.09533],
                [0.8374666666666667, 0.4125833333333333, 0.626105],
                [0.9958666666666667, 0.49591666666666673, 0.623895],
                [0.8292, 0.42074999999999996, 0.873895],
                [0.6708, 0.3374166666666667, 0.876105],
                [0.8469666666666668, 0.3468, 0.62186],
                [0.9863666666666666, 0.4301333333333333, 0.62814],
                [0.8197, 0.4865333333333333, 0.8781399999999999],
                [0.6803, 0.4032, 0.8718600000000001],
                [0.6684666666666667, 0.3659666666666667, 0.68562],
                [0.8315333333333333, 0.44930000000000003, 0.56438],
                [0.9982000000000001, 0.46736666666666665, 0.81438],
                [0.8351333333333333, 0.3840333333333333, 0.93562],
                [0.6955333333333332, 0.46326666666666666, 0.69527],
                [0.8044666666666668, 0.3799333333333333, 0.55473],
                [0.9711333333333334, 0.3700666666666667, 0.80473],
                [0.8622, 0.4534, 0.94527],
                [0.7456999999999999, 0.4128, 0.836325],
                [0.7543000000000001, 0.4961333333333333, 0.913675],
                [0.9209666666666667, 0.42053333333333337, 0.663675],
                [0.9123666666666667, 0.3372, 0.586325],
                [0.7468666666666667, 0.3475, 0.84533],
                [0.7531333333333333, 0.43083333333333335, 0.90467],
                [0.9198, 0.48583333333333334, 0.65467],
                [0.9135333333333334, 0.4025, 0.59533],
                [0.8374666666666667, 0.5792499999999999, 0.126105],
                [0.9958666666666667, 0.6625833333333334, 0.123895],
                [0.8292, 0.5874166666666666, 0.373895],
                [0.6708, 0.5040833333333333, 0.376105],
                [0.8469666666666668, 0.5134666666666666, 0.12186],
                [0.9863666666666666, 0.5968, 0.12814],
                [0.8197, 0.6532, 0.37814],
                [0.6803, 0.5698666666666666, 0.37186],
                [0.6684666666666667, 0.5326333333333334, 0.18562],
                [0.8315333333333333, 0.6159666666666667, 0.06438],
                [0.9982000000000001, 0.6340333333333333, 0.31438],
                [0.8351333333333333, 0.5507, 0.43562],
                [0.6955333333333332, 0.6299333333333333, 0.19527],
                [0.8044666666666668, 0.5466, 0.05473],
                [0.9711333333333334, 0.5367333333333334, 0.30473],
                [0.8622, 0.6200666666666667, 0.44527],
                [0.7456999999999999, 0.5794666666666667, 0.336325],
                [0.7543000000000001, 0.6628, 0.413675],
                [0.9209666666666667, 0.5872, 0.163675],
                [0.9123666666666667, 0.5038666666666667, 0.086325],
                [0.7468666666666667, 0.5141666666666667, 0.34533],
                [0.7531333333333333, 0.5975, 0.40467],
                [0.9198, 0.6525, 0.15467],
                [0.9135333333333334, 0.5691666666666667, 0.09533],
                [0.8374666666666667, 0.5792499999999999, 0.626105],
                [0.9958666666666667, 0.6625833333333334, 0.623895],
                [0.8292, 0.5874166666666666, 0.873895],
                [0.6708, 0.5040833333333333, 0.876105],
                [0.8469666666666668, 0.5134666666666666, 0.62186],
                [0.9863666666666666, 0.5968, 0.62814],
                [0.8197, 0.6532, 0.8781399999999999],
                [0.6803, 0.5698666666666666, 0.8718600000000001],
                [0.6684666666666667, 0.5326333333333334, 0.68562],
                [0.8315333333333333, 0.6159666666666667, 0.56438],
                [0.9982000000000001, 0.6340333333333333, 0.81438],
                [0.8351333333333333, 0.5507, 0.93562],
                [0.6955333333333332, 0.6299333333333333, 0.69527],
                [0.8044666666666668, 0.5466, 0.55473],
                [0.9711333333333334, 0.5367333333333334, 0.80473],
                [0.8622, 0.6200666666666667, 0.94527],
                [0.7456999999999999, 0.5794666666666667, 0.836325],
                [0.7543000000000001, 0.6628, 0.913675],
                [0.9209666666666667, 0.5872, 0.663675],
                [0.9123666666666667, 0.5038666666666667, 0.586325],
                [0.7468666666666667, 0.5141666666666667, 0.84533],
                [0.7531333333333333, 0.5975, 0.90467],
                [0.9198, 0.6525, 0.65467],
                [0.9135333333333334, 0.5691666666666667, 0.59533],
                [0.8374666666666667, 0.7459166666666667, 0.126105],
                [0.9958666666666667, 0.82925, 0.123895],
                [0.8292, 0.7540833333333333, 0.373895],
                [0.6708, 0.67075, 0.376105],
                [0.8469666666666668, 0.6801333333333334, 0.12186],
                [0.9863666666666666, 0.7634666666666666, 0.12814],
                [0.8197, 0.8198666666666666, 0.37814],
                [0.6803, 0.7365333333333334, 0.37186],
                [0.6684666666666667, 0.6993, 0.18562],
                [0.8315333333333333, 0.7826333333333334, 0.06438],
                [0.9982000000000001, 0.8007, 0.31438],
                [0.8351333333333333, 0.7173666666666666, 0.43562],
                [0.6955333333333332, 0.7966000000000001, 0.19527],
                [0.8044666666666668, 0.7132666666666667, 0.05473],
                [0.9711333333333334, 0.7033999999999999, 0.30473],
                [0.8622, 0.7867333333333333, 0.44527],
                [0.7456999999999999, 0.7461333333333333, 0.336325],
                [0.7543000000000001, 0.8294666666666667, 0.413675],
                [0.9209666666666667, 0.7538666666666667, 0.163675],
                [0.9123666666666667, 0.6705333333333333, 0.086325],
                [0.7468666666666667, 0.6808333333333333, 0.34533],
                [0.7531333333333333, 0.7641666666666667, 0.40467],
                [0.9198, 0.8191666666666667, 0.15467],
                [0.9135333333333334, 0.7358333333333333, 0.09533],
                [0.8374666666666667, 0.7459166666666667, 0.626105],
                [0.9958666666666667, 0.82925, 0.623895],
                [0.8292, 0.7540833333333333, 0.873895],
                [0.6708, 0.67075, 0.876105],
                [0.8469666666666668, 0.6801333333333334, 0.62186],
                [0.9863666666666666, 0.7634666666666666, 0.62814],
                [0.8197, 0.8198666666666666, 0.8781399999999999],
                [0.6803, 0.7365333333333334, 0.8718600000000001],
                [0.6684666666666667, 0.6993, 0.68562],
                [0.8315333333333333, 0.7826333333333334, 0.56438],
                [0.9982000000000001, 0.8007, 0.81438],
                [0.8351333333333333, 0.7173666666666666, 0.93562],
                [0.6955333333333332, 0.7966000000000001, 0.69527],
                [0.8044666666666668, 0.7132666666666667, 0.55473],
                [0.9711333333333334, 0.7033999999999999, 0.80473],
                [0.8622, 0.7867333333333333, 0.94527],
                [0.7456999999999999, 0.7461333333333333, 0.836325],
                [0.7543000000000001, 0.8294666666666667, 0.913675],
                [0.9209666666666667, 0.7538666666666667, 0.663675],
                [0.9123666666666667, 0.6705333333333333, 0.586325],
                [0.7468666666666667, 0.6808333333333333, 0.84533],
                [0.7531333333333333, 0.7641666666666667, 0.90467],
                [0.9198, 0.8191666666666667, 0.65467],
                [0.9135333333333334, 0.7358333333333333, 0.59533],
                [0.8374666666666667, 0.9125833333333334, 0.126105],
                [0.9958666666666667, 0.9959166666666667, 0.123895],
                [0.8292, 0.92075, 0.373895],
                [0.6708, 0.8374166666666666, 0.376105],
                [0.8469666666666668, 0.8468, 0.12186],
                [0.9863666666666666, 0.9301333333333334, 0.12814],
                [0.8197, 0.9865333333333334, 0.37814],
                [0.6803, 0.9032, 0.37186],
                [0.6684666666666667, 0.8659666666666667, 0.18562],
                [0.8315333333333333, 0.9493, 0.06438],
                [0.9982000000000001, 0.9673666666666666, 0.31438],
                [0.8351333333333333, 0.8840333333333333, 0.43562],
                [0.6955333333333332, 0.9632666666666667, 0.19527],
                [0.8044666666666668, 0.8799333333333333, 0.05473],
                [0.9711333333333334, 0.8700666666666667, 0.30473],
                [0.8622, 0.9533999999999999, 0.44527],
                [0.7456999999999999, 0.9128, 0.336325],
                [0.7543000000000001, 0.9961333333333333, 0.413675],
                [0.9209666666666667, 0.9205333333333333, 0.163675],
                [0.9123666666666667, 0.8372, 0.086325],
                [0.7468666666666667, 0.8475, 0.34533],
                [0.7531333333333333, 0.9308333333333333, 0.40467],
                [0.9198, 0.9858333333333333, 0.15467],
                [0.9135333333333334, 0.9025, 0.09533],
                [0.8374666666666667, 0.9125833333333334, 0.626105],
                [0.9958666666666667, 0.9959166666666667, 0.623895],
                [0.8292, 0.92075, 0.873895],
                [0.6708, 0.8374166666666666, 0.876105],
                [0.8469666666666668, 0.8468, 0.62186],
                [0.9863666666666666, 0.9301333333333334, 0.62814],
                [0.8197, 0.9865333333333334, 0.8781399999999999],
                [0.6803, 0.9032, 0.8718600000000001],
                [0.6684666666666667, 0.8659666666666667, 0.68562],
                [0.8315333333333333, 0.9493, 0.56438],
                [0.9982000000000001, 0.9673666666666666, 0.81438],
                [0.8351333333333333, 0.8840333333333333, 0.93562],
                [0.6955333333333332, 0.9632666666666667, 0.69527],
                [0.8044666666666668, 0.8799333333333333, 0.55473],
                [0.9711333333333334, 0.8700666666666667, 0.80473],
                [0.8622, 0.9533999999999999, 0.94527],
                [0.7456999999999999, 0.9128, 0.836325],
                [0.7543000000000001, 0.9961333333333333, 0.913675],
                [0.9209666666666667, 0.9205333333333333, 0.663675],
                [0.9123666666666667, 0.8372, 0.586325],
                [0.7468666666666667, 0.8475, 0.84533],
                [0.7531333333333333, 0.9308333333333333, 0.90467],
                [0.9198, 0.9858333333333333, 0.65467],
                [0.9135333333333334, 0.9025, 0.59533],
                [0.03441, 0.006066666666666667, 0.19707],
                [0.13225666666666666, 0.0894, 0.05293],
                [0.2989233333333333, 0.1606, 0.30293],
                [0.20107666666666668, 0.07726666666666666, 0.44707],
                [0.03441, 0.006066666666666667, 0.69707],
                [0.13225666666666666, 0.0894, 0.55293],
                [0.2989233333333333, 0.1606, 0.8029299999999999],
                [0.20107666666666668, 0.07726666666666666, 0.9470700000000001],
                [0.03441, 0.17273333333333332, 0.19707],
                [0.13225666666666666, 0.25606666666666666, 0.05293],
                [0.2989233333333333, 0.32726666666666665, 0.30293],
                [0.20107666666666668, 0.24393333333333334, 0.44707],
                [0.03441, 0.17273333333333332, 0.69707],
                [0.13225666666666666, 0.25606666666666666, 0.55293],
                [0.2989233333333333, 0.32726666666666665, 0.8029299999999999],
                [0.20107666666666668, 0.24393333333333334, 0.9470700000000001],
                [0.03441, 0.3394, 0.19707],
                [0.13225666666666666, 0.42273333333333335, 0.05293],
                [0.2989233333333333, 0.49393333333333334, 0.30293],
                [0.20107666666666668, 0.4106, 0.44707],
                [0.03441, 0.3394, 0.69707],
                [0.13225666666666666, 0.42273333333333335, 0.55293],
                [0.2989233333333333, 0.49393333333333334, 0.8029299999999999],
                [0.20107666666666668, 0.4106, 0.9470700000000001],
                [0.03441, 0.5060666666666667, 0.19707],
                [0.13225666666666666, 0.5894, 0.05293],
                [0.2989233333333333, 0.6606, 0.30293],
                [0.20107666666666668, 0.5772666666666667, 0.44707],
                [0.03441, 0.5060666666666667, 0.69707],
                [0.13225666666666666, 0.5894, 0.55293],
                [0.2989233333333333, 0.6606, 0.8029299999999999],
                [0.20107666666666668, 0.5772666666666667, 0.9470700000000001],
                [0.03441, 0.6727333333333334, 0.19707],
                [0.13225666666666666, 0.7560666666666668, 0.05293],
                [0.2989233333333333, 0.8272666666666666, 0.30293],
                [0.20107666666666668, 0.7439333333333332, 0.44707],
                [0.03441, 0.6727333333333334, 0.69707],
                [0.13225666666666666, 0.7560666666666668, 0.55293],
                [0.2989233333333333, 0.8272666666666666, 0.8029299999999999],
                [0.20107666666666668, 0.7439333333333332, 0.9470700000000001],
                [0.03441, 0.8394, 0.19707],
                [0.13225666666666666, 0.9227333333333334, 0.05293],
                [0.2989233333333333, 0.9939333333333332, 0.30293],
                [0.20107666666666668, 0.9106, 0.44707],
                [0.03441, 0.8394, 0.69707],
                [0.13225666666666666, 0.9227333333333334, 0.55293],
                [0.2989233333333333, 0.9939333333333332, 0.8029299999999999],
                [0.20107666666666668, 0.9106, 0.9470700000000001],
                [0.3677433333333333, 0.006066666666666667, 0.19707],
                [0.46559, 0.0894, 0.05293],
                [0.6322566666666667, 0.1606, 0.30293],
                [0.5344099999999999, 0.07726666666666666, 0.44707],
                [0.3677433333333333, 0.006066666666666667, 0.69707],
                [0.46559, 0.0894, 0.55293],
                [0.6322566666666667, 0.1606, 0.8029299999999999],
                [0.5344099999999999, 0.07726666666666666, 0.9470700000000001],
                [0.3677433333333333, 0.17273333333333332, 0.19707],
                [0.46559, 0.25606666666666666, 0.05293],
                [0.6322566666666667, 0.32726666666666665, 0.30293],
                [0.5344099999999999, 0.24393333333333334, 0.44707],
                [0.3677433333333333, 0.17273333333333332, 0.69707],
                [0.46559, 0.25606666666666666, 0.55293],
                [0.6322566666666667, 0.32726666666666665, 0.8029299999999999],
                [0.5344099999999999, 0.24393333333333334, 0.9470700000000001],
                [0.3677433333333333, 0.3394, 0.19707],
                [0.46559, 0.42273333333333335, 0.05293],
                [0.6322566666666667, 0.49393333333333334, 0.30293],
                [0.5344099999999999, 0.4106, 0.44707],
                [0.3677433333333333, 0.3394, 0.69707],
                [0.46559, 0.42273333333333335, 0.55293],
                [0.6322566666666667, 0.49393333333333334, 0.8029299999999999],
                [0.5344099999999999, 0.4106, 0.9470700000000001],
                [0.3677433333333333, 0.5060666666666667, 0.19707],
                [0.46559, 0.5894, 0.05293],
                [0.6322566666666667, 0.6606, 0.30293],
                [0.5344099999999999, 0.5772666666666667, 0.44707],
                [0.3677433333333333, 0.5060666666666667, 0.69707],
                [0.46559, 0.5894, 0.55293],
                [0.6322566666666667, 0.6606, 0.8029299999999999],
                [0.5344099999999999, 0.5772666666666667, 0.9470700000000001],
                [0.3677433333333333, 0.6727333333333334, 0.19707],
                [0.46559, 0.7560666666666668, 0.05293],
                [0.6322566666666667, 0.8272666666666666, 0.30293],
                [0.5344099999999999, 0.7439333333333332, 0.44707],
                [0.3677433333333333, 0.6727333333333334, 0.69707],
                [0.46559, 0.7560666666666668, 0.55293],
                [0.6322566666666667, 0.8272666666666666, 0.8029299999999999],
                [0.5344099999999999, 0.7439333333333332, 0.9470700000000001],
                [0.3677433333333333, 0.8394, 0.19707],
                [0.46559, 0.9227333333333334, 0.05293],
                [0.6322566666666667, 0.9939333333333332, 0.30293],
                [0.5344099999999999, 0.9106, 0.44707],
                [0.3677433333333333, 0.8394, 0.69707],
                [0.46559, 0.9227333333333334, 0.55293],
                [0.6322566666666667, 0.9939333333333332, 0.8029299999999999],
                [0.5344099999999999, 0.9106, 0.9470700000000001],
                [0.7010766666666667, 0.006066666666666667, 0.19707],
                [0.7989233333333333, 0.0894, 0.05293],
                [0.9655900000000001, 0.1606, 0.30293],
                [0.8677433333333333, 0.07726666666666666, 0.44707],
                [0.7010766666666667, 0.006066666666666667, 0.69707],
                [0.7989233333333333, 0.0894, 0.55293],
                [0.9655900000000001, 0.1606, 0.8029299999999999],
                [0.8677433333333333, 0.07726666666666666, 0.9470700000000001],
                [0.7010766666666667, 0.17273333333333332, 0.19707],
                [0.7989233333333333, 0.25606666666666666, 0.05293],
                [0.9655900000000001, 0.32726666666666665, 0.30293],
                [0.8677433333333333, 0.24393333333333334, 0.44707],
                [0.7010766666666667, 0.17273333333333332, 0.69707],
                [0.7989233333333333, 0.25606666666666666, 0.55293],
                [0.9655900000000001, 0.32726666666666665, 0.8029299999999999],
                [0.8677433333333333, 0.24393333333333334, 0.9470700000000001],
                [0.7010766666666667, 0.3394, 0.19707],
                [0.7989233333333333, 0.42273333333333335, 0.05293],
                [0.9655900000000001, 0.49393333333333334, 0.30293],
                [0.8677433333333333, 0.4106, 0.44707],
                [0.7010766666666667, 0.3394, 0.69707],
                [0.7989233333333333, 0.42273333333333335, 0.55293],
                [0.9655900000000001, 0.49393333333333334, 0.8029299999999999],
                [0.8677433333333333, 0.4106, 0.9470700000000001],
                [0.7010766666666667, 0.5060666666666667, 0.19707],
                [0.7989233333333333, 0.5894, 0.05293],
                [0.9655900000000001, 0.6606, 0.30293],
                [0.8677433333333333, 0.5772666666666667, 0.44707],
                [0.7010766666666667, 0.5060666666666667, 0.69707],
                [0.7989233333333333, 0.5894, 0.55293],
                [0.9655900000000001, 0.6606, 0.8029299999999999],
                [0.8677433333333333, 0.5772666666666667, 0.9470700000000001],
                [0.7010766666666667, 0.6727333333333334, 0.19707],
                [0.7989233333333333, 0.7560666666666668, 0.05293],
                [0.9655900000000001, 0.8272666666666666, 0.30293],
                [0.8677433333333333, 0.7439333333333332, 0.44707],
                [0.7010766666666667, 0.6727333333333334, 0.69707],
                [0.7989233333333333, 0.7560666666666668, 0.55293],
                [0.9655900000000001, 0.8272666666666666, 0.8029299999999999],
                [0.8677433333333333, 0.7439333333333332, 0.9470700000000001],
                [0.7010766666666667, 0.8394, 0.19707],
                [0.7989233333333333, 0.9227333333333334, 0.05293],
                [0.9655900000000001, 0.9939333333333332, 0.30293],
                [0.8677433333333333, 0.9106, 0.44707],
                [0.7010766666666667, 0.8394, 0.69707],
                [0.7989233333333333, 0.9227333333333334, 0.55293],
                [0.9655900000000001, 0.9939333333333332, 0.8029299999999999],
                [0.8677433333333333, 0.9106, 0.9470700000000001],
            ],
        ),
    )
end
